@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015, 2016 Laboratoire de Recherche et Développement de
-// l'Epita (LRDE).
+// Copyright (C) 2015, 2016, 2017 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -33,44 +33,59 @@
 #include <spot/tl/unabbrev.hh>
 #include "common_conv.hh"
 #include <spot/misc/escape.hh>
+#include <spot/twaalgos/hoa.hh>
+#include <spot/twaalgos/lbtt.hh>
+#include <spot/twaalgos/neverclaim.hh>
 
 // A set of tools for which we know the correct output
-static struct shorthands_t
+struct shorthands_t
 {
   const char* prefix;
   const char* suffix;
-}
-  shorthands[] = {
+};
+static shorthands_t shorthands_ltl[] = {
     { "lbt", " <%L>%O" },
     { "ltl2ba", " -f %s>%O" },
+    { "ltl2da", " %f>%O" },
+    { "ltl2dpa", " %f>%O" },
+    { "ltl2ldba", " %f>%O" },
     { "ltl2dstar", " --output-format=hoa %[MW]L %O"},
     { "ltl2tgba", " -H %f>%O" },
     { "ltl3ba", " -f %s>%O" },
     { "ltl3dra", " -f %s>%O" },
+    { "ltl3hoa", " -f %f>%O" },
+    // ltl3tela is the new name of ltl3hoa
+    { "ltl3tela", " -f %f>%O" },
     { "modella", " %[MWei^]L %O" },
     { "spin", " -f %s>%O" },
   };
 
-static void show_shorthands()
+static shorthands_t shorthands_autproc[] = {
+    { "autfilt", " %H>%O" },
+    { "dstar2tgba", " %H>%O" },
+    { "ltl2dstar", " -B %H %O" },
+    { "nba2ldpa", " <%H>%O" },
+    { "seminator", " %H>%O" },
+  };
+
+static void show_shorthands(shorthands_t* begin, shorthands_t* end)
 {
   std::cout
     << ("If a COMMANDFMT does not use any %-sequence, and starts with one of\n"
         "the following words, then the string on the right is appended.\n\n");
-  for (auto& s: shorthands)
-    std::cout << "  "
-              << std::left << std::setw(12) << s.prefix
-              << s.suffix << '\n';
-  std::cout
-    << ("\nAny {name} and directory component is skipped for the purpose of\n"
-        "matching those prefixes.  So for instance\n"
-        "  '{DRA} ~/mytools/ltl2dstar-0.5.2'\n"
-        "will changed into\n"
-        "  '{DRA} ~/mytools/ltl2dstar-0.5.2 --output-format=hoa %[MR]L %O'\n");
+  while (begin != end)
+    {
+      std::cout << "  "
+                << std::left << std::setw(12) << begin->prefix
+                << begin->suffix << '\n';
+      ++begin;
+    }
 }
 
 
-translator_spec::translator_spec(const char* spec)
-  : spec(spec), cmd(spec), name(spec)
+tool_spec::tool_spec(const char* spec, shorthands_t* begin, shorthands_t* end,
+                     bool is_ref)
+  : spec(spec), cmd(spec), name(spec), reference(is_ref)
 {
   if (*cmd == '{')
     {
@@ -110,8 +125,9 @@ translator_spec::translator_spec(const char* spec)
           ++pos;
         }
       // Match a shorthand.
-      for (auto& p: shorthands)
+      while (begin != end)
         {
+          auto& p = *begin++;
           int n = strlen(p.prefix);
           if (strncmp(basename, p.prefix, n) == 0)
             {
@@ -130,16 +146,29 @@ translator_spec::translator_spec(const char* spec)
     cmd = strdup(cmd);
 }
 
-translator_spec::translator_spec(const translator_spec& other)
-  : spec(other.spec), cmd(other.cmd), name(other.name)
+tool_spec::tool_spec(const tool_spec& other)
+  : spec(other.spec), cmd(other.cmd), name(other.name),
+    reference(other.reference)
 {
-  if (name != spec)
-    name = strdup(name);
   if (cmd != spec)
     cmd = strdup(cmd);
+  if (name != spec)
+    name = strdup(name);
 }
 
-translator_spec::~translator_spec()
+tool_spec& tool_spec::operator=(const tool_spec& other)
+{
+  spec = other.spec;
+  cmd = other.cmd;
+  if (cmd != spec)
+    cmd = strdup(cmd);
+  name = other.name;
+  if (name != spec)
+    name = strdup(name);
+  return *this;
+}
+
+tool_spec::~tool_spec()
 {
   if (name != spec)
     free(const_cast<char*>(name));
@@ -147,7 +176,23 @@ translator_spec::~translator_spec()
     free(const_cast<char*>(cmd));
 }
 
-std::vector<translator_spec> translators;
+std::vector<tool_spec> tools;
+
+void tools_push_trans(const char* trans, bool is_ref)
+{
+  tools.emplace_back(trans,
+                     std::begin(shorthands_ltl),
+                     std::end(shorthands_ltl),
+                     is_ref);
+}
+
+void tools_push_autproc(const char* proc, bool is_ref)
+{
+  tools.emplace_back(proc,
+                     std::begin(shorthands_autproc),
+                     std::end(shorthands_autproc),
+                     is_ref);
+}
 
 void
 quoted_string::print(std::ostream& os, const char*) const
@@ -238,16 +283,8 @@ printable_result_filename::print(std::ostream& os, const char*) const
   spot::quote_shell_string(os, val()->name());
 }
 
-void
-filed_formula::print(std::ostream& os, const char* pos) const
-{
-  std::ostringstream ss;
-  f_.print(ss, pos);
-  os << '\'' << string_to_tmp(ss.str(), serial_) << '\'';
-}
-
-std::string
-filed_formula::string_to_tmp(const std::string str, unsigned n) const
+static std::string
+string_to_tmp(const std::string str, unsigned n)
 {
   char prefix[30];
   snprintf(prefix, sizeof prefix, "lcr-i%u-", n);
@@ -262,6 +299,43 @@ filed_formula::string_to_tmp(const std::string str, unsigned n) const
   return tmpname;
 }
 
+void
+filed_formula::print(std::ostream& os, const char* pos) const
+{
+  std::ostringstream ss;
+  f_.print(ss, pos);
+  os << '\'' << string_to_tmp(ss.str(), serial_) << '\'';
+}
+
+void
+filed_automaton::print(std::ostream& os, const char* pos) const
+{
+  std::ostringstream ss;
+  char* opt = nullptr;
+  if (*pos == '[')
+    {
+      ++pos;
+      auto end = strchr(pos, ']');
+      opt = strndup(pos, end - pos);
+      pos = end + 1;
+    }
+  switch (*pos)
+    {
+    case 'H':
+      spot::print_hoa(ss, aut_, opt);
+      break;
+    case 'S':
+      spot::print_never_claim(ss, aut_, opt);
+      break;
+    case 'L':
+      spot::print_lbtt(ss, aut_, opt);
+      break;
+    }
+  if (opt)
+    free(opt);
+  os << '\'' << string_to_tmp(ss.str(), serial_) << '\'';
+}
+
 translator_runner::translator_runner(spot::bdd_dict_ptr dict,
                                      bool no_output_allowed)
   : dict(dict)
@@ -280,14 +354,14 @@ translator_runner::translator_runner(spot::bdd_dict_ptr dict,
   declare('T', &output);
   declare('O', &output);
 
-  size_t s = translators.size();
+  size_t s = tools.size();
   assert(s);
   for (size_t n = 0; n < s; ++n)
     {
       // Check that each translator uses at least one input and
       // one output.
       std::vector<bool> has(256);
-      const translator_spec& t = translators[n];
+      const tool_spec& t = tools[n];
       scan(t.cmd, has);
       if (!(has['f'] || has['s'] || has['l'] || has['w']
             || has['F'] || has['S'] || has['L'] || has['W']))
@@ -301,7 +375,7 @@ translator_runner::translator_runner(spot::bdd_dict_ptr dict,
         error(2, 0, "no output %%-sequence in '%s'.\n      Use  "
               "%%O to indicate where the automaton is output.",
               t.spec);
-      // Remember the %-sequences used by all translators.
+      // Remember the %-sequences used by all tools.
       prime(t.cmd);
     }
 }
@@ -329,6 +403,42 @@ translator_runner::round_formula(spot::formula f, unsigned serial)
   filename_formula.new_round(serial);
 }
 
+
+autproc_runner::autproc_runner(bool no_output_allowed)
+{
+  declare('H', &filename_automaton);
+  declare('S', &filename_automaton);
+  declare('L', &filename_automaton);
+  declare('O', &output);
+
+  size_t s = tools.size();
+  assert(s);
+  for (size_t n = 0; n < s; ++n)
+    {
+      // Check that each translator uses at least one input and
+      // one output.
+      std::vector<bool> has(256);
+      const tool_spec& t = tools[n];
+      scan(t.cmd, has);
+      if (!(has['H'] || has['S'] || has['L']))
+        error(2, 0, "no input %%-sequence in '%s'.\n       Use "
+              "one of %%H,%%S,%%L to indicate the input automaton filename.",
+              t.spec);
+      if (!no_output_allowed && !has['O'])
+        error(2, 0, "no output %%-sequence in '%s'.\n      Use  "
+              "%%O to indicate where the automaton is output.",
+              t.spec);
+      // Remember the %-sequences used by all tools.
+      prime(t.cmd);
+    }
+}
+
+void
+autproc_runner::round_automaton(spot::const_twa_graph_ptr aut, unsigned serial)
+{
+  filename_automaton.new_round(aut, serial);
+}
+
 volatile bool timed_out = false;
 unsigned timeout_count = 0;
 
@@ -480,7 +590,8 @@ skip_ws(const char*& cmd)
 //   - can have >stderr and <stdin redirection
 // In particular, variable interpolation is not supported.  Complex
 // redirections (>& and such) are not support.  Chains of commands
-// (pipes, semi-colons, etc.) are not supported.
+// (pipes, semi-colons, etc.) are not supported.  Envvar definitions
+// before the command are not supported.
 static void
 exec_command(const char* cmd)
 {
@@ -531,13 +642,20 @@ exec_command(const char* cmd)
           }
         }
     }
+  // If result is empty, we failed to found the command; let's see if
+  // the shell is smarter.  If the command contains '=', it's unlikely
+  // to be a command, but probably an environment variable defintion
+  // as in
+  //    FOO=1 command args..
+  if (result.empty() || strchr(result[0], '='))
+    goto use_shell;
   {
     if (stdin)
       {
         int fd0 = open(stdin, O_RDONLY, 0644);
         if (fd0 < 0)
           error(2, errno, "failed to open '%s'", stdin);
-        if (dup2(fd0, 0) < 0)
+        if (dup2(fd0, STDIN_FILENO) < 0)
           error(2, errno, "dup2() failed");
         if (close(fd0) < 0)
           error(2, errno, "close() failed");
@@ -547,7 +665,7 @@ exec_command(const char* cmd)
         int fd1 = creat(stdout, 0644);
         if (fd1 < 0)
           error(2, errno, "failed to open '%s'", stdout);
-        if (dup2(fd1, 1) < 0)
+        if (dup2(fd1, STDOUT_FILENO) < 0)
           error(2, errno, "dup2() failed");
         if (close(fd1) < 0)
           error(2, errno, "close() failed");
@@ -586,6 +704,11 @@ exec_with_timeout(const char* cmd)
   if (child_pid == 0)
     {
       setpgid(0, 0);
+      // Close stdin so that children may not read our input.  We had
+      // this nice surprise with Seminator, who greedily consumes its
+      // stdin (which was also ours) even if it does not use it
+      // because it was given a file.
+      close(STDIN_FILENO);
       exec_command(cmd);
       // never reached
       return -1;
@@ -610,6 +733,7 @@ exec_with_timeout(const char* cmd)
 
 enum {
   OPT_LIST = 1,
+  OPT_RELABEL = 2,
 };
 static const argp_option options[] =
 {
@@ -620,6 +744,8 @@ static const argp_option options[] =
     { "timeout", 'T', "NUMBER", 0, "kill translators after NUMBER seconds", 0 },
     { "list-shorthands", OPT_LIST, nullptr, 0,
       "list availabled shorthands to use in COMMANDFMT", 0},
+    { "relabel", OPT_RELABEL, nullptr, 0,
+      "always relabel atomic propositions before calling any translator", 0 },
     /**************************************************/
     { nullptr, 0, nullptr, 0,
       "COMMANDFMT should specify input and output arguments using the "
@@ -630,13 +756,15 @@ static const argp_option options[] =
     { "%F,%S,%L,%W", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "the formula as a file in Spot, Spin, LBT, or Wring's syntax", 0 },
     { "%O", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
-      "the automaton is output in HOA, never claim, LBTT, or ltl2dstar's "
+      "the automaton output in HOA, never claim, LBTT, or ltl2dstar's "
       "format", 0 },
     { "%%", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE, "a single %", 0 },
     { nullptr, 0, nullptr, 0,
       "If either %l, %L, or %T are used, any input formula that does "
       "not use LBT-style atomic propositions (i.e. p0, p1, ...) will be "
-      "relabeled automatically.\n"
+      "relabeled automatically.  Likewise if %s or %S are used with "
+      "atomic proposition that compatible with Spin's syntax.  You can "
+      "force this relabeling to always occur with option --relabel.\n"
       "The sequences %f,%s,%l,%w,%F,%S,%L,%W can optionally be \"infixed\""
       " by a bracketed sequence of operators to unabbreviate before outputing"
       " the formula.  For instance %[MW]f will rewrite operators M and W"
@@ -647,12 +775,14 @@ static const argp_option options[] =
     { nullptr, 0, nullptr, 0, nullptr, 0 }
 };
 
+bool opt_relabel = false;
+
 static int parse_opt_trans(int key, char* arg, struct argp_state*)
 {
   switch (key)
     {
     case 't':
-      translators.push_back(arg);
+      tools_push_trans(arg);
       break;
     case 'T':
       timeout = to_pos_int(arg);
@@ -662,8 +792,18 @@ static int parse_opt_trans(int key, char* arg, struct argp_state*)
 #endif
       break;
     case OPT_LIST:
-      show_shorthands();
+      show_shorthands(std::begin(shorthands_ltl), std::end(shorthands_ltl));
+      std::cout
+        << ("\nAny {name} and directory component is skipped for the purpose "
+            "of\nmatching those prefixes.  So for instance\n  "
+            "'{DRA} ~/mytools/ltl2dstar-0.5.2'\n"
+            "will be changed into\n  "
+            "'{DRA} ~/mytools/ltl2dstar-0.5.2 --output-format=hoa %[MW]L %O'"
+            "\n");
       exit(0);
+    case OPT_RELABEL:
+      opt_relabel = true;
+      break;
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -672,3 +812,61 @@ static int parse_opt_trans(int key, char* arg, struct argp_state*)
 
 const struct argp trans_argp = { options, parse_opt_trans, nullptr, nullptr,
                                  nullptr, nullptr, nullptr };
+
+
+
+static const argp_option options_aut[] =
+{
+    /**************************************************/
+    { nullptr, 0, nullptr, 0, "Specifying tools to call:", 2 },
+    { "tool", 't', "COMMANDFMT", 0,
+      "register one tool to call", 0 },
+    { "timeout", 'T', "NUMBER", 0, "kill tools after NUMBER seconds", 0 },
+    { "list-shorthands", OPT_LIST, nullptr, 0,
+      "list availabled shorthands to use in COMMANDFMT", 0},
+    /**************************************************/
+    { nullptr, 0, nullptr, 0,
+      "COMMANDFMT should specify input and output arguments using the "
+      "following character sequences:", 3 },
+    { "%H,%S,%L", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "filename for the input automaton in (H) HOA, (S) Spin's neverclaim, "
+      "or (L) LBTT's format", 0 },
+    { "%O", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "filename for the automaton output in HOA, never claim, LBTT, or "
+      "ltl2dstar's format", 0 },
+    { "%%", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE, "a single %", 0 },
+    { nullptr, 0, nullptr, 0, nullptr, 0 }
+};
+
+static int parse_opt_autproc(int key, char* arg, struct argp_state*)
+{
+  switch (key)
+    {
+    case 't':
+      tools_push_autproc(arg);
+      break;
+    case 'T':
+      timeout = to_pos_int(arg);
+#if !ENABLE_TIMEOUT
+      std::cerr << "warning: setting a timeout is not supported "
+                << "on your platform" << std::endl;
+#endif
+      break;
+    case OPT_LIST:
+      show_shorthands(std::begin(shorthands_autproc),
+                      std::end(shorthands_autproc));
+      std::cout
+        << ("\nAny {name} and directory component is skipped for the purpose "
+            "of\nmatching those prefixes.  So for instance\n  "
+            "'{AF} ~/mytools/autfilt-2.4 --remove-fin'\n"
+            "will be changed into\n  "
+            "'{AF} ~/mytools/autfilt-2.4 --remove-fin %H>%O'\n");
+      exit(0);
+    default:
+      return ARGP_ERR_UNKNOWN;
+    }
+  return 0;
+}
+
+const struct argp autproc_argp = { options_aut, parse_opt_autproc, nullptr,
+                                   nullptr, nullptr, nullptr, nullptr };
