@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -39,6 +39,8 @@
 #include "common_file.hh"
 #include "common_finput.hh"
 #include "common_hoaread.hh"
+#include "common_aoutput.hh"
+#include "common_color.hh"
 #include <spot/parseaut/public.hh>
 #include <spot/tl/print.hh>
 #include <spot/tl/apcollect.hh>
@@ -53,8 +55,9 @@
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/twaalgos/isweakscc.hh>
 #include <spot/twaalgos/word.hh>
-#include <spot/twaalgos/complement.hh>
+#include <spot/twaalgos/dualize.hh>
 #include <spot/twaalgos/cleanacc.hh>
+#include <spot/twaalgos/alternation.hh>
 #include <spot/misc/formater.hh>
 #include <spot/twaalgos/stats.hh>
 #include <spot/twaalgos/isdet.hh>
@@ -71,19 +74,20 @@ Call several LTL/PSL translators and cross-compare their output to detect \
 bugs, or to gather statistics.  The list of formulas to use should be \
 supplied on standard input, or using the -f or -F options.\v\
 Exit status:\n\
-  0  everything went fine (timeouts are OK too)\n\
+  0  everything went fine (without --fail-on-timeout, timeouts are OK)\n\
   1  some translator failed to output something we understand, or failed\n\
      sanity checks (statistics were output nonetheless)\n\
   2  ltlcross aborted on error\n\
 ";
 
 enum {
-  OPT_AUTOMATA = 256,
+  OPT_AMBIGUOUS = 256,
+  OPT_AUTOMATA,
   OPT_BOGUS,
-  OPT_COLOR,
   OPT_CSV,
   OPT_DENSITY,
   OPT_DUPS,
+  OPT_FAIL_ON_TIMEOUT,
   OPT_GRIND,
   OPT_IGNORE_EXEC_FAIL,
   OPT_JSON,
@@ -91,14 +95,19 @@ enum {
   OPT_NOCOMP,
   OPT_OMIT,
   OPT_PRODUCTS,
+  OPT_REFERENCE,
   OPT_SEED,
   OPT_STATES,
   OPT_STOP_ERR,
+  OPT_STRENGTH,
   OPT_VERBOSE,
 };
 
 static const argp_option options[] =
   {
+    { "reference", OPT_REFERENCE, "COMMANDFMT", 0,
+      "register one translator and assume it is correct (do not"
+      "check it for error, but use it to check other translators)", 2 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "ltlcross behavior:", 5 },
     { "allow-dups", OPT_DUPS, nullptr, 0,
@@ -117,6 +126,8 @@ static const argp_option options[] =
     { "ignore-execution-failures", OPT_IGNORE_EXEC_FAIL, nullptr, 0,
       "ignore automata from translators that return with a non-zero exit code,"
       " but do not flag this as an error", 0 },
+    { "fail-on-timeout", OPT_FAIL_ON_TIMEOUT, nullptr, 0,
+      "consider timeouts as errors", 0 },
     /**************************************************/
     { nullptr, 0, nullptr, 0, "State-space generation:", 6 },
     { "states", OPT_STATES, "INT", 0,
@@ -141,12 +152,15 @@ static const argp_option options[] =
       "do not output statistics for timeouts or failed translations", 0 },
     { "automata", OPT_AUTOMATA, nullptr, 0,
       "store automata (in the HOA format) into the CSV or JSON output", 0 },
+    { "strength", OPT_STRENGTH, nullptr, 0,
+      "output statistics about SCC strengths (non-accepting, terminal, weak, "
+      "strong) [not supported for alternating automata]", 0 },
+    { "ambiguous", OPT_AMBIGUOUS, nullptr, 0,
+      "output statistics about ambiguous automata "
+      "[not supported for alternating automata]", 0 },
+    { "unambiguous", 0, nullptr, OPTION_ALIAS, nullptr, 0 },
     /**************************************************/
-    { nullptr, 0, nullptr, 0, "Miscellaneous options:", -2 },
-    { "color", OPT_COLOR, "WHEN", OPTION_ARG_OPTIONAL,
-      "colorize output; WHEN can be 'never', 'always' (the default if "
-      "--color is used without argument), or "
-      "'auto' (the default if --color is not used)", 0 },
+    { nullptr, 0, nullptr, 0, "Output options:", -15 },
     { "grind", OPT_GRIND, "[>>]FILENAME", 0,
       "for each formula for which a problem was detected, write a simpler " \
       "formula that fails on the same test in FILENAME", 0 },
@@ -155,8 +169,10 @@ static const argp_option options[] =
     { "verbose", OPT_VERBOSE, nullptr, 0,
       "print what is being done, for debugging", 0 },
     { nullptr, 0, nullptr, 0,
-      "If an output FILENAME is prefixed with '>>', is it open "
-      "in append mode instead of being truncated.", -1 },
+      "If an output FILENAME is prefixed with '>>', it is open "
+      "in append mode instead of being truncated.", -14 },
+    /**************************************************/
+    { nullptr, 0, nullptr, 0, "Miscellaneous options:", -1 },
     { nullptr, 0, nullptr, 0, nullptr, 0 }
   };
 
@@ -165,32 +181,11 @@ const struct argp_child children[] =
     { &finput_argp, 0, nullptr, 1 },
     { &trans_argp, 0, nullptr, 0 },
     { &hoaread_argp, 0, "Parsing of automata:", 4 },
-    { &misc_argp, 0, nullptr, -2 },
+    { &color_argp, 0, nullptr, 0 },
+    { &misc_argp, 0, nullptr, -1 },
     { nullptr, 0, nullptr, 0 }
   };
 
-enum color_type { color_never, color_always, color_if_tty };
-
-static char const *const color_args[] =
-{
-  "always", "yes", "force",
-  "never", "no", "none",
-  "auto", "tty", "if-tty", nullptr
-};
-static color_type const color_types[] =
-{
-  color_always, color_always, color_always,
-  color_never, color_never, color_never,
-  color_if_tty, color_if_tty, color_if_tty
-};
-ARGMATCH_VERIFY(color_args, color_types);
-
-static color_type color_opt = color_if_tty;
-static const char* bright_red = "\033[01;31m";
-static const char* bright_blue = "\033[01;34m";
-static const char* bright_yellow = "\033[01;33m";
-static const char* reset_color = "\033[m";
-
 static unsigned states = 200;
 static float density = 0.1;
 static const char* json_output = nullptr;
@@ -211,7 +206,10 @@ static output_file* grind_output = nullptr;
 static bool verbose = false;
 static bool ignore_exec_fail = false;
 static unsigned ignored_exec_fail = 0;
+static bool fail_on_timeout = false;
 static bool opt_automata = false;
+static bool opt_strength = false;
+static bool opt_ambiguous = false;
 
 static bool global_error_flag = false;
 static unsigned oom_count = 0U;
@@ -220,25 +218,21 @@ static std::ostream&
 global_error()
 {
   global_error_flag = true;
-  if (color_opt)
-    std::cerr << bright_red;
+  std::cerr << bright_red;
   return std::cerr;
 }
 
 static std::ostream&
 example()
 {
-  if (color_opt)
-    std::cerr << bright_yellow;
+  std::cerr << bold_std;
   return std::cerr;
 }
 
-
 static void
 end_error()
 {
-  if (color_opt)
-    std::cerr << reset_color;
+  std::cerr << reset_color;
 }
 
 
@@ -246,6 +240,7 @@ struct statistics
 {
   statistics()
     : ok(false),
+      alternating(false),
       status_str(nullptr),
       status_code(0),
       time(0),
@@ -269,6 +264,7 @@ struct statistics
   // If OK is false, only the status_str, status_code, and time fields
   // should be valid.
   bool ok;
+  bool alternating;
   const char* status_str;
   int status_code;
   double time;
@@ -303,18 +299,21 @@ struct statistics
            "\"edges\","
            "\"transitions\","
            "\"acc\","
-           "\"scc\","
-           "\"nonacc_scc\","
-           "\"terminal_scc\","
-           "\"weak_scc\","
-           "\"strong_scc\","
-           "\"nondet_states\","
-           "\"nondet_aut\","
-           "\"terminal_aut\","
-           "\"weak_aut\","
-           "\"strong_aut\","
-           "\"ambiguous_aut\","
-           "\"complete_aut\"");
+           "\"scc\",");
+    if (opt_strength)
+      os << ("\"nonacc_scc\","
+             "\"terminal_scc\","
+             "\"weak_scc\","
+             "\"strong_scc\",");
+    os << ("\"nondet_states\","
+           "\"nondet_aut\",");
+    if (opt_strength)
+      os << ("\"terminal_aut\","
+             "\"weak_aut\","
+             "\"strong_aut\",");
+    if (opt_ambiguous)
+      os << "\"ambiguous_aut\",";
+    os << "\"complete_aut\"";
     size_t m = products_avg ? 1U : products;
     for (size_t i = 0; i < m; ++i)
       os << ",\"product_states\",\"product_transitions\",\"product_scc\"";
@@ -335,18 +334,36 @@ struct statistics
            << edges << ','
            << transitions << ','
            << acc << ','
-           << scc << ','
-           << nonacc_scc << ','
-           << terminal_scc << ','
-           << weak_scc << ','
-           << strong_scc << ','
-           << nondetstates << ','
-           << nondeterministic << ','
-           << terminal_aut << ','
-           << weak_aut << ','
-           << strong_aut << ','
-           << ambiguous << ','
-           << complete;
+           << scc << ',';
+        if (opt_strength)
+          {
+            if (alternating)
+              os << ",,,,";
+            else
+              os << nonacc_scc << ','
+                 << terminal_scc << ','
+                 << weak_scc << ','
+                 << strong_scc << ',';
+          }
+        os << nondetstates << ','
+           << nondeterministic << ',';
+        if (opt_strength)
+          {
+            if (alternating)
+              os << ",,,";
+            else
+              os << terminal_aut << ','
+                 << weak_aut << ','
+                 << strong_aut << ',';
+          }
+        if (opt_ambiguous)
+          {
+            if (alternating)
+              os << ',';
+            else
+              os << ambiguous << ',';
+          }
+        os << complete;
         if (!products_avg)
           {
             for (size_t i = 0; i < products; ++i)
@@ -374,7 +391,11 @@ struct statistics
       {
         size_t m = products_avg ? 1U : products;
         m *= 3;
-        m += 15;
+        m += 7;
+        if (opt_strength)
+          m += 7;
+        if (opt_ambiguous)
+          ++m;
         os << na;
         for (size_t i = 0; i < m; ++i)
           os << ',' << na;
@@ -411,7 +432,7 @@ parse_opt(int key, char* arg, struct argp_state*)
       if (arg[0] == '-' && !arg[1])
         jobs.emplace_back(arg, true);
       else
-        translators.push_back(arg);
+        tools_push_trans(arg);
       break;
     case OPT_AUTOMATA:
       opt_automata = true;
@@ -422,14 +443,6 @@ parse_opt(int key, char* arg, struct argp_state*)
         bogus_output_filename = arg;
         break;
       }
-    case OPT_COLOR:
-      {
-        if (arg)
-          color_opt = XARGMATCH("--color", arg, color_args, color_types);
-        else
-          color_opt = color_always;
-        break;
-      }
     case OPT_CSV:
       want_stats = true;
       csv_output = arg ? arg : "-";
@@ -440,6 +453,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_DUPS:
       allow_dups = true;
       break;
+    case OPT_FAIL_ON_TIMEOUT:
+      fail_on_timeout = true;
+      break;
     case OPT_GRIND:
       grind_output = new output_file(arg);
       break;
@@ -457,6 +473,8 @@ parse_opt(int key, char* arg, struct argp_state*)
           ++arg;
         }
       products = to_pos_int(arg);
+      if (products == 0)
+        products_avg = false;
       break;
     case OPT_NOCHECKS:
       no_checks = true;
@@ -468,6 +486,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_OMIT:
       opt_omit = true;
       break;
+    case OPT_REFERENCE:
+      tools_push_trans(arg, true);
+      break;
     case OPT_SEED:
       seed = to_pos_int(arg);
       break;
@@ -477,6 +498,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_STOP_ERR:
       stop_on_error = true;
       break;
+    case OPT_STRENGTH:
+      opt_strength = true;
+      break;
+    case OPT_AMBIGUOUS:
+      opt_ambiguous = true;
+      break;
     case OPT_VERBOSE:
       verbose = true;
       break;
@@ -503,26 +530,32 @@ namespace
       output.reset(translator_num);
 
       std::ostringstream command;
-      format(command, translators[translator_num].cmd);
+      format(command, tools[translator_num].cmd);
 
       std::string cmd = command.str();
       std::cerr << "Running [" << l << translator_num << "]: "
                 << cmd << std::endl;
-      spot::stopwatch sw;
-      sw.start();
+      spot::process_timer timer;
+      timer.start();
       int es = exec_with_timeout(cmd.c_str());
-      double duration = sw.stop();
-
+      timer.stop();
       const char* status_str = nullptr;
 
       spot::twa_graph_ptr res = nullptr;
       if (timed_out)
         {
-          // This is not considered to be a global error.
-          std::cerr << "warning: timeout during execution of command\n";
-          ++timeout_count;
+          if (fail_on_timeout)
+            {
+              global_error() << "error: timeout during execution of command\n";
+              end_error();
+            }
+          else
+            {
+              std::cerr << "warning: timeout during execution of command\n";
+              ++timeout_count;
+            }
           status_str = "timeout";
-          problem = false;        // A timeout is not a sign of a bug
+          problem = fail_on_timeout;
           es = -1;
         }
       else if (WIFSIGNALED(es))
@@ -593,7 +626,7 @@ namespace
           statistics* st = &(*fstats)[translator_num];
           st->status_str = status_str;
           st->status_code = es;
-          st->time = duration;
+          st->time = timer.walltime();
 
           // Compute statistics.
           if (res)
@@ -601,34 +634,43 @@ namespace
               if (verbose)
                 std::cerr << "info: getting statistics\n";
               st->ok = true;
+              st->alternating = !res->is_existential();
               spot::twa_sub_statistics s = sub_stats_reachable(res);
               st->states = s.states;
               st->edges = s.edges;
               st->transitions = s.transitions;
               st->acc = res->acc().num_sets();
-              spot::scc_info m(res);
+              spot::scc_info
+                m(res, (opt_strength
+                        ? spot::scc_info_options::TRACK_STATES
+                        : spot::scc_info_options::NONE));
               unsigned c = m.scc_count();
               st->scc = c;
               st->nondetstates = spot::count_nondet_states(res);
               st->nondeterministic = st->nondetstates != 0;
-              for (unsigned n = 0; n < c; ++n)
+              if (opt_strength && !st->alternating)
                 {
-                  if (m.is_rejecting_scc(n))
-                    ++st->nonacc_scc;
-                  else if (is_terminal_scc(m, n))
-                    ++st->terminal_scc;
-                  else if (is_weak_scc(m, n))
-                    ++st->weak_scc;
+                  m.determine_unknown_acceptance();
+                  for (unsigned n = 0; n < c; ++n)
+                    {
+                      if (m.is_rejecting_scc(n))
+                        ++st->nonacc_scc;
+                      else if (is_terminal_scc(m, n))
+                        ++st->terminal_scc;
+                      else if (is_weak_scc(m, n))
+                        ++st->weak_scc;
+                      else
+                        ++st->strong_scc;
+                    }
+                  if (st->strong_scc)
+                    st->strong_aut = true;
+                  else if (st->weak_scc)
+                    st->weak_aut = true;
                   else
-                    ++st->strong_scc;
+                    st->terminal_aut = true;
                 }
-              if (st->strong_scc)
-                st->strong_aut = true;
-              else if (st->weak_scc)
-                st->weak_aut = true;
-              else
-                st->terminal_aut = true;
-              st->ambiguous = !spot::is_unambiguous(res);
+              if (opt_ambiguous && !st->alternating)
+                st->ambiguous = !spot::is_unambiguous(res);
               st->complete = spot::is_complete(res);
 
               if (opt_automata)
@@ -698,7 +740,7 @@ namespace
           err << "*Comp(P" << j << ')';
         else
           err << "*N" << j;
-        err << " is nonempty; both automata accept the infinite word\n"
+        err << " is nonempty; both automata accept the infinite word:\n"
             << "       ";
         example() << *w << '\n';
         end_error();
@@ -732,16 +774,9 @@ namespace
     for (size_t i = 0; i < m; ++i)
       if (spot::scc_info* m = maps[i])
         {
-          // r == true iff the automaton i is accepting.
-          bool r = false;
-          for (auto& scc: *m)
-            if (scc.is_accepting())
-              {
-                r = true;
-                break;
-              }
-          res[i] = r;
-          if (r)
+          bool i_is_accepting = m->one_accepting_scc() >= 0;
+          res[i] = i_is_accepting;
+          if (i_is_accepting)
             ++verified;
           else
             ++violated;
@@ -859,14 +894,8 @@ namespace
           unsigned mutation_max;
           while        (res)
             {
-              std::cerr << "Trying to find a bogus mutation of ";
-              if (color_opt)
-                std::cerr << bright_blue;
-              std::cerr << bogus;
-              if (color_opt)
-                std::cerr << reset_color;
-              std::cerr << "...\n";
-
+              std::cerr << "Trying to find a bogus mutation of " << bold
+                        << bogus << reset_color << "...\n";
               mutations = mutate(f);
               mutation_count = 1;
               mutation_max = mutations.size();
@@ -891,19 +920,9 @@ namespace
                     bogus_output->ostream() << bogus << std::endl;
                 }
             }
-          std::cerr << "Smallest bogus mutation found for ";
-          if (color_opt)
-            std::cerr << bright_blue;
-          std::cerr << input;
-          if (color_opt)
-            std::cerr << reset_color;
-          std::cerr << " is ";
-          if (color_opt)
-            std::cerr << bright_blue;
-          std::cerr << bogus;
-          if (color_opt)
-            std::cerr << reset_color;
-          std::cerr << ".\n\n";
+          std::cerr << "Smallest bogus mutation found for "
+                    << bold << input << reset_color << " is "
+                    << bold << bogus << reset_color << ".\n\n";
           grind_output->ostream() << bogus << std::endl;
         }
       return 0;
@@ -941,10 +960,14 @@ namespace
     {
       static unsigned round = 0;
 
-      // If we need LBT atomic proposition in any of the input or
-      // output, relabel the formula.
-      if (!f.has_lbt_atomic_props() &&
-          (runner.has('l') || runner.has('L') || runner.has('T')))
+      if (opt_relabel
+          // If we need LBT atomic proposition in any of the input or
+          // output, relabel the formula.
+          ||  (!f.has_lbt_atomic_props() &&
+               (runner.has('l') || runner.has('L') || runner.has('T')))
+          // Likewise for Spin
+          || (!f.has_spin_atomic_props() &&
+              (runner.has('s') || runner.has('S'))))
         f = spot::relabel(f, spot::Pnn);
 
       // ---------- Positive Formula ----------
@@ -960,11 +983,7 @@ namespace
         std::cerr << linenum << ':';
       if (filename || linenum)
         std::cerr << ' ';
-      if (color_opt)
-        std::cerr << bright_blue;
-      std::cerr << fstr << '\n';
-      if (color_opt)
-        std::cerr << reset_color;
+      std::cerr << bold << fstr << reset_color << '\n';
 
       // Make sure we do not translate the same formula twice.
       if (!allow_dups)
@@ -980,12 +999,11 @@ namespace
             }
         }
 
-
       int problems = 0;
 
       // These store the result of the translation of the positive and
       // negative formulas.
-      size_t m = translators.size();
+      size_t m = tools.size();
       std::vector<spot::twa_graph_ptr> pos(m);
       std::vector<spot::twa_graph_ptr> neg(m);
       // These store the complement of the above results, when we can
@@ -1006,11 +1024,6 @@ namespace
           bool prob;
           pos[n] = runner.translate(n, 'P', pstats, prob);
           problems += prob;
-
-          // If the automaton is deterministic, compute its complement
-          // as well.
-          if (!no_complement && pos[n] && is_deterministic(pos[n]))
-            comp_pos[n] = dtwa_complement(pos[n]);
         }
 
       // ---------- Negative Formula ----------
@@ -1042,11 +1055,6 @@ namespace
               bool prob;
               neg[n] = runner.translate(n, 'N', nstats, prob);
               problems += prob;
-
-              // If the automaton is deterministic, compute its
-              // complement as well.
-              if (!no_complement && neg[n] && is_deterministic(neg[n]))
-                comp_neg[n] = dtwa_complement(neg[n]);
             }
         }
 
@@ -1066,9 +1074,13 @@ namespace
           auto tmp = [&](std::vector<spot::twa_graph_ptr>& x, unsigned i,
                          const char prefix)
             {
+              if (!x[i])
+                return;
               std::cerr << "info:   " << prefix << i << "\t(";
               printsize(x[i]);
               std::cerr << ')';
+              if (!x[i]->is_existential())
+                std::cerr << " univ-edges";
               if (is_deterministic(x[i]))
                 std::cerr << " deterministic";
               if (is_complete(x[i]))
@@ -1087,15 +1099,98 @@ namespace
           std::cerr << "Performing sanity checks and gathering statistics..."
                     << std::endl;
 
+          // If we have reference tools, pick the smallest of their
+          // automata for positive and negative references.
+          auto smallest_ref = [&](std::vector<spot::twa_graph_ptr>& from)
+            {
+              typedef std::tuple<bool, unsigned, unsigned, unsigned> aut_size;
+              int smallest_ref = -1;
+              aut_size smallest_size(true, -1U, -1U, -1U);
+              for (unsigned i = 0; i < m; ++i)
+                if (tools[i].reference && from[i])
+                  {
+                    aut_size pisize(!is_deterministic(from[i]),
+                                    from[i]->num_states(),
+                                    from[i]->num_edges(),
+                                    from[i]->num_sets());
+                    if (pisize < smallest_size)
+                      {
+                        smallest_ref = i;
+                        smallest_size = pisize;
+                      }
+                  }
+              return smallest_ref;
+            };
+
+          // This are not our definitive choice for reference
+          // automata, because the sizes might change after we remove
+          // alternation and Fin acceptance.  But we need to know now
+          // if we will have a pair of reference automata in order to
+          // skip some of the constructions.
+          int smallest_pos_ref = smallest_ref(pos);
+          int smallest_neg_ref = smallest_ref(neg);
+          if (smallest_pos_ref < 0 || smallest_neg_ref < 0)
+            smallest_pos_ref = smallest_neg_ref = -1;
+
+          bool print_first = verbose;
+          auto unalt = [&](std::vector<spot::twa_graph_ptr>& x,
+                           unsigned i, char prefix)
+            {
+              if (!x[i] || x[i]->is_existential())
+                return;
+              if (verbose)
+                {
+                  if (print_first)
+                    {
+                      std::cerr <<
+                        "info: getting rid of universal edges...\n";
+                      print_first = false;
+                    }
+                  std::cerr << "info:   " << prefix << i << "\t(";
+                  printsize(x[i]);
+                  std::cerr << ") ->";
+                }
+              x[i] = remove_alternation(x[i]);
+              if (verbose)
+                {
+                  std::cerr << " (";
+                  printsize(x[i]);
+                  std::cerr << ")\n";
+                }
+            };
+          auto complement = [&](const std::vector<spot::twa_graph_ptr>& x,
+                                std::vector<spot::twa_graph_ptr>& comp,
+                                unsigned i)
+            {
+              if (!no_complement && x[i] && is_universal(x[i]))
+                comp[i] = dualize(x[i]);
+            };
+
+          for (size_t i = 0; i < m; ++i)
+            {
+              unalt(pos, i, 'P');
+              unalt(neg, i, 'N');
+              // Do not complement reference automata if we have a
+              // reference pair.
+              if (smallest_pos_ref >= 0 && tools[i].reference)
+                continue;
+              complement(pos, comp_pos, i);
+              complement(neg, comp_neg, i);
+            }
+
           if (determinize && !no_complement)
             {
-              bool print_first = verbose;
+              print_first = verbose;
               auto tmp = [&](std::vector<spot::twa_graph_ptr>& from,
-                             std::vector<spot::twa_graph_ptr>& to, unsigned i,
+                             std::vector<spot::twa_graph_ptr>& to, int i,
                              char prefix)
                 {
-                  if (!to[i])
+                  if (from[i] && !to[i])
                     {
+                      // Do not complement reference automata if we have a
+                      // reference pair.
+                      if (smallest_pos_ref >= 0 && tools[i].reference)
+                        return;
                       if (print_first)
                         {
                           std::cerr << "info: complementing non-deterministic "
@@ -1106,7 +1201,7 @@ namespace
                       p.set_type(spot::postprocessor::Generic);
                       p.set_pref(spot::postprocessor::Deterministic);
                       p.set_level(spot::postprocessor::Low);
-                      to[i] = dtwa_complement(p.run(from[i]));
+                      to[i] = dualize(p.run(from[i]));
                       if (verbose)
                         {
                           std::cerr << "info:   " << prefix << i << "\t(";
@@ -1124,7 +1219,7 @@ namespace
                 }
             }
 
-          bool print_first = verbose;
+          print_first = verbose;
           auto tmp = [&](std::vector<spot::twa_graph_ptr>& x, unsigned i,
                          const char* prefix, const char* suffix)
             {
@@ -1145,9 +1240,21 @@ namespace
                       printsize(x[i]);
                       std::cerr << ") ->";
                     }
-                  cleanup_acceptance_here(x[i]);
-                  x[i] = remove_fin(x[i]);
-                  if (verbose)
+                  try
+                    {
+                      x[i] = remove_fin(x[i]);
+                    }
+                  catch (const std::runtime_error& err)
+                    {
+                      if (verbose)
+                        std::cerr << " failed (" << err.what() << ")\n";
+                      else
+                        std::cerr << "info: preprocessing "
+                                  << prefix << i << suffix
+                                  << " failed (" << err.what() << ")\n";
+                      x[i] = nullptr;
+                    }
+                  if (verbose && x[i])
                     {
                       std::cerr << " (";
                       printsize(x[i]);
@@ -1160,7 +1267,7 @@ namespace
                   cleanup_acceptance_here(x[i]);
                 }
             };
-          for (unsigned i = 0; i < m; ++i)
+          for (size_t i = 0; i < m; ++i)
             {
               tmp(pos, i, "     P", " ");
               tmp(neg, i, "     N", " ");
@@ -1168,14 +1275,40 @@ namespace
               tmp(comp_neg, i, "Comp(N", ")");
             }
 
+          if (smallest_pos_ref >= 0)
+            {
+              // Recompute the smallest references now, because removing
+              // alternation and Fin acceptance might have changed the
+              // sizes.
+              smallest_pos_ref = smallest_ref(pos);
+              smallest_neg_ref = smallest_ref(neg);
+
+              if (verbose)
+                std::cerr << "info: P" << smallest_pos_ref
+                          << " and N" << smallest_neg_ref << " assumed "
+                          << "correct and used as references\n";
+            }
+
           // intersection test
           for (size_t i = 0; i < m; ++i)
             if (pos[i])
               for (size_t j = 0; j < m; ++j)
                 if (neg[j])
                   {
-                    problems +=
-                      check_empty_prod(pos[i], neg[j], i, j, false, false);
+                    // Do not compare reference translators.
+                    if (tools[i].reference && tools[j].reference)
+                      continue;
+                    // If we have a reference pair, only compare
+                    // against that pair when i != j.
+                    if (i == j ||
+                        !((!tools[i].reference &&
+                           smallest_neg_ref >= 0 &&
+                           (size_t)smallest_neg_ref != j)
+                          || (!tools[j].reference &&
+                              smallest_pos_ref >= 0 &&
+                              (size_t)smallest_pos_ref != i)))
+                      problems +=
+                        check_empty_prod(pos[i], neg[j], i, j, false, false);
 
                     // Deal with the extra complemented automata if we
                     // have some.
@@ -1193,13 +1326,15 @@ namespace
                     // translation was not deterministic.
 
                     if (i != j && comp_pos[j] && !comp_neg[j])
-                      problems +=
-                        check_empty_prod(pos[i], comp_pos[j],
-                                         i, j, false, true);
+                      if (smallest_pos_ref < 0 || i == (size_t)smallest_pos_ref)
+                        problems +=
+                          check_empty_prod(pos[i], comp_pos[j],
+                                           i, j, false, true);
                     if (i != j && comp_neg[i] && !comp_pos[i])
-                      problems +=
-                        check_empty_prod(comp_neg[i], neg[j],
-                                         i, j, true, false);
+                      if (smallest_neg_ref < 0 || j == (size_t)smallest_neg_ref)
+                        problems +=
+                          check_empty_prod(comp_neg[i], neg[j],
+                                           i, j, true, false);
                     if (comp_pos[i] && comp_neg[j] &&
                         (i == j || (!comp_neg[i] && !comp_pos[j])))
                       problems +=
@@ -1248,70 +1383,77 @@ namespace
           std::vector<spot::scc_info*> pos_map(m);
           std::vector<spot::scc_info*> neg_map(m);
           for (size_t i = 0; i < m; ++i)
-            if (pos[i])
-              {
-                if (verbose)
-                  std::cerr << ("info: building product between state-space and"
-                                " P") << i
-                            << " (" << pos[i]->num_states() << " st., "
-                            << pos[i]->num_edges() << " ed.)\n";
-
-                spot::scc_info* sm = nullptr;
-                try
-                  {
-                    auto p = spot::product(pos[i], statespace);
-                    if (verbose)
-                      std::cerr << "info:   product has " << p->num_states()
-                                << " st., " << p->num_edges()
-                                << " ed.\n";
-                    sm = new spot::scc_info(p);
-                  }
-                catch (std::bad_alloc&)
-                  {
-                    std::cerr << ("warning: not enough memory to build "
-                                  "product of P") << i << " with state-space";
-                    if (products > 1)
-                      std::cerr << " #" << p << '/' << products << '\n';
-                    std::cerr << '\n';
-                    ++oom_count;
-                  }
-                pos_map[i] = sm;
-                product_stats(pstats, i, sm);
-              }
-
-          if (!no_checks)
-            for (size_t i = 0; i < m; ++i)
-              if (neg[i])
+            {
+              spot::scc_info* sm = nullptr;
+              if (pos[i])
                 {
                   if (verbose)
                     std::cerr << ("info: building product between state-space"
-                                  " and N") << i
-                              << " (" << neg[i]->num_states() << " st., "
-                              << neg[i]->num_edges() << " ed.)\n";
+                                  " and P") << i
+                              << " (" << pos[i]->num_states() << " st., "
+                              << pos[i]->num_edges() << " ed.)\n";
 
-                  spot::scc_info* sm = nullptr;
                   try
                     {
-                      auto p = spot::product(neg[i], statespace);
+                      auto p = spot::product(pos[i], statespace);
                       if (verbose)
                         std::cerr << "info:   product has " << p->num_states()
                                   << " st., " << p->num_edges()
                                   << " ed.\n";
-                      sm = new spot::scc_info(p);
+                      sm = new
+                        spot::scc_info(p, spot::scc_info_options::TRACK_STATES);
                     }
                   catch (std::bad_alloc&)
                     {
                       std::cerr << ("warning: not enough memory to build "
-                                    "product of N") << i << " with state-space";
+                                    "product of P") << i << " with state-space";
                       if (products > 1)
                         std::cerr << " #" << p << '/' << products << '\n';
                       std::cerr << '\n';
                       ++oom_count;
                     }
-
-                  neg_map[i] = sm;
-                  product_stats(nstats, i, sm);
+                  pos_map[i] = sm;
                 }
+              product_stats(pstats, i, sm);
+            }
+
+          if (!no_checks)
+            for (size_t i = 0; i < m; ++i)
+              {
+                spot::scc_info* sm = nullptr;
+                if (neg[i])
+                  {
+                    if (verbose)
+                      std::cerr << ("info: building product between state-space"
+                                    " and N") << i
+                                << " (" << neg[i]->num_states() << " st., "
+                                << neg[i]->num_edges() << " ed.)\n";
+
+                    try
+                      {
+                        auto p = spot::product(neg[i], statespace);
+                        if (verbose)
+                          std::cerr << "info:   product has " << p->num_states()
+                                    << " st., " << p->num_edges()
+                                    << " ed.\n";
+                        sm = new
+                          spot::scc_info(p,
+                                         spot::scc_info_options::TRACK_STATES);
+                      }
+                    catch (std::bad_alloc&)
+                      {
+                        std::cerr << ("warning: not enough memory to build "
+                                      "product of N")
+                                  << i << " with state-space";
+                        if (products > 1)
+                          std::cerr << " #" << p << '/' << products << '\n';
+                        std::cerr << '\n';
+                        ++oom_count;
+                      }
+                    neg_map[i] = sm;
+                  }
+                product_stats(nstats, i, sm);
+              }
 
           if (!no_checks)
             {
@@ -1321,7 +1463,7 @@ namespace
 
               // consistency check
               for (size_t i = 0; i < m; ++i)
-                if (pos_map[i] && neg_map[i])
+                if (pos_map[i] && neg_map[i] && !tools[i].reference)
                   {
                     if (verbose)
                       std::cerr << "info: consistency_check (P" << i
@@ -1372,7 +1514,7 @@ print_stats_csv(const char* filename)
   output_file outf(filename);
   std::ostream& out = outf.ostream();
 
-  unsigned ntrans = translators.size();
+  unsigned ntrans = tools.size();
   unsigned rounds = vstats.size();
   assert(rounds == formulas.size());
 
@@ -1391,11 +1533,12 @@ print_stats_csv(const char* filename)
           out << '"';
           spot::escape_rfc4180(out, formulas[r]);
           out << "\",\"";
-          spot::escape_rfc4180(out, translators[t].name);
+          spot::escape_rfc4180(out, tools[t].name);
           out << "\",";
           vstats[r][t].to_csv(out, !opt_omit);
           out << '\n';
         }
+  outf.close(filename);
 }
 
 static void
@@ -1407,16 +1550,16 @@ print_stats_json(const char* filename)
   output_file outf(filename);
   std::ostream& out = outf.ostream();
 
-  unsigned ntrans = translators.size();
+  unsigned ntrans = tools.size();
   unsigned rounds = vstats.size();
   assert(rounds == formulas.size());
 
   out << "{\n  \"tool\": [\n    \"";
-  spot::escape_str(out, translators[0].name);
+  spot::escape_str(out, tools[0].name);
   for (unsigned t = 1; t < ntrans; ++t)
     {
       out << "\",\n    \"";
-      spot::escape_str(out, translators[t].name);
+      spot::escape_str(out, tools[t].name);
     }
   out << "\"\n  ],\n  \"formula\": [\n    \"";
   spot::escape_str(out, formulas[0]);
@@ -1442,6 +1585,7 @@ print_stats_json(const char* filename)
           out << " ]";
         }
   out << "\n  ]\n}\n";
+  outf.close(filename);
 }
 
 int
@@ -1457,13 +1601,11 @@ main(int argc, char** argv)
 
   check_no_formula();
 
-  if (translators.empty())
+  if (tools.empty())
     error(2, 0, "No translator to run?  Run '%s --help' for usage.",
           program_name);
 
-  if (color_opt == color_if_tty)
-    color_opt = isatty(STDERR_FILENO) ? color_always : color_never;
-
+  setup_color();
   setup_sig_handler();
 
   processor p;
