@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche et
+// Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -36,6 +36,7 @@
 #include "common_range.hh"
 
 #include <spot/misc/hash.hh>
+#include <spot/misc/timer.hh>
 #include <spot/tl/simplify.hh>
 #include <spot/tl/length.hh>
 #include <spot/tl/relabel.hh>
@@ -43,13 +44,19 @@
 #include <spot/tl/remove_x.hh>
 #include <spot/tl/apcollect.hh>
 #include <spot/tl/exclusive.hh>
+#include <spot/tl/ltlf.hh>
 #include <spot/tl/print.hh>
+#include <spot/tl/hierarchy.hh>
+#include <spot/twaalgos/isdet.hh>
 #include <spot/twaalgos/ltl2tgba_fm.hh>
 #include <spot/twaalgos/minimize.hh>
+#include <spot/twaalgos/postproc.hh>
+#include <spot/twaalgos/product.hh>
+#include <spot/twaalgos/remfin.hh>
 #include <spot/twaalgos/strength.hh>
 #include <spot/twaalgos/stutter.hh>
+#include <spot/twaalgos/totgba.hh>
 #include <spot/twaalgos/word.hh>
-#include <spot/twaalgos/product.hh>
 
 const char argp_program_doc[] ="\
 Read a list of formulas and output them back after some optional processing.\v\
@@ -71,6 +78,7 @@ enum {
   OPT_EQUIVALENT_TO,
   OPT_EXCLUSIVE_AP,
   OPT_EVENTUAL,
+  OPT_FROM_LTLF,
   OPT_GUARANTEE,
   OPT_IGNORE_ERRORS,
   OPT_IMPLIED_BY,
@@ -79,6 +87,8 @@ enum {
   OPT_NEGATE,
   OPT_NNF,
   OPT_OBLIGATION,
+  OPT_PERSISTENCE,
+  OPT_RECURRENCE,
   OPT_REJECT_WORD,
   OPT_RELABEL,
   OPT_RELABEL_BOOL,
@@ -143,6 +153,9 @@ static const argp_option options[] =
       "two of them may not be true at the same time.  Use this option "
       "multiple times to declare independent groups of exclusive "
       "propositions.", 0 },
+    { "from-ltlf", OPT_FROM_LTLF, "alive", OPTION_ARG_OPTIONAL,
+      "transform LTLf (finite LTL) to LTL by introducing some 'alive'"
+      " proposition", 0 },
     DECLARE_OPT_R,
     LEVEL_DOC(4),
     /**************************************************/
@@ -173,6 +186,10 @@ static const argp_option options[] =
       "match guarantee formulas (even pathological)", 0 },
     { "obligation", OPT_OBLIGATION, nullptr, 0,
       "match obligation formulas (even pathological)", 0 },
+    { "persistence", OPT_PERSISTENCE, nullptr, 0,
+      "match persistence formulas (even pathological)", 0 },
+    { "recurrence", OPT_RECURRENCE, nullptr, 0,
+      "match recurrence formulas (even pathological)", 0 },
     { "size", OPT_SIZE, "RANGE", 0,
       "match formulas with size in RANGE", 0},
     // backward compatibility
@@ -222,6 +239,13 @@ static const argp_option options[] =
       "the name of the input file", 0 },
     { "%L", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "the original line number in the input file", 0 },
+    { "%r", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
+      "wall-clock time elapsed in seconds (excluding parsing)", 0 },
+    { "%R, %[LETTERS]R", 0, nullptr,
+      OPTION_DOC | OPTION_NO_USAGE,
+      "CPU time (excluding parsing), in seconds; Add LETTERS to restrict to"
+      "(u) user time, (s) system time, (p) parent process, "
+      "or (c) children processes.", 0 },
     { "%<", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "the part of the line before the formula if it "
       "comes from a column extracted from a CSV file", 0 },
@@ -230,6 +254,8 @@ static const argp_option options[] =
       "comes from a column extracted from a CSV file", 0 },
     { "%%", 0, nullptr, OPTION_DOC | OPTION_NO_USAGE,
       "a single %", 0 },
+    COMMON_LTL_OUTPUT_SPECS,
+    /**************************************************/
     { nullptr, 0, nullptr, 0, "Miscellaneous options:", -1 },
     { nullptr, 0, nullptr, 0, nullptr, 0 }
   };
@@ -237,8 +263,8 @@ static const argp_option options[] =
 const struct argp_child children[] =
   {
     { &finput_argp, 0, nullptr, 1 },
-    { &output_argp, 0, nullptr, -20 },
-    { &misc_argp, 0, nullptr, -1 },
+    { &output_argp, 0, nullptr, 0 },
+    { &misc_argp, 0, nullptr, 0 },
     { nullptr, 0, nullptr, 0 }
   };
 
@@ -266,6 +292,8 @@ static bool syntactic_si = false;
 static bool safety = false;
 static bool guarantee = false;
 static bool obligation = false;
+static bool recurrence = false;
+static bool persistence = false;
 static range size = { -1, -1 };
 static range bsize = { -1, -1 };
 enum relabeling_mode { NoRelabeling = 0, ApRelabeling, BseRelabeling };
@@ -276,6 +304,7 @@ static bool stutter_insensitive = false;
 static range ap_n = { -1, -1 };
 static int opt_max_count = -1;
 static long int match_count = 0;
+static const char* from_ltlf = nullptr;
 
 
 // We want all these variables to be destroyed when we exit main, to
@@ -403,6 +432,9 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_LTL:
       ltl = true;
       break;
+    case OPT_FROM_LTLF:
+      from_ltlf = arg ? arg : "alive";
+      break;
     case OPT_NEGATE:
       negate = true;
       break;
@@ -412,6 +444,12 @@ parse_opt(int key, char* arg, struct argp_state*)
     case OPT_OBLIGATION:
       obligation = true;
       break;
+    case OPT_PERSISTENCE:
+      persistence = true;
+      break;
+    case OPT_RECURRENCE:
+      recurrence = true;
+      break;
     case OPT_REJECT_WORD:
       try
         {
@@ -550,6 +588,10 @@ namespace
     process_formula(spot::formula f,
                     const char* filename = nullptr, int linenum = 0) override
     {
+      spot::process_timer timer;
+      timer.start();
+
+
       if (opt_max_count >= 0 && match_count >= opt_max_count)
         {
           abort_run = true;
@@ -559,6 +601,9 @@ namespace
       if (negate)
         f = spot::formula::Not(f);
 
+      if (from_ltlf)
+        f = spot::from_ltlf(f, from_ltlf);
+
       if (remove_x)
         {
           // If simplification are enabled, we do them before and after.
@@ -639,46 +684,80 @@ namespace
         || simpl.are_equivalent(f, opt->equivalent_to);
       matched &= !stutter_insensitive || spot::is_stutter_invariant(f);
 
-      if (matched && (obligation
+      if (matched && (obligation || recurrence || persistence
                       || !opt->acc_words.empty()
                       || !opt->rej_words.empty()))
         {
-          auto aut = ltl_to_tgba_fm(f, simpl.get_dict());
+          spot::twa_graph_ptr aut = nullptr;
 
-          if (matched && !opt->acc_words.empty())
-            for (auto& word_aut: opt->acc_words)
-              if (spot::product(aut, word_aut)->is_empty())
-                {
-                  matched = false;
-                  break;
-                }
-
-          if (matched && !opt->rej_words.empty())
-            for (auto& word_aut: opt->rej_words)
-              if (!spot::product(aut, word_aut)->is_empty())
-                {
-                  matched = false;
-                  break;
-                }
-
-          // Match obligations and subclasses using WDBA minimization.
-          // Because this is costly, we compute it later, so that we don't
-          // have to compute it on formulas that have been discarded for
-          // other reasons.
-          if (matched && obligation)
+          if (!opt->acc_words.empty() && !opt->rej_words.empty())
             {
-              auto min = minimize_obligation(aut, f);
-              assert(min);
-              if (aut == min)
+              aut = ltl_to_tgba_fm(f, simpl.get_dict(), true);
+
+              if (matched && !opt->acc_words.empty())
+                for (auto& word_aut: opt->acc_words)
+                  if (spot::product(aut, word_aut)->is_empty())
+                    {
+                      matched = false;
+                      break;
+                    }
+
+              if (matched && !opt->rej_words.empty())
+                for (auto& word_aut: opt->rej_words)
+                  if (!spot::product(aut, word_aut)->is_empty())
+                    {
+                      matched = false;
+                      break;
+                    }
+
+            }
+
+          // Some combinations of options can be simplified.
+          if (recurrence && persistence)
+            obligation = true;
+          if (obligation && recurrence)
+            recurrence = false;
+          if (obligation && persistence)
+            persistence = false;
+
+          // Try a syntactic match before looking at the automata.
+          if (matched &&
+              !((!persistence || f.is_syntactic_persistence())
+                && (!recurrence || f.is_syntactic_recurrence())
+                && (!guarantee || f.is_syntactic_guarantee())
+                && (!safety || f.is_syntactic_safety())
+                && (!obligation || f.is_syntactic_obligation())))
+            {
+              // Match obligations and subclasses using WDBA minimization.
+              // Because this is costly, we compute it later, so that we don't
+              // have to compute it on formulas that have been discarded for
+              // other reasons.
+              if (obligation && (safety || guarantee))
                 {
-                  // Not an obligation
-                  matched = false;
-                }
-              else
-                {
-                  matched &= !guarantee || is_terminal_automaton(min);
-                  matched &= !safety || is_safety_mwdba(min);
+                  if (!aut)
+                    aut = ltl_to_tgba_fm(f, simpl.get_dict(), true);
+
+                  auto min = minimize_obligation(aut, f);
+                  assert(min);
+                  if (aut == min)
+                    {
+                      // Not an obligation
+                      matched = false;
+                    }
+                  else
+                    {
+                      spot::scc_info si(min);
+                      matched &= !guarantee
+                        || is_terminal_automaton(min, &si, true);
+                      matched &= !safety || is_safety_automaton(min, &si);
+                    }
                 }
+              else if (obligation) // just obligation, not safety or guarantee
+                  matched &= is_obligation(f, aut);
+              else if (persistence)
+                matched &= spot::is_persistence(f);
+              else if (recurrence)
+                matched &= spot::is_recurrence(f, aut);
             }
         }
 
@@ -687,6 +766,8 @@ namespace
       if (unique && !unique_set.insert(f).second)
         matched = false;
 
+      timer.stop();
+
       if (matched)
         {
           if (opt->output_define
@@ -703,7 +784,7 @@ namespace
                                p.second, filename, linenum) << ")\n";
             }
           one_match = true;
-          output_formula_checked(f, filename, linenum, prefix, suffix);
+          output_formula_checked(f, &timer, filename, linenum, prefix, suffix);
           ++match_count;
         }
       return 0;
@@ -753,6 +834,7 @@ main(int argc, char** argv)
 
   if (output_format == count_output)
     std::cout << match_count << std::endl;
+  flush_cout();
 
   return one_match ? 0 : 1;
 }
