@@ -38,7 +38,6 @@
 #include <string.h>
 #include <math.h>
 #include <time.h>
-#include <assert.h>
 
 #include "kernel.h"
 #include "cache.h"
@@ -262,13 +261,13 @@ static void bdd_operator_noderesize(void)
    if (cacheratio > 0)
    {
       int newcachesize = bddnodesize / cacheratio;
-
       BddCache_resize(&applycache, newcachesize);
       BddCache_resize(&itecache, newcachesize);
       BddCache_resize(&quantcache, newcachesize);
       BddCache_resize(&appexcache, newcachesize);
       BddCache_resize(&replacecache, newcachesize);
       BddCache_resize(&misccache, newcachesize);
+      bddcachesize = misccache.tablesize;
    }
 }
 
@@ -415,7 +414,8 @@ BDD bdd_not(BDD r)
    CHECKa(r, bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
 
@@ -534,7 +534,8 @@ BDD bdd_apply(BDD l, BDD r, int op)
 #endif
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
       applyop = op;
@@ -569,111 +570,122 @@ static BDD apply_rec(BDD l, BDD r)
    BddCacheData *entry;
    BDD res;
 
-   switch (applyop)
-   {
-    case bddop_and:
-       if (l == r)
-	  return l;
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (ISZERO(l) /* ||  ISZERO(r) */)
-	  return 0;
-       if (ISONE(l))
-	  return r;
-       /* if (ISONE(r))
-	  return l; */
-       break;
-    case bddop_or:
-       if (l == r)
-	  return l;
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (ISONE(l)  ||  ISONE(r))
-	  return 1;
-       if (ISZERO(l))
-	  return r;
-       /* if (ISZERO(r))
-	  return l; */
-       break;
-    case bddop_xor:
-       if (l == r)
-	  return 0;
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (ISZERO(l))
-	  return r;
-       /* if (ISZERO(r))
-	  return l; */
-       break;
-    case bddop_nand:
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (ISZERO(l) /* || ISZERO(r) */)
-	  return 1;
-       break;
-    case bddop_nor:
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (ISONE(l)  ||  ISONE(r))
-	  return 0;
-       break;
-   case bddop_imp:
-      if (ISZERO(l))
-	 return 1;
-      if (ISONE(l))
-	 return r;
-      if (ISONE(r))
-	 return 1;
-      break;
-   case bddop_biimp:
-      /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-      if (l > r)
-	{
-	  BDD tmp = l;
-	  l = r;
-	  r = tmp;
-	}
-      break;
-   }
+#define APPLY_SHORTCUTS(op, rec)                        \
+   switch (op)                                          \
+     {                                                  \
+     case bddop_and:                                    \
+       if (l == r)                                      \
+         return rec(l);                                 \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISZERO(l) /* ||  ISZERO(r) */)               \
+         return 0;                                      \
+       if (ISONE(l))                                    \
+         return rec(r);                                 \
+       break;                                           \
+     case bddop_or:                                     \
+       if (l == r)                                      \
+         return rec(l);                                 \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISONE(l)  ||  ISONE(r))                      \
+         return 1;                                      \
+       if (ISZERO(l))                                   \
+         return rec(r);                                 \
+       break;                                           \
+     case bddop_xor:                                    \
+       if (l == r)                                      \
+         return 0;                                      \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISZERO(l))                                   \
+         return rec(r);                                 \
+       break;                                           \
+     case bddop_nand:                                   \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISZERO(l))                                   \
+         return 1;                                      \
+       break;                                           \
+     case bddop_nor:                                    \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISONE(l)  ||  ISONE(r))                      \
+         return 0;                                      \
+       break;                                           \
+     case bddop_invimp: /* l << r = r >> l */           \
+       {                                                \
+         BDD tmp = l;                                   \
+         l = r;                                         \
+         r = tmp;                                       \
+         op = bddop_imp;                                \
+       }                                                \
+       /* fall through */                               \
+     case bddop_imp:                                    \
+       if (ISONE(r) || ISZERO(l))                       \
+         return 1;                                      \
+       if (ISONE(l))                                    \
+         return rec(r);                                 \
+       break;                                           \
+     case bddop_biimp:                                  \
+       if (l == r)                                      \
+         return 1;                                      \
+       if (l > r)                                       \
+         {                                              \
+           BDD tmp = l;                                 \
+           l = r;                                       \
+           r = tmp;                                     \
+         }                                              \
+       if (ISONE(l))                                    \
+         return rec(r);                                 \
+       break;                                           \
+     case bddop_less:  /* l < r = r - l */              \
+       {                                                \
+         BDD tmp = l;                                   \
+         l = r;                                         \
+         r = tmp;                                       \
+         op = bddop_diff;                               \
+       }                                                \
+       /* fall through */                               \
+     case bddop_diff:  /* l - r = l &! r */             \
+       if (l == r || ISONE(r))                          \
+         return 0;                                      \
+       if (ISZERO(r))                                   \
+         return rec(l);                                 \
+       break;                                           \
+     }
 
-   if (ISCONST(l)  &&  ISCONST(r))
+   /* empty macro arguments are undefined in ISO C90 and ISO C++98, so
+      use + when we do not want to call any function.*/
+   APPLY_SHORTCUTS(applyop, +);
+
+   if (__unlikely(ISCONST(l)  &&  ISCONST(r)))
       res = oprres[applyop][l<<1 | r];
    else
    {
       entry = BddCache_lookup(&applycache, APPLYHASH(l,r,applyop));
 
-      // Check entry->c last, because not_rec() does not initialize it.
+      /* Check entry->c last, because not_rec() does not initialize it. */
       if (entry->i.a == l  &&  entry->i.c == applyop  &&  entry->i.b == r)
       {
 #ifdef CACHESTATS
@@ -902,7 +914,7 @@ BDD bdd_setxor(BDD l, BDD r)
 NAME    {* bdd\_implies *}
 SECTION {* operator *}
 SHORT   {* check whether one BDD implies another *}
-PROTO   {* int bdd_setxor(BDD l, BDD r) *}
+PROTO   {* int bdd_implies(BDD l, BDD r) *}
 DESCR   {* Check whether $l$ implies $r$, or whether $r$ contains $l$. *}
 RETURN  {* 1 if $l$ implies $r$, 0 otherwise *}
 */
@@ -917,7 +929,7 @@ int bdd_implies(BDD l, BDD r)
     return 0;
 
   entry = BddCache_lookup(&misccache, IMPLIESHASH(l,r));
-   // Check entry->b last, because not_rec() does not initialize it.
+  /* Check entry->b last, because not_rec() does not initialize it. */
   if (entry->i.a == l && entry->i.c == CACHEID_IMPLIES && entry->i.b == r)
    {
 #ifdef CACHESTATS
@@ -931,15 +943,30 @@ int bdd_implies(BDD l, BDD r)
 
   if (LEVEL(l) == LEVEL(r))
     {
-      res = bdd_implies(LOW(l), LOW(r)) && bdd_implies(HIGH(l), HIGH(r));
+      int hl = HIGH(l);
+      int hr = HIGH(r);
+      /* Avoid the recursion if the second implication will trivially
+         fail. */
+      if ((ISONE(hl) || ISZERO(hr)) && (hl != hr))
+        res = 0;
+      else
+        res = bdd_implies(LOW(l), LOW(r)) && bdd_implies(hl, hr);
     }
   else if (LEVEL(l) < LEVEL(r))
     {
-      res = bdd_implies(LOW(l), r) && bdd_implies(HIGH(l), r);
+      int hl = HIGH(l);
+      if (ISONE(hl) && (hl != r))
+        res = 0;
+      else
+        res = bdd_implies(LOW(l), r) && bdd_implies(HIGH(l), r);
     }
   else /* LEVEL(l) > LEVEL(r) */
     {
-      res = bdd_implies(l, LOW(r)) && bdd_implies(l, HIGH(r));
+      int hr = HIGH(r);
+      if (ISZERO(hr) && (hr != r))
+        res = 0;
+      else
+        res = bdd_implies(l, LOW(r)) && bdd_implies(l, HIGH(r));
     }
 
   entry->i.a = l;
@@ -977,7 +1004,8 @@ BDD bdd_ite(BDD f, BDD g, BDD h)
    CHECKa(h, bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
 
@@ -1153,7 +1181,8 @@ BDD bdd_restrict(BDD r, BDD var)
       return r;
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       if (varset2svartable(var) < 0)
 	 return bddfalse;
@@ -1250,7 +1279,8 @@ BDD bdd_constrain(BDD f, BDD c)
    CHECKa(c,bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
       miscid = CACHEID_CONSTRAIN;
@@ -1375,7 +1405,8 @@ BDD bdd_replace(BDD r, bddPair *pair)
    CHECKa(r, bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
       replacepair = pair->result;
@@ -1510,7 +1541,8 @@ BDD bdd_compose(BDD f, BDD g, int var)
 #endif
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE) ||
+       setjmp(bddexception) == 0)
    {
       INITREF;
       composelevel = bddvar2level[var];
@@ -1627,7 +1659,8 @@ BDD bdd_veccompose(BDD f, bddPair *pair)
    CHECKa(f, bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE) ||
+       setjmp(bddexception) == 0)
    {
       INITREF;
       replacepair = pair->result;
@@ -1715,7 +1748,8 @@ BDD bdd_simplify(BDD f, BDD d)
    CHECKa(d, bddfalse);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       INITREF;
       applyop = bddop_or;
@@ -1759,7 +1793,7 @@ static BDD simplify_rec(BDD f, BDD d)
 
    entry = BddCache_lookup(&applycache, APPLYHASH(f,d,bddop_simplify));
 
-   // Check entry->b last, because not_rec() does not initialize it.
+   /* Check entry->b last, because not_rec() does not initialize it. */
    if (entry->i.a == f && entry->i.c == bddop_simplify && entry->i.b == d)
    {
 #ifdef CACHESTATS
@@ -1824,7 +1858,8 @@ static BDD quantify(BDD r, BDD var, int op, int comp, int id)
       return r;
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       if (varset2vartable(var, comp) < 0)
 	 return bddfalse;
@@ -2015,7 +2050,8 @@ static BDD appquantify(BDD l, BDD r, int opr, BDD var,
       return bdd_apply(l,r,opr);
 
  again:
-   if (__likely(setjmp(bddexception) == 0))
+   if (__likely(bddreordermethod == BDD_REORDER_NONE)
+       || setjmp(bddexception) == 0)
    {
       if (varset2vartable(var, comp) < 0)
 	 return bddfalse;
@@ -2102,7 +2138,7 @@ SECTION {* operator *}
 SHORT   {* apply operation and universal quantification *}
 PROTO   {* BDD bdd_appall(BDD left, BDD right, int opr, BDD var) *}
 DESCR   {* Applies the binary operator {\tt opr} to the arguments
-	   {\tt left} and {\tt right} and then performs an universal
+	   {\tt left} and {\tt right} and then performs a universal
 	   quantification of the variables from the variable set
 	   {\tt var}. This is done in a bottom up manner such that both the
 	   apply and quantification is done on the lower nodes before
@@ -2124,7 +2160,7 @@ SECTION {* operator *}
 SHORT   {* apply operation and universal (complemented) quantification *}
 PROTO   {* BDD bdd_appall(BDD left, BDD right, int opr, BDD var) *}
 DESCR   {* Applies the binary operator {\tt opr} to the arguments
-	   {\tt left} and {\tt right} and then performs an universal
+	   {\tt left} and {\tt right} and then performs a universal
 	   quantification of the variables which are {\bf not} in the variable
 	   set {\tt var}. This is done in a bottom up manner such that both the
 	   apply and quantification is done on the lower nodes before
@@ -2189,95 +2225,7 @@ static int appquant_rec(int l, int r)
    BddCacheData *entry;
    int res;
 
-   switch (appexop)
-   {
-    case bddop_and:
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (l == 0 /* ||  r == 0 */)
-	  return 0;
-       if (l == r)
-	  return quant_rec(l);
-       if (l == 1)
-	  return quant_rec(r);
-       /* if (r == 1)
-	  return quant_rec(l); */
-       break;
-    case bddop_or:
-       if (l == 1  ||  r == 1)
-	  return 1;
-       if (l == r)
-	  return quant_rec(l);
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (l == 0)
-	  return quant_rec(r);
-       /* if (r == 0)
-	  return quant_rec(l); */
-       break;
-    case bddop_xor:
-       if (l == r)
-	  return 0;
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (l == 0)
-	  return quant_rec(r);
-       /* if (r == 0)
-	  return quant_rec(l); */
-       break;
-    case bddop_nand:
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       if (l == 0  /* ||  r == 0 */)
-	  return 1;
-       break;
-    case bddop_nor:
-       if (l == 1  ||  r == 1)
-	  return 0;
-       /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       break;
-    case bddop_biimp:
-      /* The operation is commutative, so lets
-	  order the arguments to favor cache hits. */
-       if (l > r)
-	 {
-	   BDD tmp = l;
-	   l = r;
-	   r = tmp;
-	 }
-       break;
-   }
+   APPLY_SHORTCUTS(appexop, quant_rec);
 
    if (ISCONST(l)  &&  ISCONST(r))
       res = oprres[appexop][(l<<1) | r];
