@@ -33,6 +33,7 @@ from spot.aux import \
 import subprocess
 import os
 import signal
+from contextlib import suppress as _supress
 
 
 # The parrameters used by default when show() is called on an automaton.
@@ -441,30 +442,33 @@ def automata(*sources, timeout=None, ignore_abort=True,
             else:
                 p = automaton_stream_parser(filename, o)
             a = True
-            while a:
-                # This returns None when we reach the end of the file.
-                a = p.parse(_bdd_dict).aut
-                if a:
-                    yield a
+            # Using proc as a context manager ensures that proc.stdout will be
+            # closed on exit, and the process will be properly waited for.
+            # This is important when running tools that produce an infinite
+            # stream of automata and that must be killed once the generator
+            # returned by spot.automata() is destroyed.  Otherwise, _supress()
+            # is just a dummy context manager that does nothing (Python 3.7
+            # introduces nullcontext() for this purpose, but at the time of
+            # writing we support Python 3.4).
+            mgr = proc if proc else _supress()
+            with mgr:
+                while a:
+                    # This returns None when we reach the end of the file.
+                    a = p.parse(_bdd_dict).aut
+                    if a:
+                        yield a
         finally:
             # Make sure we destroy the parser (p) and the subprocess
-            # (prop) in the correct order...
+            # (prop) in the correct order.
             del p
             if proc is not None:
-                if not a:
-                    # We reached the end of the stream.  Wait for the
-                    # process to finish, so that we get its exit code.
-                    ret = proc.wait()
-                else:
-                    # if a != None, we probably got there through an
-                    # exception, and the subprocess might still be
-                    # running.  Check if an exit status is available
-                    # just in case.
-                    ret = proc.poll()
+                ret = proc.returncode
                 del proc
-                if ret:
+                # Do not complain about the exit code if we are already raising
+                # an exception.
+                if ret and sys.exc_info()[0] is None:
                     raise subprocess.CalledProcessError(ret, filename[:-1])
-    # deleting o explicitely now prevents Python 3.5 from
+    # deleting o explicitly now prevents Python 3.5 from
     # reporting the following error: "<built-in function
     # delete_automaton_parser_options> returned a result with
     # an error set".  It's not clear to me if the bug is in Python
@@ -484,8 +488,11 @@ def automaton(filename, **kwargs):
 
 
 def _postproc_translate_options(obj, default_type, *args):
+    type_name_ = None
     type_ = None
+    pref_name_ = None
     pref_ = None
+    optm_name_ = None
     optm_ = None
     comp_ = 0
     unam_ = 0
@@ -493,10 +500,10 @@ def _postproc_translate_options(obj, default_type, *args):
     colo_ = 0
 
     def type_set(val):
-        nonlocal type_
-        if type_ is not None and type_ != val:
+        nonlocal type_, type_name_
+        if type_ is not None and type_name_ != val:
             raise ValueError("type cannot be both {} and {}"
-                             .format(type_, val))
+                             .format(type_name_, val))
         elif val == 'generic':
             type_ = postprocessor.Generic
         elif val == 'tgba':
@@ -529,12 +536,13 @@ def _postproc_translate_options(obj, default_type, *args):
         else:
             assert(val == 'monitor')
             type_ = postprocessor.Monitor
+        type_name_ = val
 
     def pref_set(val):
-        nonlocal pref_
-        if pref_ is not None and pref_ != val:
+        nonlocal pref_, pref_name_
+        if pref_ is not None and pref_name_ != val:
             raise ValueError("preference cannot be both {} and {}"
-                             .format(pref_, val))
+                             .format(pref_name, val))
         elif val == 'small':
             pref_ = postprocessor.Small
         elif val == 'deterministic':
@@ -542,12 +550,13 @@ def _postproc_translate_options(obj, default_type, *args):
         else:
             assert(val == 'any')
             pref_ = postprocessor.Any
+        pref_name_ = val
 
     def optm_set(val):
-        nonlocal optm_
-        if optm_ is not None and optm_ != val:
+        nonlocal optm_, optm_name_
+        if optm_ is not None and optm_name_ != val:
             raise ValueError("optimization level cannot be both {} and {}"
-                             .format(optm_, val))
+                             .format(optm_name_, val))
         if val == 'high':
             optm_ = postprocessor.High
         elif val.startswith('med'):
@@ -555,6 +564,7 @@ def _postproc_translate_options(obj, default_type, *args):
         else:
             assert(val == 'low')
             optm_ = postprocessor.Low
+        optm_name_ = val
 
     def misc_set(val):
         nonlocal comp_, unam_, sbac_, colo_
