@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016 Laboratoire de
-// Recherche et Développement de l'Epita (LRDE).
+// Copyright (C) 2010-2017 Laboratoire de Recherche et Développement de
+// l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -94,7 +94,7 @@ namespace spot
                     && (!dest->is_accepting_state()) && (!dest_trans_empty))
                   transitions_to_livelock_states->push_front(*it_trans);
 
-                // optimization to have, after minimization, an unique
+                // optimization to have, after minimization, a unique
                 // livelock state which has no successors
                 if (dest->is_livelock_accepting_state() && (dest_trans_empty))
                   dest->set_accepting_state(false);
@@ -102,33 +102,13 @@ namespace spot
                 ++it_trans;
               }
 
-          if (transitions_to_livelock_states)
-            {
-              state_ta_explicit::transitions::iterator it_trans;
-
-              for (it_trans = transitions_to_livelock_states->begin();
-                   it_trans != transitions_to_livelock_states->end();
-                   ++it_trans)
-                {
-                  if (artificial_livelock_acc_state)
-                    {
-                      testing_automata->create_transition
-                        (source,
-                         (*it_trans)->condition,
-                         (*it_trans)->acceptance_conditions,
-                         artificial_livelock_acc_state, true);
-                    }
-                  else
-                    {
-                      testing_automata->create_transition
-                        (source,
-                         (*it_trans)->condition,
-                         (*it_trans)->acceptance_conditions,
-                         ((*it_trans)->dest)->stuttering_reachable_livelock,
-                         true);
-                    }
-                }
-            }
+          for (auto* trans: *transitions_to_livelock_states)
+            testing_automata->create_transition
+              (source, trans->condition,
+               trans->acceptance_conditions,
+               artificial_livelock_acc_state ?
+               artificial_livelock_acc_state :
+               trans->dest->stuttering_reachable_livelock, true);
         }
       delete transitions_to_livelock_states;
 
@@ -333,7 +313,6 @@ namespace spot
                 {
                   auto self_loop_state = const_cast<state_ta_explicit*>
                     (down_cast<const state_ta_explicit*>(curr));
-                  assert(self_loop_state);
 
                   if (testing_aut->is_accepting_state(self_loop_state)
                       || (testing_aut->acc().accepting(acc_cond)))
