@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008, 2010, 2012, 2013, 2014, 2015 Laboratoire de
+// Copyright (C) 2008, 2010, 2012, 2013, 2014, 2015, 2016 Laboratoire de
 // Recherche et Développement de l'Epita (LRDE)
 // Copyright (C) 2003, 2004 Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -527,7 +527,7 @@ namespace spot
             emit(KNot);
             if (o == op::NegClosureMarked)
               os_ << (kw_ == utf8_kw ? "̃": "+");
-            // Fall through
+            SPOT_FALLTHROUGH;
           case op::Closure:
             os_ << '{';
             in_ratexp_ = true;
@@ -893,7 +893,7 @@ namespace spot
   std::string
   str_sere(formula f, bool full_parent)
   {
-    return str_(f, full_parent, false, spot_kw);
+    return str_(f, full_parent, true, spot_kw);
   }
 
 
@@ -918,7 +918,7 @@ namespace spot
   std::string
   str_utf8_sere(formula f, bool full_parent)
   {
-    return str_(f, full_parent, false, utf8_kw);
+    return str_(f, full_parent, true, utf8_kw);
   }
 
 
