@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche
-// et Développement de l'Epita.
+// Copyright (C) 2012-2018 Laboratoire de Recherche
+// et Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -26,6 +26,7 @@
 #include <vector>
 #include <algorithm>
 #include <iterator>
+#include <memory>
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/twa/bddprint.hh>
 
@@ -43,85 +44,133 @@ namespace spot
     struct degen_state_hash
     {
       size_t
-      operator()(const degen_state& s) const
+      operator()(const degen_state& s) const noexcept
       {
         return wang32_hash(s.first ^ wang32_hash(s.second));
       }
     };
 
     // Associate the degeneralized state to its number.
-    typedef std::unordered_map<degen_state, int, degen_state_hash> ds2num_map;
+    typedef std::unordered_map<degen_state, unsigned,
+                               degen_state_hash> ds2num_map;
 
     // Queue of state to be processed.
     typedef std::deque<degen_state> queue_t;
 
-    // Acceptance set common to all outgoing edges (of the same
-    // SCC -- we do not care about the other) of some state.
-    class outgoing_acc final
+    // Acceptance set common to all outgoing or incoming edges (of the
+    // same SCC -- we do not care about the others) of some state.
+    class inout_acc final
     {
       const_twa_graph_ptr a_;
-      typedef std::tuple<acc_cond::mark_t,
-                         acc_cond::mark_t,
-                         bool> cache_entry;
+      typedef std::tuple<acc_cond::mark_t, // common out
+                         acc_cond::mark_t, // union out
+                         acc_cond::mark_t, // common in, then common in+out
+                         bool,             // has self-loop
+                         bool> cache_entry; // is true state
       std::vector<cache_entry> cache_;
+      unsigned last_true_state_;
       const scc_info* sm_;
 
+      unsigned scc_of(unsigned s) const
+      {
+        return sm_ ? sm_->scc_of(s) : 0;
+      }
+
       void fill_cache(unsigned s)
       {
-        unsigned s1 = sm_ ? sm_->scc_of(s) : 0;
+        unsigned s1 = scc_of(s);
         acc_cond::mark_t common = a_->acc().all_sets();
         acc_cond::mark_t union_ = 0U;
         bool has_acc_self_loop = false;
+        bool is_true_state = false;
         bool seen = false;
         for (auto& t: a_->out(s))
           {
             // Ignore edges that leave the SCC of s.
             unsigned d = t.dst;
-            unsigned s2 = sm_ ? sm_->scc_of(d) : 0;
-            if (s2 != s1)
+            if (scc_of(d) != s1)
               continue;
 
             common &= t.acc;
             union_ |= t.acc;
+            std::get<2>(cache_[d]) &= t.acc;
 
             // an accepting self-loop?
-            has_acc_self_loop |= (t.dst == s) && a_->acc().accepting(t.acc);
+            if ((t.dst == s) && a_->acc().accepting(t.acc))
+              {
+                has_acc_self_loop = true;
+                if (t.cond == bddtrue)
+                  {
+                    is_true_state = true;
+                    last_true_state_ = s;
+                  }
+              }
             seen = true;
           }
         if (!seen)
           common = 0U;
-        cache_[s] = std::make_tuple(common, union_, has_acc_self_loop);
+        std::get<0>(cache_[s]) = common;
+        std::get<1>(cache_[s]) = union_;
+        std::get<3>(cache_[s]) = has_acc_self_loop;
+        std::get<4>(cache_[s]) = is_true_state;
       }
 
     public:
-      outgoing_acc(const const_twa_graph_ptr& a, const scc_info* sm):
+      inout_acc(const const_twa_graph_ptr& a, const scc_info* sm):
         a_(a), cache_(a->num_states()), sm_(sm)
       {
         unsigned n = a->num_states();
+        acc_cond::mark_t all = a_->acc().all_sets();
+        // slot 2 will hold acceptance mark that are common to the
+        // incoming transitions of each state.  For know with all
+        // marks if there is some incoming edge.  The next loop will
+        // constrain this value.
+        for (auto& e: a_->edges())
+          if (scc_of(e.src) == scc_of(e.dst))
+            std::get<2>(cache_[e.dst]) = all;
         for (unsigned s = 0; s < n; ++s)
           fill_cache(s);
+        for (unsigned s = 0; s < n; ++s)
+          std::get<2>(cache_[s]) |= std::get<0>(cache_[s]);
       }
 
       // Intersection of all outgoing acceptance sets
-      acc_cond::mark_t common_acc(unsigned s)
+      acc_cond::mark_t common_out_acc(unsigned s) const
       {
         assert(s < cache_.size());
         return std::get<0>(cache_[s]);
       }
 
       // Union of all outgoing acceptance sets
-      acc_cond::mark_t union_acc(unsigned s)
+      acc_cond::mark_t union_out_acc(unsigned s) const
       {
         assert(s < cache_.size());
         return std::get<1>(cache_[s]);
       }
 
-      // Has an accepting self-loop
-      bool has_acc_selfloop(unsigned s)
+      // Intersection of all incoming acceptance sets
+      acc_cond::mark_t common_inout_acc(unsigned s) const
       {
         assert(s < cache_.size());
         return std::get<2>(cache_[s]);
       }
+
+      bool has_acc_selfloop(unsigned s) const
+      {
+        assert(s < cache_.size());
+        return std::get<3>(cache_[s]);
+      }
+
+      bool is_true_state(unsigned s) const
+      {
+        assert(s < cache_.size());
+        return std::get<4>(cache_[s]);
+      }
+
+      unsigned last_true_state() const
+      {
+        return last_true_state_;
+      }
     };
 
     // Order of accepting sets (for one SCC)
@@ -192,13 +241,20 @@ namespace spot
     twa_graph_ptr
     degeneralize_aux(const const_twa_graph_ptr& a, bool use_z_lvl,
                      bool use_cust_acc_orders, int use_lvl_cache,
-                     bool skip_levels, bool ignaccsl)
+                     bool skip_levels, bool ignaccsl,
+                     bool remove_extra_scc)
     {
       if (!a->acc().is_generalized_buchi())
         throw std::runtime_error
-          ("degeneralize() can only works with generalized Büchi acceptance");
+          ("degeneralize() only works with generalized Büchi acceptance");
+      if (!a->is_existential())
+        throw std::runtime_error
+          ("degeneralize() does not support alternation");
 
-      bool use_scc = use_lvl_cache || use_cust_acc_orders || use_z_lvl;
+      bool use_scc = (use_lvl_cache
+                      || use_cust_acc_orders
+                      || use_z_lvl
+                      || remove_extra_scc);
 
       bdd_dict_ptr dict = a->get_dict();
 
@@ -209,7 +265,14 @@ namespace spot
       if (want_sba)
         res->prop_state_acc(true);
       // Preserve determinism, weakness, and stutter-invariance
-      res->prop_copy(a, { false, true, true, true });
+      res->prop_copy(a, { false, true, true, true, true, true });
+
+      auto orig_states = new std::vector<unsigned>();
+      auto levels = new std::vector<unsigned>();
+      orig_states->reserve(a->num_states()); // likely more are needed.
+      levels->reserve(a->num_states());
+      res->set_named_prop("original-states", orig_states);
+      res->set_named_prop("degen-levels", levels);
 
       // Create an order of acceptance conditions.  Each entry in this
       // vector correspond to an acceptance set.  Each index can
@@ -218,9 +281,7 @@ namespace spot
       // denote accepting states.
       std::vector<unsigned> order;
       {
-        // FIXME: revisit this comment once everything compiles again.
-        //
-        // The order is arbitrary, but it turns out that using push_back
+        // The order is arbitrary, but it turns out that using emplace_back
         // instead of push_front often gives better results because
         // acceptance sets at the beginning if the cycle are more often
         // used in the automaton.  (This surprising fact is probably
@@ -228,7 +289,7 @@ namespace spot
         // during the translation.)
         unsigned n = a->num_sets();
         for (unsigned i = n; i > 0; --i)
-          order.push_back(i - 1);
+          order.emplace_back(i - 1);
       }
 
       // Initialize scc_orders
@@ -245,59 +306,97 @@ namespace spot
       typedef std::map<int, unsigned> tr_cache_t;
       tr_cache_t tr_cache;
 
-      // Read this early, because it might create a state if the
-      // automaton is empty.
-      degen_state s(a->get_init_state_number(), 0);
-
       // State->level cache
       std::vector<std::pair<unsigned, bool>> lvl_cache(a->num_states());
 
       // Compute SCCs in order to use any optimization.
-      scc_info* m = nullptr;
-      if (use_scc)
-        m = new scc_info(a);
+      std::unique_ptr<scc_info> m = use_scc
+        ? std::make_unique<scc_info>(a, scc_info_options::NONE)
+        : nullptr;
 
-      // Cache for common outgoing acceptances.
-      outgoing_acc outgoing(a, m);
+      // Cache for common outgoing/incoming acceptances.
+      inout_acc inout(a, m.get());
 
       queue_t todo;
 
+      degen_state s(a->get_init_state_number(), 0);
+
       // As a heuristic for building SBA, if the initial state has at
       // least one accepting self-loop, start the degeneralization on
       // the accepting level.
-      if (want_sba && !ignaccsl && outgoing.has_acc_selfloop(s.first))
+      if (want_sba && !ignaccsl && inout.has_acc_selfloop(s.first))
         s.second = order.size();
       // Otherwise, check for acceptance conditions common to all
-      // outgoing edges, and assume we have already seen these and
-      // start on the associated level.
+      // outgoing edges, plus those common to all incoming edges, and
+      // assume we have already seen these and start on the associated
+      // level.
       if (s.second == 0)
         {
-          auto set = outgoing.common_acc(s.first);
-          if (use_cust_acc_orders)
+          auto set = inout.common_inout_acc(s.first);
+          if (SPOT_UNLIKELY(use_cust_acc_orders))
             s.second = orders.next_level(m->initial(), s.second, set);
           else
-            while (s.second < order.size()
-                   && set.has(order[s.second]))
+            while (s.second < order.size() && set.has(order[s.second]))
               {
                 ++s.second;
                 if (!skip_levels)
                   break;
               }
-          // There is not accepting level for TBA, let reuse level 0.
+          // There is no accepting level for TBA, let reuse level 0.
           if (!want_sba && s.second == order.size())
             s.second = 0;
         }
 
-      ds2num[s] = res->new_state();
-      todo.push_back(s);
+      auto new_state = [&](degen_state ds)
+        {
+          // Merge all true states into a single one.
+          bool ts = inout.is_true_state(ds.first);
+          if (ts)
+            ds = {inout.last_true_state(), 0U};
 
-      // If use_lvl_cache is on insert initial state to level cache
-      // Level cache stores first encountered level for each state.
-      // When entering an SCC first the lvl_cache is checked.
-      // If such state exists level from chache is used.
-      // If not, a new level (starting with 0) is computed.
-      if (use_lvl_cache)
-        lvl_cache[s.first] = std::make_pair(s.second, true);
+          auto di = ds2num.find(ds);
+          if (di != ds2num.end())
+              return di->second;
+
+          unsigned ns = res->new_state();
+          ds2num[ds] = ns;
+          if (ts)
+            {
+              res->new_acc_edge(ns, ns, bddtrue, true);
+              // As we do not process all outgoing transition of
+              // ds.first, it is possible that a non-deterministic
+              // automaton becomes deterministic.
+              if (res->prop_universal().is_false())
+                res->prop_universal(trival::maybe());
+            }
+          else
+            todo.emplace_back(ds);
+
+          assert(ns == orig_states->size());
+          orig_states->emplace_back(ds.first);
+          levels->emplace_back(ds.second);
+
+          // Level cache stores one encountered level for each state
+          // (the value of use_lvl_cache determinates which level
+          // should be remembered).  This cache is used when
+          // re-entering the SCC.
+          if (use_lvl_cache)
+            {
+              unsigned lvl = ds.second;
+              if (lvl_cache[ds.first].second)
+                {
+                  if (use_lvl_cache == 3)
+                    lvl = std::max(lvl_cache[ds.first].first, lvl);
+                  else if (use_lvl_cache == 2)
+                    lvl = std::min(lvl_cache[ds.first].first, lvl);
+                  else
+                    lvl = lvl_cache[ds.first].first; // Do not change
+                }
+              lvl_cache[ds.first] = std::make_pair(lvl, true);
+            }
+          return ns;
+        };
+      new_state(s);
 
       while (!todo.empty())
         {
@@ -339,7 +438,7 @@ namespace spot
 
               // The old level is slevel.  What should be the new one?
               auto acc = i.acc;
-              auto otheracc = outgoing.common_acc(d.first);
+              auto otheracc = inout.common_inout_acc(d.first);
 
               if (want_sba && is_acc)
                 {
@@ -380,10 +479,10 @@ namespace spot
                   if (!order.empty())
                     {
                       unsigned prev = order.size() - 1;
-                      auto common = outgoing.common_acc(s.first);
+                      auto common = inout.common_out_acc(s.first);
                       if (common.has(order[prev]))
                         {
-                          auto u = outgoing.union_acc(d.first);
+                          auto u = inout.union_out_acc(d.first);
                           if (!u.has(order[prev]))
                             acc -= a->acc().mark(order[prev]);
                         }
@@ -419,7 +518,8 @@ namespace spot
                     {
                       // Complete (or replace) the acceptance sets of
                       // this link with the acceptance sets common to
-                      // all edges leaving the destination state.
+                      // all edges of the destination SCC entering or
+                      // leaving the destination state.
                       if (s_scc == scc)
                         acc |= otheracc;
                       else
@@ -444,18 +544,18 @@ namespace spot
                           // the accepting level.
                           if (s_scc != scc
                               && !ignaccsl
-                              && outgoing.has_acc_selfloop(d.first))
+                              && inout.has_acc_selfloop(d.first))
                             {
                               d.second = order.size();
                             }
                           else
                             {
                               // Consider both the current acceptance
-                              // sets, and the acceptance sets common to
-                              // the outgoing edges of the
-                              // destination state.  But don't do
-                              // that if the state is accepting and we
-                              // are not skipping levels.
+                              // sets, and the acceptance sets common
+                              // to the outgoing edges of the
+                              // destination state.  But don't do that
+                              // if the state is accepting and we are
+                              // not skipping levels.
                               if (skip_levels || !is_acc)
                                 while (next < order.size()
                                        && acc.has(order[next]))
@@ -479,7 +579,7 @@ namespace spot
                   if (is_acc)        // The edge is accepting
                     {
                       d.second = 0; // Make it go to the first level.
-                      // Skip levels as much as possible.
+                      // Skip as many levels as possible.
                       if (!a->acc().accepting(acc) && !skip_levels)
                         {
                           if (use_cust_acc_orders)
@@ -497,34 +597,7 @@ namespace spot
                 }
 
               // Have we already seen this destination?
-              int dest;
-              ds2num_map::const_iterator di = ds2num.find(d);
-              if (di != ds2num.end())
-                {
-                  dest = di->second;
-                }
-              else
-                {
-                  dest = res->new_state();
-                  ds2num[d] = dest;
-                  todo.push_back(d);
-                  // Insert new state to cache
-
-                  if (use_lvl_cache)
-                    {
-                      auto lvl = d.second;
-                      if (lvl_cache[d.first].second)
-                        {
-                          if (use_lvl_cache == 3)
-                            lvl = std::max(lvl_cache[d.first].first, lvl);
-                          else if (use_lvl_cache == 2)
-                            lvl = std::min(lvl_cache[d.first].first, lvl);
-                          else
-                            lvl = lvl_cache[d.first].first; // Do not change
-                        }
-                      lvl_cache[d.first] = std::make_pair(lvl, true);
-                    }
-                }
+              int dest = new_state(d);
 
               unsigned& t = tr_cache[dest * 2 + is_acc];
 
@@ -539,16 +612,53 @@ namespace spot
 #ifdef DEGEN_DEBUG
       std::cout << "Orig. order:  \t";
       for (auto i: order)
-        {
-          std::cout << i << ", ";
-        }
+        std::cout << i << ", ";
       std::cout << '\n';
       orders.print();
 #endif
-
-      delete m;
-
       res->merge_edges();
+
+      unsigned res_ns = res->num_states();
+      if (!remove_extra_scc || res_ns <= a->num_states())
+        return res;
+
+      scc_info si_res(res, scc_info_options::TRACK_STATES);
+      unsigned res_scc_count = si_res.scc_count();
+      if (res_scc_count <= m->scc_count())
+        return res;
+
+      // If we reach this place, we have more SCCs in the output than
+      // in the input.  This means that we have created some redundant
+      // SCCs.  Often, these are trivial SCCs created in front of
+      // their larger sisters, because we did not pick the correct
+      // level when entering the SCC for the first time, and the level
+      // we picked has not been seen again when exploring the SCC.
+      // But it could also be the case that by entering the SCC in two
+      // different ways, we create two clones of the SCC (I haven't
+      // encountered any such case, but I do not want to rule it out
+      // in the code below).
+      //
+      // Now we will iterate over the SCCs in topological order to
+      // remember the "bottomost" SCCs that contain each original
+      // state.  If an original state is duplicated in a higher SCC,
+      // it can be shunted away.  Amen.
+      std::vector<unsigned> bottomost_occurence(a->num_states());
+      {
+        unsigned n = res_scc_count;
+        do
+          for (unsigned s: si_res.states_of(--n))
+            bottomost_occurence[(*orig_states)[s]] = s;
+        while (n);
+      }
+      std::vector<unsigned> retarget(res_ns);
+      for (unsigned n = 0; n < res_ns; ++n)
+        {
+          unsigned other = bottomost_occurence[(*orig_states)[n]];
+          retarget[n] = (si_res.scc_of(n) != si_res.scc_of(other)) ? other : n;
+        }
+      for (auto& e: res->edges())
+        e.dst = retarget[e.dst];
+      res->purge_unreachable_states();
       return res;
     }
   }
@@ -556,7 +666,8 @@ namespace spot
   twa_graph_ptr
   degeneralize(const const_twa_graph_ptr& a,
                bool use_z_lvl, bool use_cust_acc_orders,
-               int use_lvl_cache, bool skip_levels, bool ignaccsl)
+               int use_lvl_cache, bool skip_levels, bool ignaccsl,
+               bool remove_extra_scc)
   {
     // If this already a degeneralized digraph, there is nothing we
     // can improve.
@@ -564,13 +675,15 @@ namespace spot
       return std::const_pointer_cast<twa_graph>(a);
 
     return degeneralize_aux<true>(a, use_z_lvl, use_cust_acc_orders,
-                                  use_lvl_cache, skip_levels, ignaccsl);
+                                  use_lvl_cache, skip_levels, ignaccsl,
+                                  remove_extra_scc);
   }
 
   twa_graph_ptr
   degeneralize_tba(const const_twa_graph_ptr& a,
                    bool use_z_lvl, bool use_cust_acc_orders,
-                   int use_lvl_cache, bool skip_levels, bool ignaccsl)
+                   int use_lvl_cache, bool skip_levels, bool ignaccsl,
+                   bool remove_extra_scc)
   {
     // If this already a degeneralized digraph, there is nothing we
     // can improve.
@@ -578,6 +691,7 @@ namespace spot
       return std::const_pointer_cast<twa_graph>(a);
 
     return degeneralize_aux<false>(a, use_z_lvl, use_cust_acc_orders,
-                                   use_lvl_cache, skip_levels, ignaccsl);
+                                   use_lvl_cache, skip_levels, ignaccsl,
+                                   remove_extra_scc);
   }
 }
