@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014, 2015, 2016 Laboratoire de Recherche
-// et Developpement de l'Epita (LRDE).
+// Copyright (C) 2011, 2012, 2014-2017 Laboratoire de Recherche
+// et Développement de l'Epita (LRDE).
 // Copyright (C) 2003, 2004  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
 // et Marie Curie.
@@ -21,7 +21,9 @@
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <ostream>
+#include <sstream>
 #include <stdexcept>
+#include <spot/tl/print.hh>
 #include <spot/twa/twagraph.hh>
 #include <spot/twaalgos/dot.hh>
 #include <spot/twa/bddprint.hh>
@@ -29,13 +31,13 @@
 #include <spot/misc/escape.hh>
 #include <spot/twa/twagraph.hh>
 #include <spot/twa/formula2bdd.hh>
-#include <spot/twaalgos/copy.hh>
 #include <spot/twaalgos/sccinfo.hh>
 #include <spot/kripke/fairkripke.hh>
 #include <cstdlib>
 #include <cstring>
 #include <algorithm>
 #include <ctype.h>
+#include <utility>
 
 
 namespace spot
@@ -44,55 +46,75 @@ namespace spot
   {
     constexpr int MAX_BULLET = 20;
 
+    static constexpr const char palette[][8] =
+      {
+        "#1F78B4", /* blue */
+        "#FF4DA0", /* pink */
+        "#FF7F00", /* orange */
+        "#6A3D9A", /* purple */
+        "#33A02C", /* green */
+        "#E31A1C", /* red */
+        "#C4C400", /* yellowish */
+        "#505050", /* gray */
+        "#6BF6FF", /* light blue */
+        "#FF9AFF", /* light pink */
+        "#FF9C67", /* light orange */
+        "#B2A4FF", /* light purple */
+        "#A7ED79", /* light green */
+        "#FF6868", /* light red */
+        "#FFE040", /* light yellowish */
+        "#C0C090", /* light gray */
+      };
+
+    constexpr int palette_mod = sizeof(palette) / sizeof(*palette);
+
     class dotty_output final
     {
-      std::ostream& os_;
-      bool opt_force_acc_trans_ = false;
-      bool opt_horizontal_ = true;
-      bool opt_name_ = false;
-      enum { ShapeAuto = 0, ShapeCircle, ShapeEllipse }
-        opt_shape_ = ShapeAuto;
-      bool opt_show_acc_ = false;
-      bool mark_states_ = false;
-      bool opt_scc_ = false;
-      bool opt_html_labels_ = false;
-      bool opt_state_labels_ = false;
-      const_twa_graph_ptr aut_;
+      // Keep all 0/false-initialized values together.
       std::vector<std::string>* sn_ = nullptr;
       std::map<unsigned, unsigned>* highlight_edges_ = nullptr;
       std::map<unsigned, unsigned>* highlight_states_ = nullptr;
       std::vector<std::pair<unsigned, unsigned>>* sprod_ = nullptr;
-      std::set<unsigned>* incomplete_;
+      std::vector<unsigned>* orig_ = nullptr;
+      std::set<unsigned>* incomplete_ = nullptr;
       std::string* name_ = nullptr;
+      std::map<std::pair<int, int>, int> univ_done;
+
       acc_cond::mark_t inf_sets_ = 0U;
       acc_cond::mark_t fin_sets_ = 0U;
+      unsigned opt_shift_sets_ = 0;
+      enum { ShapeAuto = 0, ShapeCircle, ShapeEllipse } opt_shape_ = ShapeAuto;
+      bool opt_force_acc_trans_ = false;
+      bool opt_vertical_ = false;
+      bool opt_name_ = false;
+      bool opt_show_acc_ = false;
+      bool mark_states_ = false;
+      bool dcircles_ = false;
+      bool opt_scc_ = false;
+      bool opt_html_labels_ = false;
+      bool opt_color_sets_ = false;
+      bool opt_state_labels_ = false;
       bool opt_rainbow = false;
       bool opt_bullet = false;
       bool opt_bullet_but_buchi = false;
       bool opt_all_bullets = false;
       bool opt_ordered_edges_ = false;
       bool opt_numbered_edges_ = false;
-      bool opt_want_state_names_ = true;
-      unsigned opt_shift_sets_ = 0;
+      bool opt_orig_show_ = false;
+      bool max_states_given_ = false; // related to max_states_
+      bool opt_latex_ = false;
+      const char* nl_ = "\\n";
+      const char* label_pre_ = "label=\"";
+      char label_post_ = '"';
+
+      const_twa_graph_ptr aut_;
       std::string opt_font_;
       std::string opt_node_color_;
+      std::ostream& os_;
+      bool opt_want_state_names_ = true;
+      unsigned max_states_ = -1U; // related to max_states_given_
 
-      const char* const palette9[9] =
-        {
-          "#5DA5DA", /* blue */
-          "#F17CB0", /* pink */
-          "#FAA43A", /* orange */
-          "#B276B2", /* purple */
-          "#60BD68", /* green */
-          "#F15854", /* red */
-          "#B2912F", /* brown */
-          "#4D4D4D", /* gray */
-          "#DECF3F", /* yellow */
-        };
-      const char*const* palette = palette9;
-      int palette_mod = 9;
-      unsigned max_states_ = -1U;
-      bool max_states_given_ = false;
+      bool opt_shared_univ_dest_ = true;
 
     public:
 
@@ -171,6 +193,9 @@ namespace spot
             case 'a':
               opt_show_acc_ = true;
               break;
+            case 'A':
+              opt_show_acc_ = false;
+              break;
             case 'b':
               opt_bullet = true;
               opt_bullet_but_buchi = false;
@@ -195,6 +220,9 @@ namespace spot
                 options = end + 1;
               }
               break;
+            case 'd':
+              opt_orig_show_ = true;
+              break;
             case 'e':
               opt_shape_ = ShapeEllipse;
               break;
@@ -212,7 +240,7 @@ namespace spot
               }
               break;
             case 'h':
-              opt_horizontal_ = true;
+              opt_vertical_ = false;
               break;
             case 'k':
               opt_state_labels_ = true;
@@ -227,26 +255,47 @@ namespace spot
               opt_ordered_edges_ = true;
               break;
             case 'r':
-              opt_html_labels_ = true;
+              opt_color_sets_ = true;
               opt_rainbow = true;
               break;
             case 'R':
-              opt_html_labels_ = true;
+              opt_color_sets_ = true;
               opt_rainbow = false;
               break;
             case 's':
               opt_scc_ = true;
               break;
             case 'v':
-              opt_horizontal_ = false;
+              opt_vertical_ = true;
               break;
             case 't':
               opt_force_acc_trans_ = true;
               break;
+            case 'x':
+              opt_latex_ = true;
+              break;
+            case 'y':
+              opt_shared_univ_dest_ = false;
+              break;
             default:
               throw std::runtime_error
                 (std::string("unknown option for print_dot(): ") + c);
             }
+        if (opt_color_sets_ && !opt_latex_)
+          opt_html_labels_ = true;
+        if (opt_html_labels_)
+          {
+            nl_ = "<br/>";
+            label_pre_ = "label=<";
+            label_post_ = '>';
+          }
+        if (opt_latex_)
+          {
+            nl_ = "\\\\{}";
+            label_pre_ = "texlbl=\"";
+            label_post_ = '"';
+          }
+
       }
 
       dotty_output(std::ostream& os, const char* options)
@@ -255,50 +304,11 @@ namespace spot
         parse_opts(options ? options : ".");
       }
 
-      void
-      output_set(std::ostream& os, int v) const
-      {
-        v += opt_shift_sets_;
-        if (opt_bullet && (v >= 0) & (v <= MAX_BULLET))
-          {
-            static const char* const tab[MAX_BULLET + 1] = {
-              "⓿", "❶", "❷", "❸",
-              "❹", "❺", "❻", "❼",
-              "❽", "❾", "❿", "⓫",
-              "⓬", "⓭", "⓮", "⓯",
-              "⓰", "⓱", "⓲", "⓳",
-              "⓴",
-            };
-            os << tab[v];
-          }
-        else
-          {
-            os << v;
-          }
-      }
-
-      void
-      output_set(acc_cond::mark_t a) const
-      {
-        if (!opt_all_bullets)
-          os_ << '{';
-        const char* space = "";
-        for (auto v: a.sets())
-          {
-            if (!opt_all_bullets)
-              os_ << space;
-            output_set(os_, v);
-            space = ",";
-          }
-        if (!opt_all_bullets)
-          os_ << '}';
-      }
-
       const char*
       html_set_color(int v) const
       {
         if (opt_rainbow)
-          return palette[(v + opt_shift_sets_) % palette_mod];
+          return palette[v % palette_mod];
         // Color according to Fin/Inf
         if (inf_sets_.has(v))
           {
@@ -314,38 +324,88 @@ namespace spot
       }
 
       void
-      output_html_set_aux(std::ostream& os, int v) const
+      output_set(std::ostream& os, int v) const
       {
-        os << "<font color=\"" << html_set_color(v) << "\">";
-        output_set(os, v);
-        os << "</font>";
+        v += opt_shift_sets_;
+        if (opt_html_labels_)
+          os << "<font color=\"" << html_set_color(v) << "\">";
+        if (!opt_latex_ && opt_bullet && (v >= 0) & (v <= MAX_BULLET))
+          {
+            static const char* const tab[MAX_BULLET + 1] = {
+              "⓿", "❶", "❷", "❸",
+              "❹", "❺", "❻", "❼",
+              "❽", "❾", "❿", "⓫",
+              "⓬", "⓭", "⓮", "⓯",
+              "⓰", "⓱", "⓲", "⓳",
+              "⓴",
+            };
+            os << tab[v];
+          }
+        else if (opt_latex_ && opt_bullet)
+          {
+            os << "\\accset{" << v << '}';
+          }
+        else
+          {
+            os << v;
+          }
+        if (opt_html_labels_)
+          os << "</font>";
       }
 
-      void
-      output_html_set(int v) const
-      {
-        output_html_set_aux(os_, v);
-      }
 
       void
-      output_html_set(acc_cond::mark_t a) const
+      output_mark(acc_cond::mark_t a) const
       {
         if (!opt_all_bullets)
-          os_ << '{';
+          {
+            if (opt_latex_)
+              os_ << "\\{";
+            else
+              os_ << '{';
+          }
         const char* space = "";
         for (auto v: a.sets())
           {
             if (!opt_all_bullets)
               os_ << space;
-            output_html_set(v);
+            output_set(os_, v);
             space = ",";
           }
         if (!opt_all_bullets)
-          os_ << '}';
+          {
+            if (opt_latex_)
+              os_ << "\\}";
+            else
+              os_ << '}';
+          }
       }
 
-      std::string
-      state_label(unsigned s) const
+
+      std::ostream&
+      escape_for_output(std::ostream& os, const std::string& s) const
+      {
+        if (opt_html_labels_)
+          return escape_html(os, s);
+        if (opt_latex_)
+          return escape_latex(os, s);
+        return escape_str(os, s);
+      }
+
+      std::ostream&
+      format_label(std::ostream& os, bdd label) const
+      {
+        formula f = bdd_to_formula(label, aut_->get_dict());
+        if (opt_latex_)
+          {
+            print_sclatex_psl(os << '$', f) << '$';
+            return os;
+          }
+        return escape_for_output(os, str_psl(f));
+      }
+
+      std::ostream&
+      format_state_label(std::ostream& os, unsigned s) const
       {
         bdd label = bddfalse;
         for (auto& t: aut_->out(s))
@@ -355,8 +415,52 @@ namespace spot
           }
         if (label == bddfalse
             && incomplete_ && incomplete_->find(s) != incomplete_->end())
-          return "...";
-        return bdd_format_formula(aut_->get_dict(), label);
+            return os << "...";
+        return format_label(os, label);
+      }
+
+      std::string string_dst(int dst, int color_num = -1)
+      {
+        std::ostringstream tmp_dst;
+        tmp_dst << dst;
+        if (!opt_shared_univ_dest_ && color_num >= 0)
+          tmp_dst << '.' << color_num;
+        return tmp_dst.str();
+      }
+
+      void
+      print_dst(int dst, bool print_edges = false,
+                const char* style = nullptr, int color_num = -1)
+      {
+        int& univ = univ_done[std::make_pair(dst, color_num)];
+        if (univ == 2)
+          return;
+        std::string dest = string_dst(dst, color_num);
+        if (univ == 0)
+          os_ << "  " << dest << " [label=<>,shape=point]\n";
+        if (print_edges)
+          {
+            for (unsigned d: aut_->univ_dests(dst))
+              {
+                os_ << "  " << dest << " -> " << d;
+                if (style && *style)
+                  os_ << " [" << style << ']';
+                os_ << '\n';
+              }
+            univ = 2;
+          }
+        else
+          {
+            univ = 1;
+          }
+      }
+
+      void print_acceptance_for_human()
+      {
+        std::string accstr = aut_->acc().name("d");
+        if (accstr.empty())
+          return;
+        os_ << nl_ << '[' << accstr << ']';
       }
 
       void
@@ -365,47 +469,61 @@ namespace spot
         if (opt_html_labels_)
           std::tie(inf_sets_, fin_sets_) =
             aut_->get_acceptance().used_inf_fin_sets();
-        if (opt_bullet && aut_->num_sets() <= MAX_BULLET)
+        // UTF-8 has no glyphs for circled numbers larger than MAX_BULLET.
+        if (opt_bullet && (aut_->num_sets() <= MAX_BULLET || opt_latex_))
           opt_all_bullets = true;
         os_ << "digraph G {\n";
-        if (opt_horizontal_)
+        if (opt_latex_)
+          {
+            os_ << "  d2tgraphstyle=\"every node/.style={align=center}\"\n";
+            if (opt_bullet)
+              {
+                os_ << ("  d2tdocpreamble=\""
+                        "\\DeclareRobustCommand{\\accset}[1]{"
+                        "\\tikz[baseline=(num.base)]{\\node[fill=")
+                    << (opt_color_sets_ ? "accset#1" : "black")
+                    << (",draw=white,text=white,thin,circle,inner sep=1pt,"
+                        "font=\\bfseries\\sffamily] (num) {#1};}}");
+                if (opt_color_sets_)
+                  {
+                    unsigned n = aut_->num_sets();
+                    for (unsigned i = 0; i < n; ++i)
+                      os_ << "\n\\definecolor{accset" << i << "}{HTML}{"
+                          << (html_set_color(i) + 1) << '}';
+                  }
+                if (opt_bullet)
+                  os_ << "\"\n";
+              }
+          }
+        if (!opt_vertical_)
           os_ << "  rankdir=LR\n";
         if (name_ || opt_show_acc_)
           {
-            if (!opt_html_labels_)
+            os_ << "  " << label_pre_;
+            if (name_)
+              escape_for_output(os_, *name_);
+            if (opt_show_acc_)
               {
-                os_ << "  label=\"";
-                if (name_)
+                if (!dcircles_)
                   {
-                    escape_str(os_, *name_);
-                    if (opt_show_acc_)
-                      os_ << "\\n";
+                    if (name_)
+                      os_ << nl_;
+                    auto& acc = aut_->get_acceptance();
+                    auto outset = [this](std::ostream& os, int v)
+                                  {
+                                    this->output_set(os, v);
+                                  };
+                    if (opt_html_labels_)
+                      acc.to_html(os_, outset);
+                    else if (opt_latex_)
+                      acc.to_latex(os_ << '$', outset) << '$';
+                    else
+                      acc.to_text(os_, outset);
+
                   }
-                if (opt_show_acc_)
-                  aut_->get_acceptance().to_text
-                    (os_, [this](std::ostream& os, int v)
-                     {
-                       this->output_set(os, v);
-                     });
-                os_ << "\"\n";
-              }
-            else
-              {
-                os_ << "  label=<";
-                if (name_)
-                  {
-                    escape_html(os_, *name_);
-                    if (opt_show_acc_)
-                      os_ << "<br/>";
-                  }
-                if (opt_show_acc_)
-                  aut_->get_acceptance().to_html
-                    (os_, [this](std::ostream& os, int v)
-                     {
-                       this->output_html_set_aux(os, v);
-                     });
-                os_ << ">\n";
+                print_acceptance_for_human();
               }
+            os_ << label_post_ << '\n';
             os_ << "  labelloc=\"t\"\n";
           }
         switch (opt_shape_)
@@ -442,8 +560,13 @@ namespace spot
         if (!extra.empty())
           os_ << "  " << extra << '\n';
         os_ << "  I [label=\"\", style=invis, ";
-        os_ << (opt_horizontal_ ? "width" : "height");
-        os_ << "=0]\n  I -> " << aut_->get_init_state_number() << '\n';
+        os_ << (opt_vertical_ ? "height" : "width");
+        int init = (int) aut_->get_init_state_number();
+        os_ << "=0]\n  I -> " << init;
+        if (init >= 0)
+          os_ << '\n';
+        else
+          os_ << " [arrowhead=onormal]\n";
       }
 
       void
@@ -455,9 +578,17 @@ namespace spot
       void
       process_state(unsigned s)
       {
-        if (mark_states_ &&
-            ((opt_bullet && !opt_bullet_but_buchi)
-             || aut_->num_sets() != 1))
+        os_ << "  " << s << " [" << label_pre_;
+        if (sn_ && s < sn_->size() && !(*sn_)[s].empty())
+          escape_for_output(os_, (*sn_)[s]);
+        else if (sprod_)
+          os_ << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
+        else
+          os_ << s;
+        if (orig_ && s < orig_->size())
+          os_ << " (" << (*orig_)[s] << ')';
+
+        if (mark_states_ && !dcircles_)
           {
             acc_cond::mark_t acc = 0U;
             for (auto& t: aut_->out(s))
@@ -466,62 +597,24 @@ namespace spot
                 break;
               }
 
-            bool has_name = sn_ && s < sn_->size() && !(*sn_)[s].empty();
-
-            os_ << "  " << s << " [label=";
-            if (!opt_html_labels_)
+            if (acc)
               {
-                os_ << '"';
-                if (has_name)
-                  escape_str(os_, (*sn_)[s]);
-                else if (sprod_)
-                  os_ << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
-                else
-                  os_ << s;
-                if (acc)
-                  {
-                    os_ << "\\n";
-                    output_set(acc);
-                  }
-                if (opt_state_labels_)
-                  escape_str(os_ << "\\n", state_label(s));
-                os_ << '"';
-              }
-            else
-              {
-                os_ << '<';
-                if (has_name)
-                  escape_html(os_, (*sn_)[s]);
-                else if (sprod_)
-                  os_ << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
-                else
-                  os_ << s;
-                if (acc)
-                  {
-                    os_ << "<br/>";
-                    output_html_set(acc);
-                  }
-                if (opt_state_labels_)
-                  escape_html(os_ << "<br/>", state_label(s));
-                os_ << '>';
+                os_ << nl_;
+                output_mark(acc);
               }
+            if (opt_state_labels_)
+              format_state_label(os_ << nl_, s);
+            os_ << label_post_;
           }
         else
           {
-            os_ << "  " << s << " [label=\"";
-            if (sn_ && s < sn_->size() && !(*sn_)[s].empty())
-              escape_str(os_, (*sn_)[s]);
-            else if (sprod_)
-              os_ << (*sprod_)[s].first << ',' << (*sprod_)[s].second;
-            else
-              os_ << s;
             if (opt_state_labels_)
-              escape_str(os_ << "\\n", state_label(s));
-            os_ << '"';
+              format_state_label(os_ << nl_, s);
+            os_ << label_post_;
             // Use state_acc_sets(), not state_is_accepting() because
             // on co-Büchi automata we want to mark the rejecting
             // states.
-            if (mark_states_ && aut_->state_acc_sets(s))
+            if (dcircles_ && aut_->state_acc_sets(s))
               os_ << ", peripheries=2";
           }
         if (highlight_states_)
@@ -543,59 +636,71 @@ namespace spot
       }
 
       void
-      process_link(const twa_graph::edge_storage_t& t, int number)
+      process_link(const twa_graph::edge_storage_t& t, int number,
+                   bool print_edges)
       {
-        os_ << "  " << t.src << " -> " << t.dst;
-        std::string label;
-        if (!opt_state_labels_)
-          label = bdd_format_formula(aut_->get_dict(), t.cond);
-        if (!opt_html_labels_)
+        if (print_edges)
           {
-            os_ << " [label=\"";
-            escape_str(os_, label);
+            os_ << "  " << t.src << " -> " << (int)t.dst;
+            if (aut_->is_univ_dest(t.dst) && highlight_edges_
+                && !opt_shared_univ_dest_)
+              {
+                auto idx = aut_->get_graph().index_of_edge(t);
+                auto iter = highlight_edges_->find(idx);
+                if (iter != highlight_edges_->end())
+                  os_ << '.' << iter->second % palette_mod;
+              }
+            os_ << " [" << label_pre_;
+            if (!opt_state_labels_)
+              format_label(os_, t.cond);
             if (!mark_states_)
               if (auto a = t.acc)
                 {
                   if (!opt_state_labels_)
-                    os_ << "\\n";
-                  output_set(a);
+                    os_ << nl_;
+                  output_mark(a);
                 }
-            os_ << '"';
-          }
-        else
-          {
-            os_ << " [label=<";
-            escape_html(os_, label);
-            if (!mark_states_)
-              if (auto a = t.acc)
-                {
-                  if (!opt_state_labels_)
-                    os_ << "<br/>";
-                  output_html_set(a);
-                }
-            os_ << '>';
-          }
-        if (opt_ordered_edges_ || opt_numbered_edges_)
-          {
-            os_ << ", taillabel=\"";
-            if (opt_ordered_edges_)
-              os_ << number;
-            if (opt_ordered_edges_ && opt_numbered_edges_)
-              os_ << ' ';
-            if (opt_numbered_edges_)
-              os_ << '#' << aut_->get_graph().index_of_edge(t);
-            os_ << '"';
+            os_ << label_post_;
+            if (opt_ordered_edges_ || opt_numbered_edges_)
+              {
+                os_ << ", taillabel=\"";
+                if (opt_ordered_edges_)
+                  os_ << number;
+                if (opt_ordered_edges_ && opt_numbered_edges_)
+                  os_ << ' ';
+                if (opt_numbered_edges_)
+                  os_ << '#' << aut_->get_graph().index_of_edge(t);
+                os_ << '"';
+              }
           }
+
+        std::string highlight;
+        int color_num = -1;
         if (highlight_edges_)
           {
             auto idx = aut_->get_graph().index_of_edge(t);
             auto iter = highlight_edges_->find(idx);
             if (iter != highlight_edges_->end())
-              os_ << ", style=bold, color=\""
+              {
+                std::ostringstream o;
+                o << "style=bold, color=\""
                   << palette[iter->second % palette_mod]
                   << '"';
+                highlight = o.str();
+                if (print_edges)
+                  os_ << ", " << highlight;
+                if (!opt_shared_univ_dest_)
+                  color_num = iter->second % palette_mod;
+              }
           }
-        os_ << "]\n";
+        if (print_edges)
+          {
+            if (aut_->is_univ_dest(t.dst))
+              os_ << ", arrowhead=onormal";
+            os_ << "]\n";
+          }
+        if (aut_->is_univ_dest(t.dst))
+          print_dst(t.dst, print_edges, highlight.c_str(), color_num);
       }
 
       void print(const const_twa_graph_ptr& aut)
@@ -614,6 +719,8 @@ namespace spot
                   sprod_ = nullptr;
               }
           }
+        if (opt_orig_show_)
+          orig_ = aut->get_named_prop<std::vector<unsigned>>("original-states");
         if (opt_state_labels_)
           {
             // Verify that we can use state labels for this automaton.
@@ -645,13 +752,17 @@ namespace spot
           name_ = aut_->get_named_prop<std::string>("automaton-name");
         mark_states_ = (!opt_force_acc_trans_
                         && aut_->prop_state_acc().is_true());
+        dcircles_ = (mark_states_
+                     && (!opt_bullet || opt_bullet_but_buchi)
+                     && (aut_->acc().is_buchi() || aut_->acc().is_co_buchi()));
         if (opt_shape_ == ShapeAuto)
           {
-            if (sn_ || sprod_ || aut->num_states() > 100 || opt_state_labels_)
+            if (sn_ || sprod_ || aut->num_states() > 100
+                || opt_state_labels_ || orig_)
               {
                 opt_shape_ = ShapeEllipse;
                 // If all state names are short, prefer circles.
-                if (!opt_state_labels_ &&
+                if (!opt_state_labels_ && !orig_ &&
                     sn_ && std::all_of(sn_->begin(), sn_->end(),
                                        [](const std::string& s)
                                        { return s.size() <= 2; }))
@@ -668,7 +779,8 @@ namespace spot
         start();
         if (si)
           {
-            si->determine_unknown_acceptance();
+            if (aut->is_existential())
+              si->determine_unknown_acceptance();
 
             unsigned sccs = si->scc_count();
             for (unsigned i = 0; i < sccs; ++i)
@@ -693,13 +805,27 @@ namespace spot
                   {
                     // Reset the label, otherwise the graph label would
                     // be inherited by the cluster.
-                    os_ << "  label=\"\"\n";
+                    os_ << (opt_latex_ ? "  texlbl=\"\"\n" : "  label=\"\"\n");
                   }
                 for (auto s: si->states_of(i))
-                  process_state(s);
+                  {
+                    process_state(s);
+                    int trans_num = 0;
+                    unsigned scc_of_s = si->scc_of(s);
+                    for (auto& t : aut_->out(s))
+                      for (unsigned d: aut_->univ_dests(t.dst))
+                        if (si->scc_of(d) == scc_of_s)
+                          {
+                            process_link(t, trans_num++, false);
+                            break;
+                          }
+                  }
                 os_ << "  }\n";
               }
           }
+        int init = (int) aut_->get_init_state_number();
+        if (init < 0)
+          print_dst(init, true);
         unsigned ns = aut_->num_states();
         for (unsigned n = 0; n < ns; ++n)
           {
@@ -707,7 +833,7 @@ namespace spot
               process_state(n);
             int trans_num = 0;
             for (auto& t: aut_->out(n))
-              process_link(t, trans_num++);
+              process_link(t, trans_num++, true);
           }
         end();
       }
@@ -724,7 +850,7 @@ namespace spot
       d.parse_opts("k");
     auto aut = std::dynamic_pointer_cast<const twa_graph>(g);
     if (!aut || (d.max_states_given() && aut->num_states() >= d.max_states()))
-      aut = copy(g, twa::prop_set::all(), true, d.max_states() - 1);
+      aut = make_twa_graph(g, twa::prop_set::all(), true, d.max_states() - 1);
     d.print(aut);
     return os;
   }
