@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2012, 2014, 2015, 2016 Laboratoire de Recherche et
+// Copyright (C) 2011, 2012, 2014, 2015, 2016, 2017 Laboratoire de Recherche et
 // Developpement de l'Epita (LRDE).
 // Copyright (C) 2003, 2004  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -50,7 +50,7 @@ namespace spot
       std::vector<bool> common_acc;
       bool has_state_acc;
       bool is_complete;
-      bool is_deterministic;
+      bool is_universal;
       bool is_colored;
       bool use_implicit_labels;
       bool use_state_labels = true;
@@ -65,7 +65,7 @@ namespace spot
                bool state_labels)
       {
         check_det_and_comp(aut);
-        use_implicit_labels = implicit && is_deterministic && is_complete;
+        use_implicit_labels = implicit && is_universal && is_complete;
         use_state_labels &= state_labels;
         number_all_ap(aut);
       }
@@ -95,7 +95,7 @@ namespace spot
         std::string empty;
 
         unsigned ns = aut->num_states();
-        bool deterministic = true;
+        bool universal = true;
         bool complete = true;
         bool state_acc = true;
         bool nodeadend = true;
@@ -117,10 +117,10 @@ namespace spot
                   lastcond = t.cond;
                 if (complete)
                   sum |= t.cond;
-                if (deterministic)
+                if (universal)
                   {
                     if (!bdd_implies(t.cond, available))
-                      deterministic = false;
+                      universal = false;
                     else
                       available -= t.cond;
                   }
@@ -151,17 +151,31 @@ namespace spot
             common_acc.push_back(st_acc);
             state_acc &= st_acc;
           }
-        is_deterministic = deterministic;
+        is_universal = universal;
         is_complete = complete;
         has_state_acc = state_acc;
         // If the automaton has state-based acceptance and contain
         // some states without successors do not declare it as
         // colored.
         is_colored = colored && (!has_state_acc || nodeadend);
-        // If the automaton declares that it is deterministic or
+        // If the automaton declares that it is universal or
         // state-based, make sure that it really is.
-        assert(deterministic || aut->prop_deterministic() != true);
-        assert(state_acc || aut->prop_state_acc() != true);
+        if (aut->prop_universal().is_true() && !universal)
+          throw std::runtime_error("print_hoa(): automaton is not universal"
+                                   " but prop_universal()==true");
+        if (aut->prop_universal().is_false() && universal)
+          throw std::runtime_error("print_hoa(): automaton is universal"
+                                   " despite prop_universal()==false");
+        if (aut->prop_complete().is_true() && !complete)
+          throw std::runtime_error("print_hoa(): automaton is not complete"
+                                   " but prop_complete()==true");
+        if (aut->prop_complete().is_false() && complete)
+          throw std::runtime_error("print_hoa(): automaton is complete"
+                                   " but prop_complete()==false");
+        if (aut->prop_state_acc() && !state_acc)
+          throw std::runtime_error("print_hoa(): automaton has "
+                                   "transition-based acceptance despite"
+                                   " prop_state_acc()==true");
       }
 
       void number_all_ap(const const_twa_graph_ptr& aut)
@@ -186,7 +200,7 @@ namespace spot
             int v = bdd_var(all);
             all = bdd_high(all);
             ap.insert(std::make_pair(v, vap.size()));
-            vap.push_back(v);
+            vap.emplace_back(v);
           }
 
         if (use_implicit_labels)
@@ -312,16 +326,26 @@ namespace spot
             }
         }
 
-    // Calling get_init_state_number() may add a state to empty
-    // automata, so it has to be done first.
-    unsigned init = aut->get_init_state_number();
-
     metadata md(aut, implicit_labels, state_labels);
 
     if (acceptance == Hoa_Acceptance_States && !md.has_state_acc)
       acceptance = Hoa_Acceptance_Transitions;
 
+    auto print_dst = [&os, &aut](unsigned dst)
+      {
+        bool notfirst = false;
+        for (unsigned d: aut->univ_dests(dst))
+          {
+            if (notfirst)
+              os << '&';
+            else
+              notfirst = true;
+            os << d;
+          }
+      };
+
     unsigned num_states = aut->num_states();
+    unsigned init = aut->get_init_state_number();
 
     const char nl = newline ? '\n' : ' ';
     os << (v1_1 ? "HOA: v1.1" : "HOA: v1") << nl;
@@ -330,7 +354,9 @@ namespace spot
       escape_str(os << "name: \"", *n) << '"' << nl;
     unsigned nap = md.vap.size();
     os << "States: " << num_states << nl
-       << "Start: " << init << nl
+       << "Start: ";
+    print_dst(init);
+    os << nl
        << "AP: " << nap;
     auto d = aut->get_dict();
     for (auto& i: md.vap)
@@ -429,13 +455,6 @@ namespace spot
           }
         os << str;
       };
-    // We do not support alternating automata so far, and it's
-    // probable that nobody cares about the "no-univ-branch"
-    // properties.  The "univ-branch" properties seems more important
-    // to announce that the automaton might not be parsable by tools
-    // that do not support alternating automata.
-    if (verbose)
-      prop(" no-univ-branch");
     implicit_labels = md.use_implicit_labels;
     state_labels = md.use_state_labels;
     if (implicit_labels)
@@ -456,19 +475,63 @@ namespace spot
       prop(" complete");
     else if (v1_1)
       prop(" !complete");
-    if (md.is_deterministic)
-      prop(" deterministic");
-    else if (v1_1)
-      prop(" !deterministic");
+    // The definition of "deterministic" was changed between HOA v1
+    // (were it meant "universal") and HOA v1.1 were it means
+    // ("universal" and "existential").
+    if (!v1_1)
+      {
+        if (md.is_universal)
+          prop(" deterministic");
+        // It's probable that nobody cares about the "no-univ-branch"
+        // property.  The "univ-branch" property seems more important to
+        // announce that the automaton might not be parsable by tools that
+        // do not support alternating automata.
+        if (!aut->is_existential())
+          {
+            prop(" univ-branch");
+          }
+        else if (verbose)
+          {
+            if (v1_1)
+              prop(" !univ-branch");
+            else
+              prop(" no-univ-branch");
+          }
+      }
+    else
+      {
+        if (md.is_universal && aut->is_existential())
+          {
+            prop(" deterministic");
+            if (verbose)
+              prop(" !univ-branch !exist-branch");
+          }
+        else
+          {
+            prop(" !deterministic");
+            if (!aut->is_existential())
+              prop(" univ-branch");
+            else if (verbose)
+              prop(" !univ-branch");
+            if (!md.is_universal)
+              prop(" exist-branch");
+            else if (verbose)
+              prop(" !exist-branch");
+          }
+      }
     // Deterministic automata are also unambiguous, so writing both
     // properties seems redundant.  People working on unambiguous
     // automata are usually concerned about non-deterministic
     // unambiguous automata.  So do not mention "unambiguous"
     // in the case of deterministic automata.
-    if (aut->prop_unambiguous() && (verbose || !md.is_deterministic))
+    if (aut->prop_unambiguous() && (verbose || !md.is_universal))
       prop(" unambiguous");
     else if (v1_1 && !aut->prop_unambiguous())
       prop(" !unambiguous");
+    if (aut->prop_semi_deterministic() && (verbose || !md.is_universal))
+      prop(" semi-deterministic");
+    else if (v1_1 && !aut->prop_semi_deterministic())
+      prop(" !semi-deterministic");
     if (aut->prop_stutter_invariant())
       prop(" stutter-invariant");
     if (!aut->prop_stutter_invariant())
@@ -480,12 +543,18 @@ namespace spot
       }
     if (aut->prop_terminal())
       prop(" terminal");
-    if (aut->prop_weak() && (verbose || aut->prop_terminal() != true))
+    if (aut->prop_very_weak() && (verbose || aut->prop_terminal() != true))
+      prop(" very-weak");
+    if (aut->prop_weak() && (verbose || (aut->prop_terminal() != true &&
+                                         aut->prop_very_weak() != true)))
       prop(" weak");
     if (aut->prop_inherently_weak() && (verbose || aut->prop_weak() != true))
       prop(" inherently-weak");
     if (v1_1 && !aut->prop_terminal() && (verbose || aut->prop_weak() != false))
       prop(" !terminal");
+    if (v1_1 && !aut->prop_very_weak() && (verbose
+                                           || aut->prop_weak() != false))
+      prop(" !very-weak");
     if (v1_1 && !aut->prop_weak() && (verbose ||
                                       aut->prop_inherently_weak() != false))
       prop(" !weak");
@@ -542,6 +611,7 @@ namespace spot
       }
 
     os << "--BODY--" << nl;
+
     auto sn = aut->get_named_prop<std::vector<std::string>>("state-names");
     for (unsigned i = 0; i < num_states; ++i)
       {
@@ -583,7 +653,8 @@ namespace spot
 
             for (auto& t: aut->out(i))
               {
-                os << '[' << md.sup[t.cond] << "] " << t.dst;
+                os << '[' << md.sup[t.cond] << "] ";
+                print_dst(t.dst);
                 if (this_acc == Hoa_Acceptance_Transitions)
                   md.emit_acc(os, t.acc);
                 os << nl;
@@ -594,7 +665,7 @@ namespace spot
             unsigned n = 0;
             for (auto& t: aut->out(i))
               {
-                os << t.dst;
+                print_dst(t.dst);
                 if (this_acc == Hoa_Acceptance_Transitions)
                   {
                     md.emit_acc(os, t.acc);
@@ -640,7 +711,7 @@ namespace spot
             unsigned n = out.size();
             for (unsigned i = 0; i < n;)
               {
-                os << out[i];
+                print_dst(out[i]);
                 if (this_acc != Hoa_Acceptance_States)
                   {
                     md.emit_acc(os, outm[i]) << nl;
