@@ -1,9 +1,9 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
-// Laboratoire de Recherche et Développement de l'Epita (LRDE).
-// Copyright (C) 2003, 2004, 2005, 2006 Laboratoire
-// d'Informatique de Paris 6 (LIP6), département Systèmes Répartis
-// Coopératifs (SRC), Université Pierre et Marie Curie.
+// Copyright (C) 2008-2017 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
+// Copyright (C) 2003-2006 Laboratoire d'Informatique de Paris 6
+// (LIP6), département Systèmes Répartis Coopératifs (SRC), Université
+// Pierre et Marie Curie.
 //
 // This file is part of Spot, a model checking library.
 //
@@ -247,7 +247,7 @@ namespace spot
             a = bdd_low(a);
             if (h != bddfalse)
               {
-                t.push_back(bm[v]);
+                t.emplace_back(bm[v]);
                 if (a == bddfalse)
                   a = h;
               }
@@ -434,7 +434,7 @@ namespace spot
                 b = high;
               }
             assert(b != bddfalse);
-            v.push_back(res);
+            v.emplace_back(res);
           }
         return formula::multop(o, std::move(v));
       }
@@ -455,7 +455,7 @@ namespace spot
         minato_isop isop(f);
         bdd cube;
         while ((cube = isop.next()) != bddfalse)
-          v.push_back(conj_bdd_to_formula(cube));
+          v.emplace_back(conj_bdd_to_formula(cube));
         return formula::Or(std::move(v));
       }
 
@@ -469,7 +469,7 @@ namespace spot
         minato_isop isop(f);
         bdd cube;
         while ((cube = isop.next()) != bddfalse)
-          v.push_back(conj_bdd_to_sere(cube));
+          v.emplace_back(conj_bdd_to_sere(cube));
         return formula::OrRat(std::move(v));
       }
 
@@ -742,9 +742,9 @@ namespace spot
 
               for (auto g: f)
                 if (g.accepts_eword())
-                  final.push_back(g);
+                  final.emplace_back(g);
                 else
-                  non_final.push_back(g);
+                  non_final.emplace_back(g);
 
               if (non_final.empty())
                 // (a* & b*);c = (a*|b*);c
@@ -780,9 +780,9 @@ namespace spot
                       formula g = f[m];
                       if (n != m)
                         g = formula::Concat({g, star});
-                      conj.push_back(g);
+                      conj.emplace_back(g);
                     }
-                  disj.push_back(formula::AndRat(std::move(conj)));
+                  disj.emplace_back(formula::AndRat(std::move(conj)));
                 }
               return recurse_and_concat(formula::OrRat(std::move(disj)));
             }
@@ -813,9 +813,9 @@ namespace spot
               unsigned s = f.size();
               v.reserve(s);
               for (unsigned n = 1; n < s; ++n)
-                v.push_back(f[n]);
+                v.emplace_back(f[n]);
               if (to_concat_)
-                v.push_back(to_concat_);
+                v.emplace_back(to_concat_);
               return recurse(f[0], formula::Concat(std::move(v)));
             }
           case op::Fusion:
@@ -956,8 +956,8 @@ namespace spot
               all_props -= label;
 
               formula dest =
-                dict_.bdd_to_sere(bdd_exist(res & label, dict_.var_set));
-
+                dict_.bdd_to_sere(bdd_appex(res, label, bddop_and,
+                                            dict_.var_set));
               f2a_t::const_iterator i = f2a_.find(dest);
               if (i != f2a_.end() && i->second.first == nullptr)
                 continue;
@@ -1264,6 +1264,7 @@ namespace spot
             }
           case op::NegClosureMarked:
             has_marked_ = true;
+            SPOT_FALLTHROUGH;
           case op::NegClosure:
             rat_seen_ = true;
             {
@@ -1325,10 +1326,20 @@ namespace spot
               bdd f1 = recurse(node[0]);
               bdd f2 = recurse(node[1]);
               // r(f1 U f2) = r(f2) + a(f2)r(f1)X(f1 U f2) if not recurring
+              //                                           and f1 not universal
+              // r(f1 U f2) = r(f2) + a(f2)r(f1)X(Ff2)     if not recurring
+              //                                           and f1 universal
               // r(f1 U f2) = r(f2) + a(f2)r(f1)           if recurring
               f1 &= bdd_ithvar(dict_.register_a_variable(node[1]));
               if (!recurring_)
-                f1 &= bdd_ithvar(dict_.register_next_variable(node));
+                {
+                  formula nxt =
+                    node[0].is_universal() ? formula::F(node[1]) : node;
+                  // rewriting r(f1)X(f1 U f2) as r(f1)X(Ff2) when f1 is
+                  // universal is an optimization that helps with formulas
+                  // such as (G(Fa & Fb)) U a.
+                  f1 &= bdd_ithvar(dict_.register_next_variable(nxt));
+                }
               if (dict_.unambiguous)
                 f1 &= neg_of(node[1]);
               return f2 | f1;
@@ -1394,7 +1405,7 @@ namespace spot
             }
           case op::EConcatMarked:
             has_marked_ = true;
-            /* fall through */
+            SPOT_FALLTHROUGH;
           case op::EConcat:
             rat_seen_ = true;
             {
@@ -1420,7 +1431,7 @@ namespace spot
                       all_props -= label;
 
                       formula dest =
-                        dict_.bdd_to_sere(bdd_exist(f1 & label,
+                        dict_.bdd_to_sere(bdd_appex(f1, label, bddop_and,
                                                     dict_.var_set));
 
                       formula dest2 = formula::binop(o, dest, node[1]);
@@ -1504,7 +1515,8 @@ namespace spot
                       all_props -= label;
 
                       formula dest =
-                        dict_.bdd_to_sere(bdd_exist(f1 & label, dict_.var_set));
+                        dict_.bdd_to_sere(bdd_appex(f1, label, bddop_and,
+                                                    dict_.var_set));
 
                       formula dest2 = formula::binop(o, dest, node[1]);
 
@@ -1780,15 +1792,11 @@ namespace spot
           }
 
         // Apply the fair-loop approximation if requested.
-        if (fair_loop_approx_)
-          {
+        if (fair_loop_approx_
             // If the source cannot possibly be part of a fair
             // loop, make all possible promises.
-            if (fair_loop_approx_
-                && f != formula::tt()
-                && !pflc_.check(f))
-              t.symbolic &= all_promises_;
-          }
+            && !f.is_tt() && !pflc_.check(f))
+          t.symbolic &= all_promises_;
 
         // Register the reverse mapping if it is not already done.
         if (b2f_.find(t.symbolic) == b2f_.end())
@@ -1855,11 +1863,6 @@ namespace spot
       {
       }
 
-      transition(const transition& other)
-        : dest(other.dest), prom(other.prom), cond(other.cond)
-      {
-      }
-
       bool operator<(const transition& other) const
       {
         if (dest < other.dest)
@@ -2099,7 +2102,7 @@ namespace spot
 
                 bdd conds = bdd_existcomp(label, d.var_set);
                 bdd promises = bdd_existcomp(label, d.a_set);
-                dests.push_back(transition(dest, conds, promises));
+                dests.emplace_back(transition(dest, conds, promises));
               }
           }
 
