@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013, 2014, 2015, 2016 Laboratoire de recherche et
+// Copyright (C) 2011, 2013-2017  Laboratoire de recherche et
 // développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
@@ -59,7 +59,11 @@ namespace spot
         : emptiness_check(a, o),
           h(size)
       {
-        assert(a->num_sets() <= 1);
+        if (!(a->prop_weak().is_true()
+              || a->num_sets() == 0
+              || a->acc().is_buchi()))
+          throw std::runtime_error
+            ("magic_search requires Büchi or weak automata");
       }
 
       virtual ~magic_search_()
@@ -346,21 +350,21 @@ namespace spot
           for (; i != end; ++i, ++j)
             {
               twa_run::step s = { i->s->clone(), j->label, j->acc };
-              l->push_back(s);
+              l->emplace_back(s);
             }
 
           l = &run->cycle;
 
           j = ms_->st_red.rbegin();
           twa_run::step s = { i->s->clone(), j->label, j->acc };
-          l->push_back(s);
+          l->emplace_back(s);
 
           i = j; ++j;
           end = ms_->st_red.rend(); --end;
           for (; i != end; ++i, ++j)
             {
               twa_run::step s = { i->s->clone(), j->label, j->acc };
-              l->push_back(s);
+              l->emplace_back(s);
             }
 
           return run;
