@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016 Laboratoire de
-// Recherche et Développement de l'Epita (LRDE).
+// Copyright (C) 2010-2017 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -44,28 +44,27 @@
 #include <spot/twaalgos/ltl2tgba_fm.hh>
 #include <spot/twaalgos/bfssteps.hh>
 #include <spot/twaalgos/isdet.hh>
-#include <spot/twaalgos/complement.hh>
+#include <spot/twaalgos/dualize.hh>
 #include <spot/twaalgos/remfin.hh>
+#include <spot/tl/hierarchy.hh>
 
 namespace spot
 {
   // This is called hash_set for historical reason, but we need the
   // order inside hash_set to be deterministic.
-  typedef std::set<const state*, state_ptr_less_than> hash_set;
-  typedef state_map<unsigned> hash_map;
+  typedef std::set<unsigned> hash_set;
 
   namespace
   {
     static std::ostream&
     dump_hash_set(const hash_set* hs,
-                  const const_twa_ptr& aut,
                   std::ostream& out)
     {
       out << '{';
       const char* sep = "";
-      for (hash_set::const_iterator i = hs->begin(); i != hs->end(); ++i)
+      for (auto i: *hs)
         {
-          out << sep << aut->format_state(*i);
+          out << sep << i;
           sep = ", ";
         }
       out << '}';
@@ -73,47 +72,35 @@ namespace spot
     }
 
     static std::string
-    format_hash_set(const hash_set* hs, const_twa_ptr aut)
+    format_hash_set(const hash_set* hs)
     {
       std::ostringstream s;
-      dump_hash_set(hs, aut, s);
+      dump_hash_set(hs, s);
       return s.str();
     }
 
     // Find all states of an automaton.
     static void
-    build_state_set(const const_twa_ptr& a, hash_set* seen)
+    build_state_set(const const_twa_graph_ptr& a, hash_set* seen)
     {
-      std::queue<const state*> tovisit;
-      // Perform breadth-first traversal.
-      const state* init = a->get_init_state();
-      tovisit.push(init);
+      std::stack<unsigned> todo;
+      unsigned init = a->get_init_state_number();
+      todo.push(init);
       seen->insert(init);
-      while (!tovisit.empty())
+      while (!todo.empty())
         {
-          const state* src = tovisit.front();
-          tovisit.pop();
-
-          for (auto sit: a->succ(src))
-            {
-              const state* dst = sit->dst();
-              // Is it a new state ?
-              if (seen->find(dst) == seen->end())
-                {
-                  // Register the successor for later processing.
-                  tovisit.push(dst);
-                  seen->insert(dst);
-                }
-              else
-                dst->destroy();
-            }
+          unsigned s = todo.top();
+          todo.pop();
+          for (auto& e: a->out(s))
+            if (seen->insert(e.dst).second)
+              todo.push(e.dst);
         }
     }
 
     // From the base automaton and the list of sets, build the minimal
     // resulting automaton
     static twa_graph_ptr
-    build_result(const const_twa_ptr& a,
+    build_result(const const_twa_graph_ptr& a,
                  std::list<hash_set*>& sets,
                  hash_set* final)
     {
@@ -122,55 +109,46 @@ namespace spot
       res->copy_ap_of(a);
       res->prop_state_acc(true);
 
-      // For each set, create a state in the resulting automaton.
-      // For a state s, state_num[s] is the number of the state in the minimal
-      // automaton.
-      hash_map state_num;
-      std::list<hash_set*>::iterator sit;
-      for (sit = sets.begin(); sit != sets.end(); ++sit)
-        {
-          hash_set::iterator hit;
-          hash_set* h = *sit;
-          unsigned num = res->new_state();
-          for (hit = h->begin(); hit != h->end(); ++hit)
-            state_num[*hit] = num;
-        }
-
-      // For each transition in the initial automaton, add the corresponding
-      // transition in res.
+      // For each set, create a state in the output automaton.  For an
+      // input state s, state_num[s] is the corresponding the state in
+      // the output automaton.
+      std::vector<unsigned> state_num(a->num_states(), -1U);
+      {
+        unsigned num = res->new_states(sets.size());
+        for (hash_set* h: sets)
+          {
+            for (unsigned s: *h)
+              state_num[s] = num;
+            ++num;
+          }
+      }
 
       if (!final->empty())
         res->set_buchi();
 
-      for (sit = sets.begin(); sit != sets.end(); ++sit)
+      // For each transition in the initial automaton, add the
+      // corresponding transition in res.
+      for (hash_set* h: sets)
         {
-          hash_set* h = *sit;
-
           // Pick one state.
-          const state* src = *h->begin();
+          unsigned src = *h->begin();
           unsigned src_num = state_num[src];
           bool accepting = (final->find(src) != final->end());
 
           // Connect it to all destinations.
-          for (auto succit: a->succ(src))
+          for (auto& e: a->out(src))
             {
-              const state* dst = succit->dst();
-              hash_map::const_iterator i = state_num.find(dst);
-              dst->destroy();
-              if (i == state_num.end()) // Ignore useless destinations.
+              unsigned dn = state_num[e.dst];
+              if ((int)dn < 0)  // Ignore useless destinations.
                 continue;
-              res->new_acc_edge(src_num, i->second,
-                                succit->cond(), accepting);
+              res->new_acc_edge(src_num, dn, e.cond, accepting);
             }
         }
       res->merge_edges();
       if (res->num_states() > 0)
-        {
-          const state* init_state = a->get_init_state();
-          unsigned init_num = state_num[init_state];
-          init_state->destroy();
-          res->set_init_state(init_num);
-        }
+        res->set_init_state(state_num[a->get_init_state_number()]);
+      else
+        res->set_init_state(res->new_state());
       return res;
     }
 
@@ -271,7 +249,7 @@ namespace spot
       // The list of equivalent states.
       partition_t done;
 
-      hash_map state_set_map;
+      std::vector<unsigned> state_set_map(det_a->num_states(), -1U);
 
       // Size of det_a
       unsigned size = final->size() + non_final->size();
@@ -293,12 +271,11 @@ namespace spot
           used_var[set_num] = s;
           free_var.erase(set_num);
           if (s > 1)
-            cur_run.push_back(final);
+            cur_run.emplace_back(final);
           else
-            done.push_back(final);
-          for (hash_set::const_iterator i = final->begin();
-               i != final->end(); ++i)
-            state_set_map[*i] = set_num;
+            done.emplace_back(final);
+          for (auto i: *final)
+            state_set_map[i] = set_num;
 
           final_copy = new hash_set(*final);
         }
@@ -314,12 +291,11 @@ namespace spot
           used_var[num] = s;
           free_var.erase(num);
           if (s > 1)
-            cur_run.push_back(non_final);
+            cur_run.emplace_back(non_final);
           else
-            done.push_back(non_final);
-          for (hash_set::const_iterator i = non_final->begin();
-               i != non_final->end(); ++i)
-            state_set_map[*i] = num;
+            done.emplace_back(non_final);
+          for (auto i: *non_final)
+            state_set_map[i] = num;
         }
       else
         {
@@ -341,21 +317,17 @@ namespace spot
               hash_set* cur = cur_run.front();
               cur_run.pop_front();
 
-              trace << "processing " << format_hash_set(cur, det_a)
+              trace << "processing " << format_hash_set(cur)
                     << std::endl;
 
-              hash_set::iterator hi;
               bdd_states_map bdd_map;
-              for (hi = cur->begin(); hi != cur->end(); ++hi)
+              for (unsigned src: *cur)
                 {
-                  const state* src = *hi;
                   bdd f = bddfalse;
-                  for (auto si: det_a->succ(src))
+                  for (auto si: det_a->out(src))
                     {
-                      const state* dst = si->dst();
-                      hash_map::const_iterator i = state_set_map.find(dst);
-                      dst->destroy();
-                      if (i == state_set_map.end())
+                      unsigned i = state_set_map[si.dst];
+                      if ((int)i < 0)
                         // The destination state is not in our
                         // partition.  This can happen if the initial
                         // FINAL and NON_FINAL supplied to the algorithm
@@ -363,7 +335,7 @@ namespace spot
                         // want to ignore some useless states).  Simply
                         // ignore these states here.
                         continue;
-                      f |= (bdd_ithvar(i->second) & si->cond());
+                      f |= (bdd_ithvar(i) & si.cond);
                     }
 
                   // Have we already seen this formula ?
@@ -382,13 +354,13 @@ namespace spot
                     }
                 }
 
-              bdd_states_map::iterator bsi = bdd_map.begin();
+              auto bsi = bdd_map.begin();
               if (bdd_map.size() == 1)
                 {
                   // The set was not split.
-                  trace << "set " << format_hash_set(bsi->second, det_a)
+                  trace << "set " << format_hash_set(bsi->second)
                         << " was not split" << std::endl;
-                  next_run.push_back(bsi->second);
+                  next_run.emplace_back(bsi->second);
                 }
               else
                 {
@@ -413,21 +385,20 @@ namespace spot
                       num = *free_var.begin();
                       free_var.erase(free_var.begin());
                       used_var[num] = set->size();
-                      for (hash_set::iterator hit = set->begin();
-                           hit != set->end(); ++hit)
-                        state_set_map[*hit] = num;
-                      // Trivial sets can't be splitted any further.
+                      for (unsigned s: *set)
+                        state_set_map[s] = num;
+                      // Trivial sets can't be split any further.
                       if (set->size() == 1)
                         {
-                          trace << "set " << format_hash_set(set, det_a)
+                          trace << "set " << format_hash_set(set)
                                 << " is minimal" << std::endl;
-                          done.push_back(set);
+                          done.emplace_back(set);
                         }
                       else
                         {
-                          trace << "set " << format_hash_set(set, det_a)
+                          trace << "set " << format_hash_set(set)
                                 << " should be processed further" << std::endl;
-                          next_run.push_back(set);
+                          next_run.emplace_back(set);
                         }
                     }
                 }
@@ -444,8 +415,8 @@ namespace spot
 
 #ifdef TRACE
       trace << "Final partition: ";
-      for (partition_t::const_iterator i = done.begin(); i != done.end(); ++i)
-        trace << format_hash_set(*i, det_a) << ' ';
+      for (hash_set* hs: done)
+        trace << format_hash_set(hs) << ' ';
       trace << std::endl;
 #endif
 
@@ -454,15 +425,9 @@ namespace spot
 
       // Free all the allocated memory.
       delete final_copy;
-      hash_map::iterator hit;
-      for (hit = state_set_map.begin(); hit != state_set_map.end();)
-        {
-          hash_map::iterator old = hit++;
-          old->first->destroy();
-        }
-      std::list<hash_set*>::iterator it;
-      for (it = done.begin(); it != done.end(); ++it)
-        delete *it;
+
+      for (hash_set* hs: done)
+        delete hs;
 
       return res;
     }
@@ -470,6 +435,10 @@ namespace spot
 
   twa_graph_ptr minimize_monitor(const const_twa_graph_ptr& a)
   {
+    if (!a->is_existential())
+      throw std::runtime_error
+        ("minimize_monitor() does not support alternation");
+
     hash_set* final = new hash_set;
     hash_set* non_final = new hash_set;
     twa_graph_ptr det_a = tgba_powerset(a);
@@ -478,15 +447,26 @@ namespace spot
     // final is empty: there is no acceptance condition
     build_state_set(det_a, non_final);
     auto res = minimize_dfa(det_a, final, non_final);
-    res->prop_copy(a, { false, false, false, true });
-    res->prop_deterministic(true);
+    res->prop_copy(a, { false, false, false, false, true, true });
+    res->prop_universal(true);
     res->prop_weak(true);
     res->prop_state_acc(true);
+    // Quickly check if this is a terminal automaton
+    for (auto& e: res->edges())
+      if (e.src == e.dst && e.cond == bddtrue)
+        {
+          res->prop_terminal(true);
+          break;
+        }
     return res;
   }
 
   twa_graph_ptr minimize_wdba(const const_twa_graph_ptr& a)
   {
+    if (!a->is_existential())
+      throw std::runtime_error
+        ("minimize_wdba() does not support alternation");
+
     hash_set* final = new hash_set;
     hash_set* non_final = new hash_set;
 
@@ -494,7 +474,11 @@ namespace spot
 
     {
       power_map pm;
-      det_a = tgba_powerset(a, pm);
+      bool input_is_det = is_deterministic(a);
+      if (input_is_det)
+        det_a = std::const_pointer_cast<twa_graph>(a);
+      else
+        det_a = tgba_powerset(a, pm);
 
       // For each SCC of the deterministic automaton, determine if it
       // is accepting or not.
@@ -508,7 +492,9 @@ namespace spot
       // (i.e., it is not the start of any accepting word).
 
       scc_info sm(det_a);
-      sm.determine_unknown_acceptance();
+      if (input_is_det)
+        sm.determine_unknown_acceptance();
+
       unsigned scc_count = sm.scc_count();
       // SCC that have been marked as useless.
       std::vector<bool> useless(scc_count);
@@ -527,18 +513,13 @@ namespace spot
           bool transient = sm.is_trivial(m);
           auto& succ = sm.succ(m);
 
-          if (transient && succ.empty())
-            {
-              // A trivial SCC without successor is useless.
-              useless[m] = true;
-              d[m] = k - 1;
-              continue;
-            }
-
-          // Compute the minimum color l of the successors.
-          // Also SCCs are useless if all their successor are
-          // useless.
-          unsigned l = k;
+          // Compute the minimum color l of the successors.  Also SCCs
+          // are useless if all their successor are useless.  Note
+          // that Löding uses k-1 as level for non-final SCCs without
+          // successors but that seems bogus: using k+1 will make sure
+          // that a non-final SCCs without successor (i.e., a useless
+          // SCC) will be ignored in the computation of the level.
+          unsigned l = k + 1;
           for (unsigned j: succ)
             {
               is_useless &= useless[j];
@@ -555,14 +536,20 @@ namespace spot
             {
               // Regular SCCs are accepting if any of their loop
               // corresponds to an accepted word in the original
-              // automaton.
-              if (wdba_scc_is_accepting(det_a, m, a, sm, pm))
+              // automaton.  If the automaton is the same as det_a, we
+              // can simply ask that to sm.
+              bool acc_scc = input_is_det
+                ? sm.is_accepting_scc(m)
+                : wdba_scc_is_accepting(det_a, m, a, sm, pm);
+              if (acc_scc)
                 {
                   is_useless = false;
                   d[m] = l & ~1; // largest even number inferior or equal
                 }
               else
                 {
+                  if (succ.empty())
+                    is_useless = true;
                   d[m] = (l - 1) | 1; // largest odd number inferior or equal
                 }
             }
@@ -572,15 +559,15 @@ namespace spot
           if (!is_useless)
             {
               hash_set* dest_set = (d[m] & 1) ? non_final : final;
-              for (auto s: sm.states_of(m))
-                dest_set->insert(det_a->state_from_number(s));
+              auto& con = sm.states_of(m);
+              dest_set->insert(con.begin(), con.end());
             }
         }
     }
 
     auto res = minimize_dfa(det_a, final, non_final);
-    res->prop_copy(a, { false, false, false, true });
-    res->prop_deterministic(true);
+    res->prop_copy(a, { false, false, false, false, false, true });
+    res->prop_universal(true);
     res->prop_weak(true);
     // If the input was terminal, then the output is also terminal.
     // FIXME:
@@ -595,12 +582,67 @@ namespace spot
     return res;
   }
 
+  // Declared in tl/hierarchy.cc, but defined here because it relies on
+  // other internal functions from this file.
+  SPOT_LOCAL bool is_wdba_realizable(formula f, twa_graph_ptr aut = nullptr);
+
+  bool is_wdba_realizable(formula f, twa_graph_ptr aut)
+  {
+    if (f.is_syntactic_obligation())
+      return true;
+
+    if (aut == nullptr)
+      aut = ltl_to_tgba_fm(f, make_bdd_dict(), true);
+
+    if (!aut->is_existential())
+      throw std::runtime_error
+        ("is_wdba_realizable() does not support alternation");
+
+    if ((f.is_syntactic_persistence() || aut->prop_weak())
+        && (f.is_syntactic_recurrence() || is_deterministic(aut)))
+      return true;
+
+    if (is_terminal_automaton(aut))
+      return true;
+
+    // FIXME: we do not need to minimize the wdba to test realizability.
+    auto min_aut = minimize_wdba(aut);
+
+    if (!is_deterministic(aut)
+        && !product(aut, remove_fin(dualize(min_aut)))->is_empty())
+      return false;
+
+    twa_graph_ptr aut_neg;
+    if (is_deterministic(aut))
+      {
+        aut_neg = remove_fin(dualize(aut));
+      }
+    else
+      {
+        aut_neg = ltl_to_tgba_fm(formula::Not(f), aut->get_dict());
+        aut_neg = scc_filter(aut_neg, true);
+      }
+
+    if (is_terminal_automaton(aut_neg))
+      return true;
+
+    return product(min_aut, aut_neg)->is_empty();
+  }
+
+
   twa_graph_ptr
   minimize_obligation(const const_twa_graph_ptr& aut_f,
                       formula f,
                       const_twa_graph_ptr aut_neg_f,
                       bool reject_bigger)
   {
+    if (!aut_f->is_existential())
+      throw std::runtime_error
+        ("minimize_obligation() does not support alternation");
+
+    // FIXME: We should build scc_info once, pass it to minimize_wdba
+    // and reuse it for is_terminal_automaton(), and
+    // is_weak_automaton().
     auto min_aut_f = minimize_wdba(aut_f);
 
     if (reject_bigger)
@@ -611,25 +653,42 @@ namespace spot
           return std::const_pointer_cast<twa_graph>(aut_f);
       }
 
-    // If the input automaton was already weak and deterministic, the
-    // output is necessary correct.
-    if (aut_f->prop_weak() && aut_f->prop_deterministic())
-      return min_aut_f;
-
     // if f is a syntactic obligation formula, the WDBA minimization
     // must be correct.
     if (f && f.is_syntactic_obligation())
       return min_aut_f;
 
+    // If the input automaton was already weak and deterministic, the
+    // output is necessary correct.
+    if (aut_f->prop_weak() && is_deterministic(aut_f))
+      return min_aut_f;
+
     // If aut_f is a guarantee automaton, the WDBA minimization must be
     // correct.
     if (is_terminal_automaton(aut_f))
       return min_aut_f;
 
+    // If the input was deterministic, then by construction
+    // min_aut_f accepts at least all the words of aut_f, so we do
+    // not need the reverse check.  Otherwise, complement the
+    // minimized WDBA to test the reverse inclusion.
+    // (remove_fin() has a special handling of weak automata, and
+    // dualize also has a special handling of deterministic
+    // automata, so all these steps are quite efficient.)
+    if (!is_deterministic(aut_f)
+        && !product(aut_f, remove_fin(dualize(min_aut_f)))->is_empty())
+      return std::const_pointer_cast<twa_graph>(aut_f);
+
     // Build negation automaton if not supplied.
     if (!aut_neg_f)
       {
-        if (f)
+        if (is_deterministic(aut_f))
+          {
+            // If the automaton is deterministic, complementing is
+            // easy.
+            aut_neg_f = remove_fin(dualize(aut_f));
+          }
+        else if (f)
           {
             // If we know the formula, simply build the automaton for
             // its negation.
@@ -637,12 +696,6 @@ namespace spot
             // Remove useless SCCs.
             aut_neg_f = scc_filter(aut_neg_f, true);
           }
-        else if (is_deterministic(aut_f))
-          {
-            // If the automaton is deterministic, complementing is
-            // easy.
-            aut_neg_f = remove_fin(dtwa_complement(aut_f));
-          }
         else
           {
             // Otherwise, we cannot check if the minimization is safe.
@@ -655,21 +708,16 @@ namespace spot
     if (is_terminal_automaton(aut_neg_f))
       return min_aut_f;
 
-    bool ok = false;
-
+    // Make sure the minimized WDBA does not accept more words than
+    // the input.
     if (product(min_aut_f, aut_neg_f)->is_empty())
       {
-        // Complement the minimized WDBA.
         assert((bool)min_aut_f->prop_weak());
-        auto neg_min_aut_f = remove_fin(dtwa_complement(min_aut_f));
-        if (product(aut_f, neg_min_aut_f)->is_empty())
-          // Finally, we are now sure that it was safe
-          // to minimize the automaton.
-          ok = true;
+        return min_aut_f;
+      }
+    else
+      {
+        return std::const_pointer_cast<twa_graph>(aut_f);
       }
-
-    if (ok)
-      return min_aut_f;
-    return std::const_pointer_cast<twa_graph>(aut_f);
   }
 }
