@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche et
-// Développement de l'Epita (LRDE).
+// Copyright (C) 2012-2018 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -34,6 +34,11 @@
 #include <spot/twaalgos/sbacc.hh>
 #include <spot/twaalgos/sepsets.hh>
 #include <spot/twaalgos/determinize.hh>
+#include <spot/twaalgos/alternation.hh>
+#include <spot/twaalgos/parity.hh>
+#include <spot/twaalgos/cobuchi.hh>
+#include <spot/twaalgos/dot.hh>
+#include <spot/twaalgos/rabin2parity.hh>
 
 namespace spot
 {
@@ -61,6 +66,7 @@ namespace spot
         degen_cache_ = opt->get("degen-lcache", 1);
         degen_lskip_ = opt->get("degen-lskip", 1);
         degen_lowinit_ = opt->get("degen-lowinit", 0);
+        degen_remscc_ = opt->get("degen-remscc", 1);
         det_scc_ = opt->get("det-scc", 1);
         det_simul_ = opt->get("det-simul", 1);
         det_stutter_ = opt->get("det-stutter", 1);
@@ -69,13 +75,15 @@ namespace spot
         ba_simul_ = opt->get("ba-simul", -1);
         tba_determinisation_ = opt->get("tba-det", 0);
         sat_minimize_ = opt->get("sat-minimize", 0);
+        sat_incr_steps_ = opt->get("sat-incr-steps", -2); // -2 or any num < -1
+        sat_langmap_ = opt->get("sat-langmap", 0);
         sat_acc_ = opt->get("sat-acc", 0);
         sat_states_ = opt->get("sat-states", 0);
         state_based_ = opt->get("state-based", 0);
         wdba_minimize_ = opt->get("wdba-minimize", 1);
 
         if (sat_acc_ && sat_minimize_ == 0)
-          sat_minimize_ = 1;        // 2?
+          sat_minimize_ = 1;        // Dicho.
         if (sat_states_ && sat_minimize_ == 0)
           sat_minimize_ = 1;
         if (sat_minimize_)
@@ -86,6 +94,12 @@ namespace spot
             if (sat_states_ <= 0)
               sat_states_ = -1;
           }
+
+        // Set default param value if not provided and used.
+        if (sat_minimize_ == 2 && sat_incr_steps_ < 0) // Assume algorithm.
+            sat_incr_steps_ = 6;
+        else if (sat_minimize_ == 3 && sat_incr_steps_ < -1) // Incr algorithm.
+            sat_incr_steps_ = 2;
       }
   }
 
@@ -133,17 +147,26 @@ namespace spot
     auto d = degeneralize(a,
                           degen_reset_, degen_order_,
                           degen_cache_, degen_lskip_,
-                          degen_lowinit_);
+                          degen_lowinit_, degen_remscc_);
     return do_sba_simul(d, ba_simul_);
   }
 
+  twa_graph_ptr
+  postprocessor::do_degen_tba(const twa_graph_ptr& a)
+  {
+    return degeneralize_tba(a,
+                            degen_reset_, degen_order_,
+                            degen_cache_, degen_lskip_,
+                            degen_lowinit_, degen_remscc_);
+  }
+
   twa_graph_ptr
   postprocessor::do_scc_filter(const twa_graph_ptr& a, bool arg)
   {
     if (scc_filter_ == 0)
       return a;
     // If the automaton is weak, using transition-based acceptance
-    // won't help, so let's preserve it.
+    // won't help, so let's preserve state-based acceptance.
     if ((state_based_ || a->prop_inherently_weak().is_true())
         && a->prop_state_acc().is_true())
       return scc_filter_states(a, arg);
@@ -160,6 +183,7 @@ namespace spot
 #define PREF_ (pref_ & (Small | Deterministic))
 #define COMP_ (pref_ & Complete)
 #define SBACC_ (pref_ & SBAcc)
+#define COLORED_ (pref_ & Colored)
 
   twa_graph_ptr
   postprocessor::run(twa_graph_ptr a, formula f)
@@ -173,9 +197,61 @@ namespace spot
     if (type_ == BA || SBACC_)
       state_based_ = true;
 
-    if (type_ != Generic && !a->acc().is_generalized_buchi())
+    bool via_gba = (type_ == BA) || (type_ == TGBA) || (type_ == Monitor);
+    bool want_parity = (type_ & Parity) == Parity;
+    if (COLORED_ && !want_parity)
+      throw std::runtime_error("postprocessor: the Colored setting only works "
+                               "for parity acceptance");
+
+    auto finalize = [&](twa_graph_ptr tmp)
       {
-        a = to_generalized_buchi(a);
+        if (COMP_)
+          tmp = complete(tmp);
+        if (want_parity && tmp->acc().is_generalized_buchi())
+          tmp = SBACC_ ? do_degen(tmp) : do_degen_tba(tmp);
+        if (SBACC_)
+          tmp = sbacc(tmp);
+        if (want_parity)
+          {
+            if (COLORED_)
+              colorize_parity_here(tmp);
+            parity_kind kind = parity_kind_any;
+            parity_style style = parity_style_any;
+            if ((type_ & ParityMin) == ParityMin)
+              kind = parity_kind_min;
+            else if ((type_ & ParityMax) == ParityMax)
+              kind = parity_kind_max;
+            if ((type_ & ParityOdd) == ParityOdd)
+              style = parity_style_odd;
+            else if ((type_ & ParityEven) == ParityEven)
+              style = parity_style_even;
+            change_parity_here(tmp, kind, style);
+          }
+        return tmp;
+      };
+
+    if (!a->is_existential() &&
+        // We will probably have to revisit this condition later.
+        // Currently, the intent is that postprocessor should never
+        // return an alternating automaton, unless it is called with
+        // its laxest settings.
+        !(type_ == Generic && PREF_ == Any && level_ == Low))
+      a = remove_alternation(a);
+
+    if ((via_gba && !a->acc().is_generalized_buchi())
+        || (want_parity && !a->acc().is_parity()))
+      {
+        twa_graph_ptr b = nullptr;
+        if (want_parity && is_deterministic(a) &&
+            !a->acc().is_generalized_buchi())
+          b = iar_maybe(a);
+        // possible only if a was deterministic and (Rabin-like or Streett-like)
+        // and we want parity and a is not a TGBA
+        // NB: on a TGBA, degeneralization is better than IAR
+        if (b)
+          a = b;
+        else
+          a = to_generalized_buchi(a);
         if (PREF_ == Any && level_ == Low)
           a = do_scc_filter(a, true);
       }
@@ -184,14 +260,10 @@ namespace spot
         && (type_ == Generic
             || type_ == TGBA
             || (type_ == BA && a->is_sba())
-            || (type_ == Monitor && a->num_sets() == 0)))
-      {
-        if (COMP_)
-          a = complete(a);
-        if (SBACC_)
-          a = sbacc(a);
-        return a;
-      }
+            || (type_ == Monitor && a->num_sets() == 0)
+            || (want_parity && a->acc().is_parity())
+            || (type_ == CoBuchi && a->acc().is_co_buchi())))
+      return finalize(a);
 
     int original_acc = a->num_sets();
 
@@ -210,34 +282,32 @@ namespace spot
         else
           strip_acceptance_here(a);
 
-        if (PREF_ == Any)
-          return a;
-
-        a = do_simul(a, simul_);
-
-        // For Small,High we return the smallest between the output of
-        // the simulation, and that of the deterministic minimization.
-        if (PREF_ == Small && level_ == High && simul_)
+        if (PREF_ != Any)
           {
-            auto m = minimize_monitor(a);
-            if (m->num_states() < a->num_states())
-              a = m;
+            if (PREF_ != Deterministic)
+              a = do_simul(a, simul_);
+
+            // For Small,High we return the smallest between the output of
+            // the simulation, and that of the deterministic minimization.
+            // Prefer the deterministic automaton in case of equality.
+            if (PREF_ == Small && level_ == High && simul_)
+              {
+                auto m = minimize_monitor(a);
+                if (m->num_states() <= a->num_states())
+                  a = m;
+              }
+            a->remove_unused_ap();
           }
-        a->remove_unused_ap();
-        if (COMP_)
-          a = complete(a);
-        return a;
+        return finalize(a);
       }
 
     if (PREF_ == Any)
       {
         if (type_ == BA)
           a = do_degen(a);
-        if (COMP_)
-          a = complete(a);
-        if (SBACC_)
-          a = sbacc(a);
-        return a;
+        else if (type_ == CoBuchi)
+          a = to_nca(a);
+        return finalize(a);
       }
 
     bool dba_is_wdba = false;
@@ -253,7 +323,7 @@ namespace spot
         dba = minimize_obligation(a, f, nullptr, reject_bigger);
         if (dba
             && dba->prop_inherently_weak().is_true()
-            && dba->prop_deterministic().is_true())
+            && dba->prop_universal().is_true())
           {
             // The WDBA is a BA, so no degeneralization is required.
             // We just need to add an acceptance set if there is none.
@@ -285,6 +355,8 @@ namespace spot
             // No need to do that if tba_determinisation_ will be used.
             if (type_ == BA && !tba_determinisation_)
               sim = do_degen(sim);
+            else if (want_parity && !sim->acc().is_parity())
+              sim = do_degen_tba(sim);
             else if (SBACC_ && !tba_determinisation_)
               sim = sbacc(sim);
           }
@@ -362,7 +434,7 @@ namespace spot
           }
       }
 
-    if (PREF_ == Deterministic && type_ == Generic && !dba)
+    if ((PREF_ == Deterministic && (type_ == Generic || want_parity)) && !dba)
       {
         dba = tgba_determinize(to_generalized_buchi(sim),
                                false, det_scc_, det_simul_, det_stutter_);
@@ -416,10 +488,17 @@ namespace spot
           {
             if (sat_states_ != -1)
               res = dtba_sat_synthetize(res, sat_states_, state_based_);
-            else if (sat_minimize_ == 1 || sat_minimize_ == -1)
+            else if (sat_minimize_ == 1)
+              res = dtba_sat_minimize_dichotomy
+                (res, state_based_, sat_langmap_);
+            else if (sat_minimize_ == 2)
+              res = dtba_sat_minimize_assume(res, state_based_, -1,
+                                             sat_incr_steps_);
+            else if (sat_minimize_ == 3)
+              res = dtba_sat_minimize_incr(res, state_based_, -1,
+                                           sat_incr_steps_);
+            else // if (sat_minimize == 4 || sat_minimize == -1)
               res = dtba_sat_minimize(res, state_based_);
-            else  // sat_minimize_ == 2
-              res = dtba_sat_minimize_dichotomy(res, state_based_);
           }
         else
           {
@@ -428,13 +507,23 @@ namespace spot
                 (res, target_acc,
                  acc_cond::acc_code::generalized_buchi(target_acc),
                  sat_states_, state_based_);
-            else if (sat_minimize_ == 1 || sat_minimize_ == -1)
-              res = dtwa_sat_minimize
+            else if (sat_minimize_ == 1)
+              res = dtwa_sat_minimize_dichotomy
                 (res, target_acc,
                  acc_cond::acc_code::generalized_buchi(target_acc),
-                 state_based_);
-            else  // sat_minimize_ == 2
-              res = dtwa_sat_minimize_dichotomy
+                 state_based_, sat_langmap_);
+            else if (sat_minimize_ == 2)
+              res = dtwa_sat_minimize_assume
+                (res, target_acc,
+                 acc_cond::acc_code::generalized_buchi(target_acc),
+                 state_based_, -1, false, sat_incr_steps_);
+            else if (sat_minimize_ == 3)
+              res = dtwa_sat_minimize_incr
+                (res, target_acc,
+                 acc_cond::acc_code::generalized_buchi(target_acc),
+                 state_based_, -1, false, sat_incr_steps_);
+            else // if (sat_minimize_ == 4 || sat_minimize_ == -1)
+              res = dtwa_sat_minimize
                 (res, target_acc,
                  acc_cond::acc_code::generalized_buchi(target_acc),
                  state_based_);
@@ -481,11 +570,20 @@ namespace spot
 
     sim->remove_unused_ap();
 
-    if (COMP_)
-      sim = complete(sim);
-    if (SBACC_)
-      sim = sbacc(sim);
+    if (type_ == CoBuchi)
+      {
+        unsigned ns = sim->num_states();
+        if (PREF_ == Deterministic)
+          sim = to_dca(sim);
+        else
+          sim = to_nca(sim);
 
-    return sim;
+        // if the input of to_dca/to_nca was weak, the number of
+        // states has not changed, and running simulation is useless.
+        if (level_ != Low && ns < sim->num_states())
+          sim = do_simul(sim, simul_);
+      }
+
+    return finalize(sim);
   }
 }
