@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013, 2014, 2015, 2016 Laboratoire de Recherche
-// et Développement de l'Epita (LRDE).
+// Copyright (C) 2011, 2013-2017  Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
 // et Marie Curie.
@@ -59,7 +59,10 @@ namespace spot
         : emptiness_check(a, o),
           h(size)
       {
-        assert(a->num_sets() <= 1);
+        if (!(a->prop_weak().is_true()
+              || a->num_sets() == 0
+              || a->acc().is_buchi()))
+          throw std::runtime_error("se05 requires Büchi or weak automata");
       }
 
       virtual ~se05_search()
@@ -356,7 +359,7 @@ namespace spot
               if (l == &run->prefix && i->s->compare(target) == 0)
                 l = &run->cycle;
               twa_run::step s = { i->s->clone(), j->label, j->acc };
-              l->push_back(s);
+              l->emplace_back(s);
             }
 
           if (l == &run->prefix && i->s->compare(target) == 0)
@@ -365,14 +368,14 @@ namespace spot
 
           j = ms_->st_red.rbegin();
           twa_run::step s = { i->s->clone(), j->label, j->acc };
-          l->push_back(s);
+          l->emplace_back(s);
 
           i = j; ++j;
           end = ms_->st_red.rend(); --end;
           for (; i != end; ++i, ++j)
             {
               twa_run::step s = { i->s->clone(), j->label, j->acc };
-              l->push_back(s);
+              l->emplace_back(s);
             }
 
           return run;
