@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2012, 2013, 2014, 2015, 2016 Laboratoire de Recherche
-// et Développement de l'Epita (LRDE).
+// Copyright (C) 2012-2017 Laboratoire de Recherche et Développement
+// de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
 //
@@ -27,60 +27,30 @@
 #include <spot/twa/bddprint.hh>
 #include <spot/twaalgos/reachiter.hh>
 #include <spot/twaalgos/sccfilter.hh>
-#include <spot/twaalgos/sccinfo.hh>
 #include <spot/twaalgos/sepsets.hh>
+#include <spot/twaalgos/isdet.hh>
 #include <spot/misc/bddlt.hh>
 
-// The way we developed this algorithm is the following: We take an
-// automaton, and reverse all these acceptance conditions.  We reverse
-// them to go make the meaning of the signature easier. We are using
-// bdd, and we want to let it make all the simplification. Because of
-// the format of the acceptance condition, it doesn't allow easy
-// simplification. Instead of encoding them as: "a!b!c + !ab!c", we
-// use them as: "ab". We complement them because we want a
-// simplification if the condition of the edge A implies the
-// edge of B, and if the acceptance condition of A is included
-// in the acceptance condition of B. To let the bdd makes the job, we
-// revert them.
-
-// Then, to check if a edge i-dominates another, we'll use the bdd:
-// "sig(transA) = cond(trans) & acc(trans) & implied(class(trans->state))"
-// Idem for sig(transB). The 'implied'
-// (represented by a hash table 'relation_' in the implementation) is
-// a conjunction of all the class dominated by the class of the
-// destination. This is how the relation is included in the
-// signature. It makes the simplifications alone, and the work is
-// done.  The algorithm is cut into several step:
+//  Simulation-based reduction, implemented using bdd-based signatures.
 //
-// 1. Run through the tgba and switch the acceptance condition to their
-//    negation, and initializing relation_ by the 'init_ -> init_' where
-//    init_ is the bdd which represents the class. This function is the
-//    constructor of Simulation.
-// 2. Enter in the loop (run).
-//    - Rename the class.
-//    - run through the automaton and computing the signature of each
-//      state. This function is `update_sig'.
-//    - Enter in a double loop to adapt the partial order, and set
-//      'relation_' accordingly. This function is `update_po'.
-// 3. Rename the class (to actualize the name in the previous_class and
-//    in relation_).
-// 4. Building an automaton with the result, with the condition:
-// "a edge in the original automaton appears in the simulated one
-// iff this edge is included in the set of i-maximal neighbour."
-// This function is `build_output'.
-// The automaton simulated is recomplemented to come back to its initial
-// state when the object Simulation is destroyed.
+//  The signature of a state is a Boolean function (implemented as a
+//  BDD) representing the set of outgoing transitions of that state.
+//  Two states with the same signature are equivalent and can be
+//  merged.
 //
-// Obviously these functions are possibly cut into several little ones.
-// This is just the general development idea.
-
+// We define signatures in a way that implications between signatures
+//  entail language inclusion.  These inclusions are used to remove
+//  redundant transitions, but this occurs implicitly while
+//  transforming the signature into irredundant-sum-of-product before
+//  building the automaton: redundant terms that are left over
+//  correspond to ignored transitions.
+//
+//  See our Spin'13 paper for background on this procedure.
 
 namespace spot
 {
   namespace
   {
-    // Some useful typedef:
-
     // Used to get the signature of the state.
     typedef std::vector<bdd> vector_state_bdd;
 
@@ -88,12 +58,9 @@ namespace spot
     typedef std::map<bdd, std::list<unsigned>,
                      bdd_less_than> map_bdd_lstate;
 
-    typedef std::map<bdd, unsigned, bdd_less_than> map_bdd_state;
-
-
     // This class helps to compare two automata in term of
     // size.
-    struct automaton_size
+    struct automaton_size final
     {
       automaton_size()
         : edges(0),
@@ -158,7 +125,6 @@ namespace spot
     class direct_simulation final
     {
     protected:
-      // Shortcut used in update_po and go_to_next_it.
       typedef std::map<bdd, bdd, bdd_less_than> map_bdd_bdd;
       int acc_vars;
       acc_cond::mark_t all_inf_;
@@ -179,7 +145,7 @@ namespace spot
         std::vector<unsigned> res;
         while (b != bddtrue)
           {
-            res.push_back(bdd_var(b) - acc_vars);
+            res.emplace_back(bdd_var(b) - acc_vars);
             b = bdd_high(b);
           }
         return acc_cond::mark_t(res.begin(), res.end());
@@ -193,15 +159,13 @@ namespace spot
         if (!has_separate_sets(in))
           throw std::runtime_error
             ("direct_simulation() requires separate Inf and Fin sets");
-
-        // Call get_init_state_number() before anything else as it
-        // might add a state.
-        unsigned init_state_number = in->get_init_state_number();
-        scc_info_.reset(new scc_info(in));
+        if (!in->is_existential())
+          throw std::runtime_error
+            ("direct_simulation() does not yet support alternation");
 
         unsigned ns = in->num_states();
-        assert(ns > 0);
         size_a_ = ns;
+        unsigned init_state_number = in->get_init_state_number();
 
         auto all_inf = in->get_acceptance().used_inf_fin_sets().first;
         all_inf_ = all_inf;
@@ -252,6 +216,8 @@ namespace spot
           }
         assert(a_->num_states() == size_a_);
 
+        want_implications_ = !is_deterministic(a_);
+
         // Now, we have to get the bdd which will represent the
         // class. We register one bdd by state, because in the worst
         // case, |Class| == |State|.
@@ -269,7 +235,7 @@ namespace spot
         bdd_initial = bdd_ithvar(set_num++);
         bdd init = bdd_ithvar(set_num++);
 
-        used_var_.push_back(init);
+        used_var_.emplace_back(init);
 
         // Initialize all classes to init.
         previous_class_.resize(size_a_);
@@ -306,7 +272,7 @@ namespace spot
 
         // We run through the map bdd/list<state>, and we update
         // the previous_class_ with the new data.
-        for (auto& p: bdd_lstate_)
+        for (auto& p: sorted_classes_)
           {
             // If the signature of a state is bddfalse (no
             // edges) the class of this state is bddfalse
@@ -314,11 +280,11 @@ namespace spot
             // simplifications in the signature by removing a
             // edge which has as a destination a state with
             // no outgoing edge.
-            if (p.first == bddfalse)
-              for (auto s: p.second)
+            if (p->first == bddfalse)
+              for (unsigned s: p->second)
                 previous_class_[s] = bddfalse;
             else
-              for (auto s: p.second)
+              for (unsigned s: p->second)
                 previous_class_[s] = *it_bdd;
             ++it_bdd;
           }
@@ -333,18 +299,18 @@ namespace spot
           {
             update_previous_class();
             nb_partition_before = bdd_lstate_.size();
-            bdd_lstate_.clear();
             nb_po_before = po_size_;
             po_size_ = 0;
             update_sig();
             go_to_next_it();
+            // print_partition();
           }
 
         update_previous_class();
       }
 
       // The core loop of the algorithm.
-      twa_graph_ptr run(std::map<int, bdd>* implications = nullptr)
+      twa_graph_ptr run(std::vector<bdd>* implications = nullptr)
       {
         main_loop();
         return build_result(implications);
@@ -378,12 +344,22 @@ namespace spot
 
       void update_sig()
       {
+        bdd_lstate_.clear();
+        sorted_classes_.clear();
         for (unsigned s = 0; s < size_a_; ++s)
-          bdd_lstate_[compute_sig(s)].push_back(s);
+          {
+            bdd sig = compute_sig(s);
+            auto p = bdd_lstate_.emplace(std::piecewise_construct,
+                                         std::make_tuple(sig),
+                                         std::make_tuple());
+            p.first->second.emplace_back(s);
+            if (p.second)
+              sorted_classes_.emplace_back(p.first);
+          }
       }
 
 
-      // This method rename the color set, update the partial order.
+      // This method renames the color set, updates the partial order.
       void go_to_next_it()
       {
         int nb_new_color = bdd_lstate_.size() - used_var_.size();
@@ -394,7 +370,7 @@ namespace spot
         for (int i = 0; i < nb_new_color; ++i)
           {
             assert(!free_var_.empty());
-            used_var_.push_back(bdd_ithvar(free_var_.front()));
+            used_var_.emplace_back(bdd_ithvar(free_var_.front()));
             free_var_.pop();
           }
 
@@ -413,44 +389,30 @@ namespace spot
                || (bdd_lstate_.find(bddfalse) != bdd_lstate_.end()
                    && bdd_lstate_.size() == used_var_.size() + 1));
 
-        // Now we make a temporary hash_table which links the tuple
-        // "C^(i-1), N^(i-1)" to the new class coloring.  If we
-        // rename the class before updating the partial order, we
-        // loose the information, and if we make it after, I can't
-        // figure out how to apply this renaming on rel_.
-        // It adds a data structure but it solves our problem.
-        map_bdd_bdd now_to_next;
+        // This vector links the tuple "C^(i-1), N^(i-1)" to the
+        // new class coloring for the next iteration.
+        std::vector<std::pair<bdd, bdd>> now_to_next;
+        unsigned sz = bdd_lstate_.size();
+        now_to_next.reserve(sz);
 
         std::list<bdd>::iterator it_bdd = used_var_.begin();
 
-        for (auto& p: bdd_lstate_)
+        for (auto& p: sorted_classes_)
           {
-            // If the signature of a state is bddfalse (no
-            // edges) the class of this state is bddfalse
-            // instead of an anonymous variable. It allows
-            // simplifications in the signature by removing a
-            // edge which has as a destination a state with
-            // no outgoing edge.
+            // If the signature of a state is bddfalse (no edges) the
+            // class of this state is bddfalse instead of an anonymous
+            // variable. It allows simplifications in the signature by
+            // removing an edge which has as a destination a state
+            // with no outgoing edge.
             bdd acc = bddfalse;
-            if (p.first != bddfalse)
+            if (p->first != bddfalse)
               acc = *it_bdd;
-            now_to_next[p.first] = acc;
+            now_to_next.emplace_back(p->first, acc);
             ++it_bdd;
           }
 
-        update_po(now_to_next, relation_);
-      }
+        // Update the partial order.
 
-      // This function computes the new po with previous_class_ and
-      // the argument. `now_to_next' contains the relation between the
-      // signature and the future name of the class.  We need a
-      // template parameter because we use this function with a
-      // map_bdd_bdd, but later, we need a list_bdd_bdd. So to
-      // factorize some code we use a template.
-      template <typename container_bdd_bdd>
-      void update_po(const container_bdd_bdd& now_to_next,
-                     map_bdd_bdd& relation)
-      {
         // This loop follows the pattern given by the paper.
         // foreach class do
         // |  foreach class do
@@ -458,51 +420,54 @@ namespace spot
         // |  od
         // od
 
-        for (typename container_bdd_bdd::const_iterator it1
-               = now_to_next.begin();
-             it1 != now_to_next.end();
-             ++it1)
+        for (unsigned n = 0; n < sz; ++n)
           {
-            bdd accu = it1->second;
-            for (typename container_bdd_bdd::const_iterator it2
-                   = now_to_next.begin();
-                 it2 != now_to_next.end();
-                 ++it2)
-              {
-                // Skip the case managed by the initialization of accu.
-                if (it1 == it2)
-                  continue;
-
-                if (bdd_implies(it1->first, it2->first))
-                  {
-                    accu &= it2->second;
-                    ++po_size_;
-                  }
-              }
-            relation[it1->second] = accu;
+            bdd n_sig = now_to_next[n].first;
+            bdd n_class = now_to_next[n].second;
+            if (want_implications_)
+              for (unsigned m = 0; m < sz; ++m)
+                {
+                  if (n == m)
+                    continue;
+                  if (bdd_implies(n_sig, now_to_next[m].first))
+                    {
+                      n_class &= now_to_next[m].second;
+                      ++po_size_;
+                    }
+                }
+            relation_[now_to_next[n].second] = n_class;
           }
       }
 
       // Build the minimal resulting automaton.
-      twa_graph_ptr build_result(std::map<int, bdd>* implications = nullptr)
+      twa_graph_ptr build_result(std::vector<bdd>* implications = nullptr)
       {
         twa_graph_ptr res = make_twa_graph(a_->get_dict());
         res->copy_ap_of(a_);
         res->copy_acceptance_of(a_);
 
+        auto state_mapping = new std::vector<unsigned>();
+        state_mapping->resize(a_->num_states());
+        res->set_named_prop("simulated-states", state_mapping);
+
         // Non atomic propositions variables (= acc and class)
         bdd nonapvars = all_proms_ & bdd_support(all_class_var_);
 
         auto* gb = res->create_namer<int>();
 
-        // Create one state per partition.
-        for (auto& p: bdd_lstate_)
+        if (implications)
+          implications->resize(bdd_lstate_.size());
+        // Create one state per class.
+        for (auto& p: sorted_classes_)
           {
-            bdd cl = previous_class_[p.second.front()];
+            bdd cl = previous_class_[p->second.front()];
             // A state may be referred to either by
             // its class, or by all the implied classes.
             auto s = gb->new_state(cl.id());
             gb->alias_state(s, relation_[cl].id());
+            // update state_mapping
+            for (auto& st : p->second)
+              (*state_mapping)[st] = s;
             if (implications)
               (*implications)[s] = relation_[cl];
           }
@@ -521,14 +486,14 @@ namespace spot
         auto all_inf = all_inf_;
         // For each class, we will create
         // all the edges between the states.
-        for (auto& p: bdd_lstate_)
+        for (auto& p: sorted_classes_)
           {
             // All states in p.second have the same class, so just
             // pick the class of the first one first one.
-            bdd src = previous_class_[p.second.front()];
+            bdd src = previous_class_[p->second.front()];
 
             // Get the signature to derive successors.
-            bdd sig = compute_sig(p.second.front());
+            bdd sig = compute_sig(p->second.front());
 
             if (Cosimulation)
               sig = bdd_compose(sig, bddfalse, bdd_var(bdd_initial));
@@ -614,7 +579,9 @@ namespace spot
         res->set_init_state(gb->get_state(previous_class_
                                           [a_->get_init_state_number()].id()));
 
-        res->merge_edges(); // FIXME: is this really needed?
+        res->merge_edges(); // This helps merging some edges with
+                            // identical conditions, but different
+                            // mark sets.
 
         // Mark all accepting state in a second pass, when
         // dealing with SBA in cosimulation.
@@ -631,18 +598,27 @@ namespace spot
               }
           }
 
-        res->purge_unreachable_states();
+        // If we recorded implications for the determinization
+        // procedure, we should not remove unreachable states, as that
+        // will invalidate the contents of the IMPLICATIONS vector.
+        // It's OK not to purge the result, as the determinization
+        // will only explore the reachable part anyway.
+        if (!implications)
+          res->purge_unreachable_states();
 
         delete gb;
         res->prop_copy(original_,
                        { false, // state-based acc forced below
                          true,  // weakness preserved,
-                         true, // determinism checked and overridden below
-                               // and "unambiguous" property preserved
+                         false, true, // determinism improved
+                         true, // completeness preserved
                          true, // stutter inv.
                        });
-        if (nb_minato == nb_satoneset && !Cosimulation)
-          res->prop_deterministic(true);
+        // !unambiguous and !semi-deterministic are not preserved
+        if (!Cosimulation && nb_minato == nb_satoneset)
+          // Note that nb_minato != nb_satoneset does not imply
+          // non-deterministic, because of the merge_edges() above.
+          res->prop_universal(true);
         if (Sba)
           res->prop_state_acc(true);
         return res;
@@ -656,47 +632,53 @@ namespace spot
       // where is the new class name.
       void print_partition()
       {
-        for (auto& p: bdd_lstate_)
-          {
-            std::cerr << "partition: "
-                      << bdd_format_isop(a_->get_dict(), p.first)
-                      << std::endl;
-            for (auto s: p.second)
-              std::cerr << "  - "
-                        << a_->format_state(a_->state_from_number(s))
-                        << '\n';
-          }
-
-        std::cerr << "\nPrevious iteration\n" << std::endl;
-
+        std::cerr << "\n-----\n\nClasses from previous iteration:\n";
         unsigned ps = previous_class_.size();
         for (unsigned p = 0; p < ps; ++p)
           {
-            std::cerr << a_->format_state(a_->state_from_number(p))
-                      << " was in "
+            std::cerr << "- " << a_->format_state(a_->state_from_number(p))
+                      << " was in class "
                       << bdd_format_set(a_->get_dict(), previous_class_[p])
                       << '\n';
           }
+        std::cerr << "\nPartition:\n";
+        std::list<bdd>::iterator it_bdd = used_var_.begin();
+        for (auto& p: sorted_classes_)
+          {
+            std::cerr << "- ";
+            if (p->first != bddfalse)
+              std::cerr << "new class " << *it_bdd++ << " from ";
+            std::cerr << "sig "
+                      << bdd_format_isop(a_->get_dict(), p->first);
+            std::cerr << '\n';
+            for (auto s: p->second)
+              std::cerr << "    - "
+                        << a_->format_state(a_->state_from_number(s))
+                        << '\n';
+          }
+        std::cerr << "\nClass implications:\n";
+        for (auto& p: relation_)
+          std::cerr << "  - " << p.first << "  =>  " << p.second << '\n';
+
       }
 
     protected:
       // The automaton which is simulated.
       twa_graph_ptr a_;
 
-      // Relation is aimed to represent the same thing than
-      // rel_. The difference is in the way it does.
-      // If A => A /\ A => B, rel will be (!A U B), but relation_
-      // will have A /\ B at the key A. This trick is due to a problem
-      // with the computation of the resulting automaton with the signature.
-      // rel_ will pollute the meaning of the signature.
+      // Implications between classes.
       map_bdd_bdd relation_;
 
       // Represent the class of each state at the previous iteration.
       vector_state_bdd previous_class_;
 
-      // The list of state for each class at the current_iteration.
+      // The list of states for each class at the current_iteration.
       // Computed in `update_sig'.
       map_bdd_lstate bdd_lstate_;
+      // The above map, sorted by states number instead of BDD
+      // identifier to avoid non-determinism while iterating over all
+      // states.
+      std::vector<map_bdd_lstate::const_iterator> sorted_classes_;
 
       // The queue of free bdd. They will be used as the identifier
       // for the class.
@@ -708,10 +690,13 @@ namespace spot
       // Size of the automaton.
       unsigned int size_a_;
 
-      // Used to know when there is no evolution in the po. Updated
-      // in the `update_po' method.
+      // Used to know when there is no evolution in the partial order.
       unsigned int po_size_;
 
+      // Whether to compute implications between classes.  This is costly
+      // and useless for deterministic automata.
+      bool want_implications_;
+
       // All the class variable:
       bdd all_class_var_;
 
@@ -722,8 +707,6 @@ namespace spot
 
       automaton_size stat;
 
-      std::unique_ptr<scc_info> scc_info_;
-
       const const_twa_graph_ptr original_;
     };
 
@@ -739,7 +722,7 @@ namespace spot
 
   twa_graph_ptr
   simulation(const const_twa_graph_ptr& t,
-             std::map<int, bdd>* implications)
+             std::vector<bdd>* implications)
   {
     direct_simulation<false, false> simul(t);
     return simul.run(implications);
@@ -780,7 +763,7 @@ namespace spot
         prev = next;
         direct_simulation<false, Sba> simul(res ? res : t);
         res = simul.run();
-        if (res->prop_deterministic())
+        if (res->prop_universal())
           break;
 
         direct_simulation<true, Sba> cosimul(res);
