@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2011, 2013, 2014, 2015, 2016 Laboratoire de Recherche
-// et Developpement de l'Epita (LRDE).
+// Copyright (C) 2011, 2013-2017  Laboratoire de Recherche et
+// Developpement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005  Laboratoire d'Informatique de Paris 6 (LIP6),
 // département Systèmes Répartis Coopératifs (SRC), Université Pierre
 // et Marie Curie.
@@ -58,7 +58,9 @@ namespace spot
         : emptiness_check(a, o),
           h(size)
       {
-        assert(a->num_sets() > 0);
+        if (!(a->num_sets() > 0 && a->acc().is_generalized_buchi()))
+          throw std::runtime_error
+            ("tau03 requires generalized Büchi with at least one set");
       }
 
       virtual ~tau03_search()
