@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2007-2016 Laboratoire de Recherche et Développement
+// Copyright (C) 2007-2018 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 // Copyright (C) 2003-2007 Laboratoire d'Informatique de Paris 6
 // (LIP6), département Systèmes Répartis Coopératifs (SRC), Université
@@ -39,7 +39,6 @@
 #include <spot/twaalgos/degen.hh>
 #include <spot/twa/twaproduct.hh>
 #include <spot/parseaut/public.hh>
-#include <spot/twaalgos/copy.hh>
 #include <spot/twaalgos/minimize.hh>
 #include <spot/taalgos/minimize.hh>
 #include <spot/twaalgos/neverclaim.hh>
@@ -57,7 +56,7 @@
 #include <spot/twaalgos/simulation.hh>
 #include <spot/twaalgos/compsusp.hh>
 #include <spot/twaalgos/powerset.hh>
-#include <spot/twaalgos/complement.hh>
+#include <spot/twaalgos/dualize.hh>
 #include <spot/twaalgos/remfin.hh>
 #include <spot/twaalgos/complete.hh>
 #include <spot/twaalgos/dtbasat.hh>
@@ -1146,7 +1145,7 @@ checked_main(int argc, char** argv)
       if (opt_dtwacomp)
         {
           tm.start("DTωA complement");
-          a = remove_fin(dtwa_complement(ensure_digraph(a)));
+          a = remove_fin(dualize(ensure_digraph(a)));
           tm.stop("DTωA complement");
         }
 
@@ -1156,8 +1155,7 @@ checked_main(int argc, char** argv)
           if (scc_filter && (reduction_dir_sim || reduction_rev_sim))
             {
               tm.start("SCC-filter post-sim");
-              auto aa = std::dynamic_pointer_cast<const spot::twa_graph>(a);
-              assert(aa);
+              auto aa = spot::down_cast<spot::const_twa_graph_ptr>(a);
               // Do not filter_all for SBA
               a = spot::scc_filter(aa, assume_sba ?
                                    false : scc_filter_all);
@@ -1186,7 +1184,7 @@ checked_main(int argc, char** argv)
         }
 
       if (dupexp)
-        a = copy(a, spot::twa::prop_set::all());
+        a = make_twa_graph(a, spot::twa::prop_set::all());
 
       //TA, STA, GTA, SGTA and TGTA
       if (ta_opt || tgta_opt)
@@ -1373,8 +1371,9 @@ checked_main(int argc, char** argv)
                 }
               else
                 {
-                  bool g = is_terminal_automaton(ensure_digraph(a));
-                  bool s = is_safety_mwdba(ensure_digraph(a));
+                  bool g = is_terminal_automaton(ensure_digraph(a),
+                                                 nullptr, true);
+                  bool s = is_safety_automaton(ensure_digraph(a));
                   if (g && !s)
                     {
                       std::cout << "this is a guarantee property (hence, "
@@ -1531,7 +1530,7 @@ checked_main(int argc, char** argv)
                               if (graph_run_tgba_opt)
                                 spot::print_dot(std::cout, run->as_twa());
                               else
-                                std::cout << run;
+                                std::cout << *run;
                               tm.stop("printing accepting run");
                             }
                         }
