@@ -1,6 +1,6 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008, 2009, 2010, 2011, 2012, 2014, 2015, 2016 Laboratoire
-// de Recherche et Développement de l'Epita (LRDE).
+// Copyright (C) 2008-2012, 2014-2017 Laboratoire de Recherche et
+// Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005 Laboratoire d'Informatique de Paris
 // 6 (LIP6), département Systèmes Répartis Coopératifs (SRC),
 // Université Pierre et Marie Curie.
@@ -65,6 +65,8 @@ const char* default_algos[] = {
   "Cou99(poprem)",
   "Cou99(poprem shy !group)",
   "Cou99(poprem shy group)",
+  "Cou99new",
+  "Cou99abs",
   "CVWY90",
   "CVWY90(bsh=4K)",
   "GV04",
@@ -143,8 +145,6 @@ syntax(char* prog)
             << "            otherwise be skipped (implies -e)" << std::endl
             << "  -e N    compare result of all "
             << "emptiness checks on N randomly generated graphs" << std::endl
-            << "  -H      halt on the first statistic difference in algorithms"
-            << std::endl
             << "  -m      try to reduce runs, in a second pass (implies -r)"
             << std::endl
             << "  -R N    repeat each emptiness-check and accepting run "
@@ -570,8 +570,6 @@ main(int argc, char** argv)
 
   int exit_code = 0;
 
-  bool stop_on_first_difference = false;
-
   spot::twa_graph_ptr formula = nullptr;
   spot::twa_graph_ptr product = nullptr;
 
@@ -675,12 +673,6 @@ main(int argc, char** argv)
         {
           opt_dot = true;
         }
-      else if (!strcmp(argv[argn], "-H"))
-        {
-          if (argc < argn + 1)
-            syntax(argv[0]);
-          stop_on_first_difference = true;
-        }
       else if (!strcmp(argv[argn], "-i"))
         {
           if (argc < argn + 2)
@@ -923,7 +915,6 @@ main(int argc, char** argv)
                   int n_empty = 0;
                   int n_non_empty = 0;
                   int n_maybe_empty = 0;
-                  spot::unsigned_statistics_copy ostats_ec, ostats_arc;
 
                   for (int i = 0; i < n_alg; ++i)
                     {
@@ -971,14 +962,6 @@ main(int argc, char** argv)
                             }
                         }
 
-                      if (stop_on_first_difference && ecs)
-                        if (!ostats_ec.seteq(*ecs))
-                          {
-                            std::cout << "DIFFERING STATS for emptiness check,"
-                                      << " halting... ";
-                            opt_ec = n_alg = opt_F = 0;
-                          }
-
                       if (res)
                         {
                           if (!opt_paper)
@@ -987,44 +970,35 @@ main(int argc, char** argv)
                           if (opt_replay)
                             {
                               spot::twa_run_ptr run;
-                              bool done = false;
                               tm_ar.start(algo);
                               for (int count = opt_R;;)
                                 {
                                   run = res->accepting_run();
-                                  const spot::unsigned_statistics* s
-                                    = res->statistics();
-                                  if (opt_z && !done)
-                                    {
-                                      // Count only the first run (the
-                                      // other way would be to divide
-                                      // the stats by opt_R).
-                                      done = true;
-                                      sc_arc.count(algo, s);
-                                      arc_ratio_stats.count(algo, s);
-                                    }
-                                  if (stop_on_first_difference && s)
-                                    if (!ostats_arc.seteq(*s))
-                                      {
-                                        std::cout << "DIFFERING STATS for "
-                                                  << "accepting runs,"
-                                                  << " halting... ";
-                                        opt_ec = n_alg = opt_F = 0;
-                                        break;
-                                      }
-
                                   if (count-- <= 0 || !run)
                                     break;
                                 }
+                              if (!run)
+                                tm_ar.cancel(algo);
+                              else
+                                tm_ar.stop(algo);
+
+                              const spot::unsigned_statistics* s
+                                = res->statistics();
+                              if (opt_z)
+                                {
+                                  // Count only the last run (the
+                                  // other way would be to divide
+                                  // the stats by opt_R).
+                                  sc_arc.count(algo, s);
+                                  arc_ratio_stats.count(algo, s);
+                                }
                               if (!run)
                                 {
-                                  tm_ar.cancel(algo);
                                   if (!opt_paper)
                                     std::cout << " exists, not computed";
                                 }
                               else
                                 {
-                                  tm_ar.stop(algo);
                                   std::ostringstream s;
                                   if (!run->replay(s))
                                     {
